@@ -26,6 +26,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <unistd.h>
 #include <string.h>
 #include <ctype.h>
 #include "gmime-part.h"
@@ -115,7 +116,7 @@ g_mime_part_destroy (GMimePart *mime_part)
 			
 			g_list_free (mime_part->disposition->params);
 		}
-
+		
 		g_free (mime_part->disposition);
 	}
 	
@@ -542,8 +543,9 @@ g_mime_part_get_content_disposition_parameter (GMimePart *mime_part, const gchar
 	GMimeParam *param;
 	
 	g_return_val_if_fail (mime_part != NULL, NULL);
-	g_return_val_if_fail (mime_part->disposition != NULL, NULL);
-	g_return_val_if_fail (mime_part->disposition->param_hash != NULL, NULL);
+	
+	if (!mime_part->disposition || !mime_part->disposition->param_hash)
+		return NULL;
 	
 	param = g_hash_table_lookup (mime_part->disposition->param_hash, name);
 	
@@ -599,21 +601,23 @@ g_mime_part_set_filename (GMimePart *mime_part, const gchar *filename)
  * and if not then checks the "name" parameter in the Content-Type.
  **/
 const gchar *
-g_mime_part_get_filename (const GMimePart * part)
+g_mime_part_get_filename (const GMimePart *mime_part)
 {
-	const gchar * retval = NULL;
-	GMimeParam * param = NULL;
-
-	g_return_val_if_fail (part!=NULL, NULL);
-
-	if (part->disposition!=NULL && part->disposition->param_hash!=NULL)
-		param = g_hash_table_lookup (part->disposition->param_hash, "filename");
-	if (param != NULL)
-		retval = (const gchar*) param->value;
-	if (retval == NULL)
-		retval = g_mime_content_type_get_parameter (part->mime_type, "name");
-
-	return retval;
+	GMimeParam *param;
+	
+	g_return_val_if_fail (mime_part != NULL, NULL);
+	
+	if (!mime_part->disposition || !mime_part->disposition->param_hash)
+		return NULL;
+	
+	param = g_hash_table_lookup (mime_part->disposition->param_hash, "filename");
+	
+	if (!param || !param->value) {
+		/* check the "name" param in the content-type */
+		return g_mime_content_type_get_parameter (mime_part->mime_type, "name");
+	}
+	
+	return param->value;
 }
 
 
@@ -733,7 +737,8 @@ g_mime_part_set_content_byte_array (GMimePart *mime_part, GByteArray *content)
  * Sets the encoding type and raw content on the mime part after decoding the content.
  **/
 void
-g_mime_part_set_pre_encoded_content (GMimePart *mime_part, const gchar *content, guint len, GMimePartEncodingType encoding)
+g_mime_part_set_pre_encoded_content (GMimePart *mime_part, const gchar *content,
+				     guint len, GMimePartEncodingType encoding)
 {
 	gchar *raw;
 	gint save = 0, state = 0;
@@ -780,7 +785,9 @@ const gchar *
 g_mime_part_get_content (const GMimePart *mime_part, guint *len)
 {
 	g_return_val_if_fail (mime_part != NULL, NULL);
-	g_return_val_if_fail (mime_part->content != NULL, NULL);
+	
+	if (!mime_part->content)
+		return NULL;
 	
 	*len = mime_part->content->len;
 	
@@ -816,7 +823,8 @@ get_content_disposition (GMimePart *mime_part)
 	GList *params;
 	gchar *str;
 	
-	g_return_val_if_fail (mime_part->disposition != NULL, NULL);
+	if (!mime_part->disposition)
+		return NULL;
 	
 	params = mime_part->disposition->params;
 	
@@ -1047,7 +1055,7 @@ g_mime_part_to_string (GMimePart *mime_part, gboolean toplevel)
 		
 		/* Content-Location: */
 		if (mime_part->content_location) {
-			content_md5 = g_strdup_printf ("Content-Location: %s\n", mime_part->content_location);
+			content_location = g_strdup_printf ("Content-Location: %s\n", mime_part->content_location);
 			g_string_append (string, content_location);
 			g_free (content_location);
 		}
