@@ -294,22 +294,34 @@ datetok (const gchar *date)
 	return tokens;
 }
 
+#if 0
 static gint
-get_days_in_month (gint mon, gint year)
+get_days_in_month (gint month, gint year)
 {
-	switch (mon) {
-	case 1: case 3: case 5: case 7: case 8: case 10: case 12:
-		return 31;
-	case 4: case 6: case 9: case 11:
-		return 30;
+        switch (month) {
+	case 1:
+	case 3:
+	case 5:
+	case 7:
+	case 8:
+	case 10:
+	case 12:
+	        return 31;
+	case 4:
+	case 6:
+	case 9:
+	case 11:
+	        return 30;
 	case 2:
-		if ((year % 4 == 0 && year % 100 != 0) || year % 400 == 0)
-			return 29;
-		return 28;
+	        if (g_date_is_leap_year (year))
+		        return 29;
+		else
+		        return 28;
 	default:
-		return 30;
+	        return 0;
 	}
 }
+#endif
 
 static gint
 get_wday (gchar *str)
@@ -360,10 +372,10 @@ get_month (gchar *str)
 }
 
 static gint
-get_year (gchar *str)
+get_year (const gchar *str)
 {
 	gint year;
-	gchar *p;
+	const gchar * p;
 	
 	for (p = str; *p; p++)
 		if (!isdigit (*p))
@@ -381,9 +393,9 @@ get_year (gchar *str)
 }
 
 static gboolean
-get_time (gchar *in, gint *hour, gint *min, gint *sec)
+get_time (const gchar *in, gint *hour, gint *min, gint *sec)
 {
-	gchar *p;
+	const gchar *p;
 	gint colons = 0;
 	gboolean digits = TRUE;
 	
@@ -394,10 +406,17 @@ get_time (gchar *in, gint *hour, gint *min, gint *sec)
 			digits = FALSE;
 	}
 	
-	if (!digits || colons != 2)
+/* Ameol software doesn't put the seconds on the time */
+
+	if (!digits || (colons != 2 && colons != 1))
 		return FALSE;
 	
-	return sscanf (in, "%d:%d:%d", hour, min, sec) == 3;
+	if (colons == 2)
+		return sscanf (in, "%d:%d:%d", hour, min, sec) == 3;
+	else {
+		*sec = 0;
+		return sscanf (in, "%d:%d", hour, min) == 2;
+	}
 }
 
 static gint
@@ -510,13 +529,14 @@ parse_rfc822_date (GList *tokens, int *tzone)
 static time_t
 parse_broken_date (GList *tokens, int *tzone)
 {
+#if 0
 	GList *token;
 	struct tm tm;
 	gint hour, min, sec, n;
 	
 	if (tzone)
 		*tzone = 0;
-	
+#endif	
 	return (time_t) 0;
 }
 
@@ -783,7 +803,7 @@ quoted_decode (const guchar *in, gint len, guchar *out)
 	while (inptr < inend) {
 		c = *inptr++;
 		if (c == '=') {
-			if (inend - in >= 2) {
+			if (inend - inptr >= 2) {
 				c = toupper (*inptr++);
 				c1 = toupper (*inptr++);
 				*outptr++ = (((c >= 'A' ? c - 'A' + 10 : c - '0') & 0x0f) << 4)
@@ -883,7 +903,7 @@ gchar *
 g_mime_utils_8bit_header_decode (const guchar *in)
 {
 	GString *out, *whtspc, *word;
-	guchar *inptr;
+	const guchar *inptr;
 	guchar *decoded;
 	gboolean last_was_encoded = FALSE;
 	gboolean last_was_space = FALSE;
@@ -891,7 +911,7 @@ g_mime_utils_8bit_header_decode (const guchar *in)
 	out = g_string_new ("");
 	whtspc = g_string_new ("");
 	word = g_string_new ("");
-	inptr = (guchar *) in;
+	inptr = in;
 	
 	while (inptr && *inptr) {
 		guchar c = *inptr++;
