@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #define __ACACHE_H__
 
 #include <glib.h>
+#include "pan-callback.h"
 
 void       acache_init              (void);
 
@@ -31,19 +32,24 @@ gint       acache_expire            (void);
 
 gint       acache_expire_all        (void);
 
-void       acache_file_checkin      (const gchar * message_id);
+void       acache_file_checkin      (const gchar   ** message_id,
+                                     gint             message_id_qty);
 
-void       acache_file_checkout     (const gchar * message_id);
+void       acache_file_checkout     (const gchar   ** message_ids,
+                                     gint             message_id_qty);
 
-gchar*     acache_get_filename      (const gchar * message_id);
+gboolean   acache_has_message       (const gchar    * message_id);
 
-void       acache_delete            (const gchar * message_id);
+gchar*     acache_get_message       (const gchar    * message_id);
 
-gboolean   acache_has_message       (const gchar * message_id);
+void       acache_set_message       (const gchar    * message_id,
+                                     const gchar    * message,
+                                     gboolean         checkout);
 
-gchar*     acache_get_message       (const gchar * message_id);
 
-void       acache_set_message       (const gchar * message_id,
-                                     const gchar * message);
+extern PanCallback* acache_bodies_added_callback;
+ 
+extern PanCallback* acache_bodies_removed_callback;
+
 
 #endif /* __ACACHE_H__ */
