@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 
 #include "article.h"
 #include "article-thread.h"
+#include "prefs.h"
 #include "util.h"
 
 /**
@@ -215,14 +216,9 @@ sort_articles (Article      ** buf,
 			ArticleStruct * as = g_new (ArticleStruct, article_qty);
 			for (i=0; i<article_qty; ++i)
 			{
-				const gchar * author = buf[i]->author_real;
-				if (!is_nonempty_string(author))
-					author = buf[i]->author_addr;
-				if (!is_nonempty_string(author))
-					author = _("?");
-				as[i].data = g_strdup(author);
-				g_strdown (as[i].data);
+				as[i].data = article_get_short_author_str (buf[i]);
 				as[i].article = buf[i];
+				g_strdown (as[i].data);
 			}
 			msort (as,
 			       article_qty,
@@ -250,6 +246,11 @@ sort_articles (Article      ** buf,
 			msort (buf, article_qty, sizeof(Article*), compare_ppA_to_ppA_by_unread_children);
 			break;
 		}
+		case ARTICLE_SORT_MSG_ID:
+		{
+			msort (buf, article_qty, sizeof(Article*), compare_ppA_to_ppA_by_message_id);
+			break;
+		}
 		case ARTICLE_SORT_SUBJECT:
 		default:
 		{
@@ -361,10 +362,11 @@ thread_articles (GPtrArray    * articles,
 		}
 
 		/* thread by reference
-		   (except for multiparts, which are top-level) */
-		if (a->parts==0 && ((pch=article_get_header(a,HEADER_REFERENCES)))!=NULL && *pch=='<')
+		   (except for parts 2...n of multiparts, which need to be threaded by multipart) */
+		if (a->parts<2 && ((pch=article_get_header(a,HEADER_REFERENCES)))!=NULL && *pch=='<')
 		{
 			gboolean exact = FALSE;
+
 			search_a.message_id = strrchr (pch, '<' );
 
 			index = lower_bound (
@@ -377,7 +379,18 @@ thread_articles (GPtrArray    * articles,
 
 			if (exact && !is_child_of(refs[index],a))
 			{
-				parent = refs[index];
+				gboolean subject_changed = FALSE;
+
+				if (break_thread_when_subject_changes)
+				{
+					gchar * old_subject = normalize_subject (refs[index]);
+					const gchar * new_subject = norm[i].subject;
+					subject_changed = strcmp (old_subject, new_subject);
+					g_free (old_subject);
+				}
+
+				if (!subject_changed)
+					parent = refs[index];
 			}
 		}
 
@@ -505,7 +518,7 @@ check_multipart_articles (GPtrArray * articles)
 			continue;
 
 		/* not a multipart because it's not the first of the set */
-		if (a->part!=1 || a->parent!=NULL)
+		if (a->part!=1)
 			continue;
 
 		/* handle the single-part attachment message */
