@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,7 +34,8 @@ typedef enum
 	ARTICLE_SORT_AUTHOR = 3,
 	ARTICLE_SORT_LINES = 4,
 	ARTICLE_SORT_UNREAD_CHILDREN = 5,
-	ARTICLE_SORT_TYPES_QTY = 5
+	ARTICLE_SORT_MSG_ID = 6,
+	ARTICLE_SORT_TYPES_QTY = 6,
 }
 ArticleSortType;
 
