@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,6 +46,7 @@
 #include "xpm/article_read.xpm"
 #include "xpm/article_unread.xpm"
 #include "xpm/article_new.xpm"
+#include "xpm/disk.xpm"
 #include "xpm/favorite.xpm"
 #include "xpm/bomb.xpm"
 #include "xpm/binary_complete.xpm"
@@ -53,7 +54,6 @@
 #include "xpm/binary.xpm"
 #include "xpm/bluecheck.xpm"
 #include "xpm/mini_page.xpm"
-#include "xpm/thread.xpm"
 
 static GtkWidget* toolbar = NULL;
 
@@ -69,8 +69,8 @@ static GtkWidget* show_normal_rank_button = NULL;
 static GtkWidget* show_saved_button = NULL;
 static GtkWidget* show_queued_button = NULL;
 static GtkWidget* show_idle_button = NULL;
-
-static GtkWidget* threaded_tb = NULL;
+static GtkWidget* show_cached_button = NULL;
+static GtkWidget* show_non_cached_button = NULL;
 
 /*****
 ******  SUBJECT FILTER
@@ -78,16 +78,16 @@ static GtkWidget* threaded_tb = NULL;
 
 static gchar *subject_filter_str = NULL;
 
-const gchar*
+gchar*
 article_toolbar_get_filter (void)
 {
-	return subject_filter_str;
+	return g_strdup (subject_filter_str);
 }
 
 static gchar*
 filter_selected_describe (const StatusItem* item)
 {
-	return g_strdup ("Filtering Articles");
+	return g_strdup (_("Filtering Articles"));
 }
 
 static void
@@ -101,8 +101,6 @@ filter_selected_thread (void* data)
 
 	/* create a new status item to get thread messages */
 	item = STATUS_ITEM(status_item_new(filter_selected_describe));
-	pan_object_ref (PAN_OBJECT(item));
-	pan_object_sink (PAN_OBJECT(item));
 	status_item_set_active (item, TRUE);
 
 	/* update the articlelist */
@@ -131,79 +129,62 @@ article_filter_cb (GtkWidget* w, gpointer data)
 	else
 		pch = NULL;
 	g_free (s);
-	replace_gstr (&subject_filter_str, pch);
 
-	/* update the article list */
-	pthread_create (&thread, NULL, (void*)filter_selected_thread, NULL);
-	pthread_detach (thread);
-}
-
-/*****
-******  THREAD / UNTHREAD ARTICLES
-*****/
-
-static void
-threaded_toggled_cb (GtkToggleButton *tb, gpointer user_data)
-{
-	articlelist_set_threaded (gtk_toggle_button_get_active (tb));
-}
-
-static int
-threaded_changed_cb (gpointer obj, gpointer arg, gpointer user)
-{
-	gtk_toggle_button_set_active (
-		GTK_TOGGLE_BUTTON(threaded_tb), arg!=NULL);
-	return 0;
+	/* if the new string differs from the old one, then update. */
+	if (!pan_strcmp (subject_filter_str, pch))
+		g_free (pch);
+	else {
+		replace_gstr (&subject_filter_str, pch);
+		pthread_create (&thread, NULL, (void*)filter_selected_thread, NULL);
+		pthread_detach (thread);
+	}
 }
 
 /*****
 ******  STATE FILTER
 *****/
 
+static gboolean dampen_feedback_loop = FALSE;
+
 static gint
 state_filter_changed_idle (gpointer data)
 {
 	guint state_filter = GPOINTER_TO_UINT(data);
 
 	pan_lock_unconditional ();
+	dampen_feedback_loop = TRUE;
 
-	gtk_toggle_button_set_active (
-		GTK_TOGGLE_BUTTON(show_new_button),
-		state_filter & STATE_FILTER_NEW);
-	gtk_toggle_button_set_active (
-		GTK_TOGGLE_BUTTON(show_unread_button),
-		state_filter & STATE_FILTER_UNREAD);
-	gtk_toggle_button_set_active (
-		GTK_TOGGLE_BUTTON(show_read_button),
-		state_filter & STATE_FILTER_READ);
-	gtk_toggle_button_set_active (
-		GTK_TOGGLE_BUTTON(show_complete_binary_button),
-		state_filter & STATE_FILTER_COMPLETE_BINARIES);
-	gtk_toggle_button_set_active (
-		GTK_TOGGLE_BUTTON(show_incomplete_binary_button),
-		state_filter & STATE_FILTER_INCOMPLETE_BINARIES);
-	gtk_toggle_button_set_active (
-		GTK_TOGGLE_BUTTON(show_nonbinary_button),
-		state_filter & STATE_FILTER_NONBINARIES);
-	gtk_toggle_button_set_active (
-		GTK_TOGGLE_BUTTON(show_watched_button),
-		state_filter & STATE_FILTER_WATCHED);
-	gtk_toggle_button_set_active (
-		GTK_TOGGLE_BUTTON(show_killfile_button),
-		state_filter & STATE_FILTER_IGNORED);
-	gtk_toggle_button_set_active (
-		GTK_TOGGLE_BUTTON(show_normal_rank_button),
-		state_filter & STATE_FILTER_NORMAL_RANK);
-	gtk_toggle_button_set_active (
-		GTK_TOGGLE_BUTTON(show_saved_button),
-		state_filter & STATE_FILTER_SAVED);
-	gtk_toggle_button_set_active (
-		GTK_TOGGLE_BUTTON(show_queued_button),
-		state_filter & STATE_FILTER_QUEUED);
-	gtk_toggle_button_set_active (
-		GTK_TOGGLE_BUTTON(show_idle_button),
-		state_filter & STATE_FILTER_IDLE);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(show_new_button),
+	                              state_filter & STATE_FILTER_NEW);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(show_unread_button),
+	                              state_filter & STATE_FILTER_UNREAD);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(show_read_button),
+	                              state_filter & STATE_FILTER_READ);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(show_complete_binary_button),
+	                              state_filter & STATE_FILTER_COMPLETE_BINARIES);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(show_incomplete_binary_button),
+	                              state_filter & STATE_FILTER_INCOMPLETE_BINARIES);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(show_nonbinary_button),
+	                              state_filter & STATE_FILTER_NONBINARIES);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(show_watched_button),
+	                              state_filter & STATE_FILTER_WATCHED);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(show_killfile_button),
+	                              state_filter & STATE_FILTER_IGNORED);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(show_normal_rank_button),
+	                              state_filter & STATE_FILTER_NORMAL_RANK);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(show_saved_button),
+	                              state_filter & STATE_FILTER_SAVED);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(show_queued_button),
+	                              state_filter & STATE_FILTER_QUEUED);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(show_idle_button),
+	                              state_filter & STATE_FILTER_IDLE);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(show_cached_button),
+	                              state_filter & STATE_FILTER_CACHED);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(show_non_cached_button),
+	                              state_filter & STATE_FILTER_NOT_CACHED);
 
+
+	dampen_feedback_loop = FALSE;
 	pan_unlock_unconditional ();
 
 	return 0;
@@ -231,7 +212,9 @@ state_filter_button_toggled_cb (GtkToggleButton* tb, gpointer user_data)
 {
         const guint flag = GPOINTER_TO_UINT(user_data);
 	const gboolean on = gtk_toggle_button_get_active (tb);
-	articlelist_poke_state_filter (flag, on);
+
+	if (!dampen_feedback_loop)
+		articlelist_poke_state_filter (flag, on);
 }
 
 GtkWidget*
@@ -359,6 +342,27 @@ article_toolbar_new (void)
 	gtk_container_add (GTK_CONTAINER(frame), hbox);
 	gtk_box_pack_start (GTK_BOX(toolbar), frame, FALSE, FALSE, 0);
 
+	/* filter five: cached/not cached */
+	frame = gtk_frame_new (NULL);
+	gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_IN);
+	hbox = gtk_hbox_new (FALSE, 0);
+	show_cached_button = w = gtk_toggle_button_new ();
+	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("Show Articles which are Cached Locally"), "");
+	gtk_signal_connect (GTK_OBJECT(w), "toggled",
+		state_filter_button_toggled_cb, GUINT_TO_POINTER(STATE_FILTER_CACHED));
+	w2 = gnome_pixmap_new_from_xpm_d (disk_xpm);
+	gtk_container_add (GTK_CONTAINER(w), w2);
+	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
+	show_non_cached_button = w = gtk_toggle_button_new ();
+	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("Show Articles which are not Cached Locally"), "");
+	gtk_signal_connect (GTK_OBJECT(w), "toggled",
+		state_filter_button_toggled_cb, GUINT_TO_POINTER(STATE_FILTER_NOT_CACHED));
+	w2 = gnome_pixmap_new_from_xpm_d (mini_page_xpm);
+	gtk_container_add (GTK_CONTAINER(w), w2);
+	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER(frame), hbox);
+	gtk_box_pack_start (GTK_BOX(toolbar), frame, FALSE, FALSE, 0);
+
 	/* Filter label */
         w = gtk_label_new (_("Find:"));
 	gtk_box_pack_start (GTK_BOX(toolbar), w, FALSE, FALSE, 0);
@@ -366,35 +370,20 @@ article_toolbar_new (void)
 	/* Filter */
 	w = gtk_entry_new();
 	gtk_signal_connect (GTK_OBJECT (w), "activate", GTK_SIGNAL_FUNC(article_filter_cb), w);
+	gtk_signal_connect (GTK_OBJECT (w), "focus-out-event", GTK_SIGNAL_FUNC(article_filter_cb), w);
 	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w,
 		_("Type in a subject search string and press ENTER. "
 	          "Wildcards are allowed."), "");
 	gtk_box_pack_start (GTK_BOX(toolbar), w, TRUE, TRUE, 0);
 
-	/* Thread? */
-	frame = gtk_frame_new (NULL);
-	gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_IN);
-	w = threaded_tb = gtk_toggle_button_new ();
-	gtk_container_add (GTK_CONTAINER(w), gnome_pixmap_new_from_xpm_d (thread_xpm));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w),  TRUE);
-	gtk_signal_connect (GTK_OBJECT(w), "toggled", threaded_toggled_cb, NULL);
-	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("Turn article threading on/off"), "");
-	gtk_container_add (GTK_CONTAINER(frame), w);
-	gtk_box_pack_start (GTK_BOX(toolbar), frame, FALSE, FALSE, 0);
-
 	/* show the toolbar */
 	gtk_widget_show_all (toolbar);
 
 	/* listen for changes to the articlelist "state filter"
 	   so that we can update the menus accordingly */
-	pan_callback_add (
-		articlelist_state_filter_changed,
-		state_filter_changed_cb,
-		NULL);
-	pan_callback_add (
-		articlelist_thread_changed,
-		threaded_changed_cb,
-		NULL);
+	pan_callback_add (articlelist_state_filter_changed,
+	                  state_filter_changed_cb,
+	                  NULL);
 
 	return toolbar;
 }
