@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,6 @@
 
 extern GtkWidget*       article_toolbar_new          (void);
 
-extern const gchar*     article_toolbar_get_filter   (void);
+extern gchar*           article_toolbar_get_filter   (void);
 
 #endif
