@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,6 +42,9 @@
 #include "util.h"
 #include "util-mime.h"
 
+const gchar * default_incoming_name_addr;
+const gchar * default_incoming_name_real;
+
 /***
 ****
 ****  LIFE CYCLE
@@ -56,9 +59,23 @@ article_destructor (PanObject* o)
 
 	/* clean up article parts */
 	debug1 (DEBUG_PAN_OBJECT, "article_destructor: %p", a);
-	g_slist_free (a->threads);
-	if (a->headers != NULL)
+	if (a->headers != NULL) {
 		g_hash_table_destroy (a->headers);
+		a->headers = NULL;
+	}
+	if (a->threads != NULL) {
+		g_slist_free (a->threads);
+		a->threads = NULL;
+	}
+	a->date = 0;
+	a->number = 0;
+	a->linecount = 0;
+	a->message_id = NULL;
+	a->references = NULL;
+	a->subject = NULL;
+	a->author_addr = NULL;
+	a->author_real = NULL;
+	a->group = NULL;
 
 	/* clean up parent */
 	pan_object_destructor (o);
@@ -154,13 +171,19 @@ article_dup (Group * group, const Article * article)
 ***/
 
 static void
-article_set_dirty (Article * article)
+articles_set_dirty (Article ** articles, gint qty)
 {
-	g_return_if_fail (article != NULL);
-	g_return_if_fail (article->group != NULL);
+	gint i;
 
-	article->group->articles_dirty = TRUE;
-	articlelist_update_node (article);
+	g_return_if_fail (articles != NULL);
+	g_return_if_fail (qty >= 1);
+
+	/* flag the group as dirty */
+	for (i=0; i<qty; ++i)
+		group_set_articles_dirty (articles[i]->group, TRUE);
+
+	/* update the articlelist */
+	articlelist_update_articles ((const Article **)articles, qty);
 }
 
 
@@ -201,24 +224,6 @@ article_is_new (const Article * a)
 	return a->number > a->group->article_high_old;
 }
 
-gboolean
-article_thread_has_new (const Article * top)
-{
-	guint i;
-	GPtrArray * thread = g_ptr_array_new ();
-	gboolean has_new = FALSE;
-
-	g_return_val_if_fail (top!=NULL, FALSE);
-
-	article_get_subthread (ARTICLE(top), thread);
-	for (i=0; i!=thread->len && !has_new; ++i)
-		if (article_is_new(ARTICLE(g_ptr_array_index(thread,i))))
-			has_new = TRUE;
-	g_ptr_array_free (thread, TRUE);
-
-	return has_new;
-}
-
 
 /***
 ****
@@ -235,22 +240,15 @@ article_is_read (const Article * a)
 	return newsrc_is_article_read (group_get_newsrc(a->group), a->number);
 }
 
-/**
- * Mark the article (and possibly multipart children) unread.
- * @param article the article to mark as unread.
- * @return the number of articles marked unread in this call to this function.
- */
-static int
-article_set_unread_impl (Article * article)
+static void
+article_set_unread_impl (Article    * article,
+                         GPtrArray  * changed)
 {
-	int unread_qty = 0;
 	const gboolean was_read = newsrc_mark_article (group_get_newsrc(article->group),
 	                                               article->number,
 	                                               FALSE);
-	if (was_read) {
-		++unread_qty;
-		articlelist_article_read_changed (article);
-	}
+	if (was_read)
+		g_ptr_array_add (changed, article);
 
 	/* FIXME: I don't think this special case is such a great idea.
 	   Special case: if it's a multipart, mark 1..n as read too */
@@ -259,29 +257,20 @@ article_set_unread_impl (Article * article)
 		for (l=article->threads; l!=NULL; l=l->next) {
 			Article * child = ARTICLE(l->data);
 			if (child->part > 0)
-				unread_qty += article_set_unread_impl (child);
+				article_set_unread_impl (child, changed);
 		}
 	}
-
-	return unread_qty;
 }
 
-/**
- * Mark the article (and possibly multipart children) read.
- * @param article the article to mark as read.
- * @return the number of articles marked read in this call to this function.
- */
-static int
-article_set_read_impl (Article * article)
+static void
+article_set_read_impl (Article * article,
+		       GPtrArray * changed)
 {
-	int read_qty = 0;
 	const gboolean was_read = newsrc_mark_article (group_get_newsrc(article->group),
 	                                               article->number,
 	                                               TRUE);
-	if (!was_read) {
-		++read_qty;
-		articlelist_article_read_changed (article);
-	}
+	if (!was_read)
+		g_ptr_array_add (changed, article);
 
 	/* FIXME: I don't think this special case is such a great idea.
 	   Special case: if it's a multipart, mark 1..n as read too */
@@ -290,37 +279,50 @@ article_set_read_impl (Article * article)
 		for (l=article->threads; l!=NULL; l=l->next) {
 			Article * child = ARTICLE(l->data);
 			if (child->part > 0)
-				read_qty += article_set_read_impl (child);
+				article_set_read_impl (child, changed);
 		}
 	}
-
-	return read_qty;
 }
 
 void
 articles_set_read (Article ** articles, int article_qty, gboolean read)
 {
 	int i;
-	int read_inc;
+	GPtrArray * changed;
+	debug_enter ("articles_set_read");
 
 	g_return_if_fail (articles != NULL);
 
-	for (read_inc=i=0; i<article_qty; ++i)
+	/* build an array of articles whose read state changed */
+	changed = g_ptr_array_new ();
+	pan_g_ptr_array_reserve (changed, article_qty);
+
+	/* change the articles */
+	for (i=0; i<article_qty; ++i)
 	{
 		Article * article = articles[i];
 
 		g_return_if_fail (article != NULL);
 		g_return_if_fail (article->group != NULL);
-		g_assert (articles[0]->group == article->group);
+		if (articles[0]->group != article->group) {
+			pan_warn_if_reached ();
+			continue;
+		}
 
 		if (read)
-			read_inc += article_set_read_impl (article);
+			article_set_read_impl (article, changed);
 		else 
-			read_inc -= article_set_unread_impl (article);
+			article_set_unread_impl (article, changed);
 	}
 
-	if (read_inc)
-		group_inc_article_read_qty (articles[0]->group, read_inc);
+	if (changed->len != 0)
+	{
+		const gint inc = read ? changed->len : -(int)changed->len;
+		group_inc_article_read_qty (articles[0]->group, inc);
+		articlelist_articles_read_changed ((const Article**)changed->pdata, changed->len);
+	}
+
+	debug_exit ("articles_set_read");
 }
 
 void
@@ -337,34 +339,38 @@ article_set_read (Article * article, gboolean read)
 **/
 
 void
-article_add_flag (Article  * article,
-                  gushort    flag)
+articles_add_flag (Article  ** articles,
+                   gint        qty,
+                   gushort     flag)
 {
-	g_return_if_fail (article != NULL);
-	article->state |= flag;
-	article_set_dirty (article);
+	gint i;
+
+	g_return_if_fail (articles != NULL);
+	g_return_if_fail (qty >= 1);
+
+	for (i=0; i<qty; ++i)
+		articles[i]->state |= flag;
+
+	articles_set_dirty (articles, qty);
 }
 
 
 void
-article_remove_flag (Article         * article,
-                     gushort           flag)
+articles_remove_flag (Article         ** articles,
+                      gint               qty,
+                      gushort            flag)
 {
-	g_return_if_fail (article != NULL);
-	article->state &= ~flag;
-	article_set_dirty (article);
+	gint i;
+
+	g_return_if_fail (articles != NULL);
+	g_return_if_fail (qty >= 1);
+
+	for (i=0; i<qty; ++i)
+		articles[i]->state &= ~flag;
+
+	articles_set_dirty (articles, qty);
 }
 
-void
-article_set_flag (Article * a, gushort flag, gboolean b)
-{
-	g_return_if_fail (a != NULL);
-	if (b)
-		a->state |= flag;
-	else
-		a->state &= ~flag;
-	article_set_dirty (a);
-}
 
 
 /***
@@ -518,13 +524,15 @@ article_set_header (Article         * a,
                     const gchar     * val,
                     HeaderAction      action)
 {
+	g_return_if_fail (a!=NULL);
 	g_return_if_fail (is_nonempty_string(key));
 
 	/* chunk if necessary */
 	if (action & DO_CHUNK)
 	{
 		key = group_chunk_string (a->group, key);
-		val = group_chunk_string (a->group, val);
+		if (val != NULL)
+			val = group_chunk_string (a->group, val);
 	}
 
 	/* if we're possibly erasing a header, remove the old value */
@@ -543,7 +551,7 @@ article_set_header (Article         * a,
 			g_hash_table_insert (a->headers,
 			                     (gpointer)key,
 			                     (gpointer)val);
-			article_set_dirty (a);
+			articles_set_dirty (&a, 1);
 		}
 	}
 }
@@ -561,6 +569,10 @@ article_get_all_headers (const Article * a)
 {
 	GPtrArray * retval = g_ptr_array_new ();
 
+	/* sanity clause */
+	g_return_val_if_fail (a!=NULL, retval);
+
+	/* add the headers */
 	if (a->headers != NULL)
 	{
 		pan_g_ptr_array_reserve (retval, g_hash_table_size(a->headers)*2);
@@ -573,11 +585,7 @@ article_get_all_headers (const Article * a)
 gboolean
 article_has_attachment (const Article *a)
 {
-	const gchar * filename = article_get_header (a, PAN_ATTACH_FILE);
-	if (filename != NULL)
-		return TRUE;
-	else
-		return FALSE;
+	return article_get_header(a,PAN_ATTACH_FILE) != NULL;
 }
 
 /***
@@ -640,6 +648,32 @@ article_get_author_str (const Article * a)
 	return retval;
 }
 
+gchar*
+article_get_short_author_str (const Article * a)
+{
+	gchar * author;
+
+	if (is_nonempty_string (a->author_real) && strcmp(a->author_real,default_incoming_name_real))
+	{
+		author = g_strdup (a->author_real);
+	}
+	else if (is_nonempty_string (a->author_addr) && strcmp(a->author_addr,default_incoming_name_addr))
+	{
+		author = g_strdup (a->author_addr);
+		if (author != NULL) {
+			gchar * pch = strchr (author, '@');
+			if (pch != NULL)
+				*pch = '\0';
+		}
+	}
+	else
+	{
+		author = g_strdup (default_incoming_name_real);
+	}
+
+	return author;
+}
+
 
 gchar*
 article_get_message (const Article * a)
@@ -807,6 +841,7 @@ article_get_attribution_string (const Article * a)
 void
 article_set_author_from_header (Article * a, const gchar * header_from)
 {
+	gchar * pch;
 	gchar * author_addr = NULL;
 	gchar * author_real = NULL;
 
@@ -819,16 +854,23 @@ article_set_author_from_header (Article * a, const gchar * header_from)
 	   syntactically invalid */
 	gnksa_do_check_from (header_from, &author_addr, &author_real);
 
-	if (is_nonempty_string (author_addr))
-		a->author_addr = group_chunk_string (a->group, author_addr);
+	/* use the real mail address, or fill in a default */
+	pch = author_addr;
+	if (!is_nonempty_string (pch))
+		pch = (gchar*)default_incoming_name_addr;
+	a->author_addr = group_chunk_string (a->group, pch);
 
-	if (is_nonempty_string (author_real)) {
-		gnksa_strip_realname (author_real);
-		a->author_real = group_chunk_string (a->group, author_real);
-	}
+	/* use the real name, or fill in a default. */
+	pch = author_real;
+	if (is_nonempty_string (pch))
+		gnksa_strip_realname (pch);
+	else
+		pch = (gchar*)default_incoming_name_real;
+	a->author_real = group_chunk_string (a->group, pch);
 
-	article_set_dirty (a);
+	articles_set_dirty (&a, 1);
 
+	/* cleanup */
 	g_free (author_addr);
 	g_free (author_real);
 }
@@ -852,16 +894,19 @@ article_set_author_from_header (Article * a, const gchar * header_from)
 static Article*
 find_matching_sent_article (const Article * article)
 {
-	Group * sent = folder_get_by_name(PAN_SENT);
+	Group * sent;
 	Article * retval = NULL;
 	GPtrArray * articles;
 	guint i;
+	debug_enter ("find_matching_sent_article");
+
+       	sent = folder_get_by_name(PAN_SENT);
 
 	g_return_val_if_fail (article!=NULL, NULL);
 	g_return_val_if_fail (sent!=NULL, NULL);
 
 	group_ref_articles (sent, NULL);
-	articles = group_get_articles (sent);
+	articles = group_get_article_array (sent);
 
 	for (i=0; retval==NULL && i!=articles->len; ++i)
 	{
@@ -894,6 +939,8 @@ find_matching_sent_article (const Article * article)
 	if (retval == NULL)
 		group_unref_articles (sent, NULL);
 
+	debug_exit ("find_matching_sent_article");
+	g_ptr_array_free (articles, TRUE);
 	return retval;
 }
 
@@ -919,7 +966,11 @@ article_cancel_describe (const StatusItem * status)
 static void
 article_cancel_dtor (gpointer data)
 {
+	debug_enter ("article_cancel_dtor");
+
 	group_unref_articles (folder_get_by_name(PAN_SENT), NULL);
+
+	debug_exit ("article_cancel_dtor");
 }
 void
 article_cancel (Article * article)
@@ -953,8 +1004,10 @@ void
 article_supersede (const Article * article)
 {
 	const Article * ours;
-	Group * sendlater = folder_get_by_name (PAN_SENDLATER);
+	Group * sendlater;
+	debug_enter ("article_supersede");
 
+       	sendlater = folder_get_by_name (PAN_SENDLATER);
 	/* entry assertions */
 	g_return_if_fail (article != NULL);
 	g_return_if_fail (sendlater != NULL);
@@ -982,6 +1035,8 @@ article_supersede (const Article * article)
 		group_unref_articles (ours->group, NULL);
 		/* FIXME: super dangles */
 	}
+
+	debug_exit ("article_supersede");
 }
 
 /****
