@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -71,8 +71,8 @@ typedef struct _Article
 	/* holds infrequently-existing headers.
 	   It should be used sparingly to avoid the overhead of hashtable.
 	   But, since it lets us remove char*'s from Article for lesser-used
-	   headers, it actually decreases sizeof(Article) and helps memory
-	   use 99% of the time.  See the header #defines below...
+	   headers, it actually decreases sizeof(Article) and reduces memory
+	   use most of the time.  See the header #defines below...
 	*/
 	GHashTable* headers;
 }
@@ -122,18 +122,16 @@ Article*     article_dup                    (Group           * group,
 ***  PUBLIC MUTATORS
 **/
 
-void         article_set_author_from_header (Article        * article,
-                                             const gchar    * header_from);
+void         article_set_author_from_header (Article          * article,
+                                             const gchar      * header_from);
 
-void         article_add_flag               (Article         * article,
-                                             gushort           flag_to_set);
+void         articles_add_flag               (Article        ** articles,
+                                              gint              qty,
+                                              gushort           flag_to_set);
 
-void         article_remove_flag            (Article         * article,
-                                             gushort           flag_to_remove);
-
-void         article_set_flag               (Article         * article,
-                                             gushort           flag,
-                                             gboolean          b);
+void         articles_remove_flag            (Article        ** articles,
+                                              gint              qty,
+                                              gushort           flag_to_remove);
 
 /**
 ***  Headers
@@ -176,6 +174,8 @@ gchar*       article_get_attribution_string (const Article   * article);
 
 gchar*       article_get_author_str         (const Article   * article);
 
+gchar*       article_get_short_author_str    (const Article  * article);
+
 void         article_set_from_raw_message   (Article         * article,
                                              const gchar     * text);
 
@@ -187,8 +187,6 @@ gboolean     article_is_read                (const Article   * article);
 
 gboolean     article_is_new                 (const Article   * article);
 
-gboolean     article_thread_has_new         (const Article   * article);
-
 void         article_set_read               (Article         * article,
                                              gboolean          read);
 
@@ -226,8 +224,12 @@ gchar*         article_decode_header        (const gchar     * encoded);
  * removes article from its parent's child list, and
  * clears out all of article's parent/child references.
  */
-void            article_isolate             (Article         * article);
+void           article_isolate             (Article         * article);
 
-gchar*  article_get_thread_message_id (const Article*);
+gchar*         article_get_thread_message_id (const Article*);
+
+extern const gchar * default_incoming_name_real;
+
+extern const gchar * default_incoming_name_addr;
 
 #endif /* __ARTICLE_H__ */
