@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -190,7 +190,7 @@ void articlelist_set_group_thread (Group*);
 
 void articlelist_reload (void);
 
-void articlelist_update_node (const Article* article);
+void articlelist_update_articles (const Article** articles, gint qty);
 
 /**
 ***  Perform Operations on Articles
@@ -272,8 +272,6 @@ void articlelist_download_flagged_bodies (void);
 
 GtkWidget* create_articlelist_ctree (void);
 
-void articlelist_article_read_changed (const Article*);
-
-void search_deja (void);
+void articlelist_articles_read_changed (const Article**, int qty);
 
 #endif /* __ARTICLE_LIST_H__ */
