@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +31,8 @@
 
 unsigned long debug_level = 0;
 
+gint depth = 0;
+ 
 void
 set_debug_level (unsigned long level)
 {
@@ -60,8 +62,8 @@ debug_printf (const gchar   * filename,
 	else if (flag&DEBUG_DECODE) prefix="DECODE";
 	else if (flag&DEBUG_ACACHE) prefix="ACACHE";
 	else if (flag&DEBUG_LOCK) prefix="LOCK";
-	else if (flag&DEBUG_DB) prefix="DATABASE";
 	else if (flag&DEBUG_PAN_OBJECT) prefix="PAN OBJECT";
+	else if (flag&DEBUG_TRACE) prefix="TRACE";
 	else {prefix="FOO";}
 
 	va_start (args, fmt);
@@ -72,13 +74,33 @@ debug_printf (const gchar   * filename,
 	gmtime_r (&now_time_t, &now_tm);
 	strftime (timebuf, sizeof(timebuf), "%H:%M:%S", &now_tm);
 
-	printf ("(%s:%d)(thread %d)(time %s) %s: %s\n",
-		filename,
-		line_num,
-		(int)pthread_self(),
-		timebuf,
-		prefix,
-		string);
+	if (flag&DEBUG_TRACE)
+	{
+		if (fmt[1]=='n')
+			++depth;
+
+		printf ("(%s:%d)(thread %d)(time %s)(depth %d) %s: %s\n",
+			filename,
+			line_num,
+			(int)pthread_self(),
+			timebuf,
+			depth,
+			prefix,
+			string);
+
+		if (fmt[1]=='x')
+			--depth;
+	}
+	else
+	{
+		printf ("(%s:%d)(thread %d)(time %s) %s: %s\n",
+			filename,
+			line_num,
+			(int)pthread_self(),
+			timebuf,
+			prefix,
+			string);
+	}
 	fflush (NULL);
 
 	g_free (string);
