@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,6 +49,7 @@ create_filename (const gchar       * path,
 {
 	gchar * retval;
 	GString * filename;
+	debug_enter ("create_filename");
 
 	g_return_val_if_fail (is_nonempty_string(path), NULL);
 	g_return_val_if_fail (part!=NULL, NULL);
@@ -74,13 +75,17 @@ create_filename (const gchar       * path,
 	if (!filename->len)
 		g_string_assign (filename, _("UNKNOWN"));
 
-	/* filter out any wacky stuff that might make fopen fail */
+	/* filter out directory characters */
 	if (1) {
-		GString * tmp = g_string_new (NULL);
 		const gchar * pch;
+		GString * tmp = g_string_sized_new (filename->len);
 		for (pch=filename->str; *pch; ++pch) {
-			const gchar ch = strchr("<>/\\ ",*pch)==NULL ? *pch : '_';
-			g_string_append_c (tmp, ch);
+			if (*pch==G_DIR_SEPARATOR)
+				g_string_append_c (tmp, '_');
+			else if (*pch=='\\')
+				g_string_append (tmp, "\\\\");
+			else
+				g_string_append_c (tmp, *pch);
 		}
 		g_string_assign (filename, tmp->str);
 		g_string_free (tmp, TRUE);
@@ -104,25 +109,22 @@ create_filename (const gchar       * path,
 		}
 	}
 
-	g_message (_("Using filename [%s]"), filename->str);
 	retval = filename->str;
 	g_string_free (filename, FALSE);
+	pan_normalize_filename_inplace (retval);
+	g_message (_("Using filename [%s]"), retval);
+	debug_exit ("create_filename");
 	return retval;
 }
 
-typedef struct
-{
-	gchar * text;
-}
-Body;
-
 static gchar*
 create_big_article (GPtrArray * articles)
 {
-	Body * bodies;
+	gchar ** bodies;
 	gchar * full = NULL;
 	gulong full_len = 0;
 	gint i;
+	debug_enter ("create_big_article");
 
 	/* sanity clause */
 	g_return_val_if_fail (articles!=NULL, NULL);
@@ -130,13 +132,18 @@ create_big_article (GPtrArray * articles)
 
 	/* get the articles & find out how big the buffer must be */
 	full_len = 0;
-	bodies = g_new (Body, articles->len);
+	bodies = g_new (gchar*, articles->len);
 	for (i=0; i!=articles->len; ++i)
 	{
-		const Article * a = ARTICLE(g_ptr_array_index(articles,i));
-		Body * body = bodies+i;
-		body->text = acache_get_message (a->message_id);
-		full_len += body->text==NULL ? 0 : strlen(body->text);
+		const Article * a;
+		gchar * body;
+
+		a = ARTICLE(g_ptr_array_index(articles,i));
+		body = bodies[i] = acache_get_message (a->message_id);
+		if (body == NULL)
+			log_add_va (LOG_ERROR, _("Couldn't find body for message `%s' - decoding may be incomplete or incorrect"), a->subject);
+		else
+			full_len += strlen(body);
 	}
 
 	/* make the buffer */
@@ -150,34 +157,61 @@ create_big_article (GPtrArray * articles)
 		/* move the articles into it */
 		for (i=0; i!=articles->len; ++i)
 		{
-			const gchar * pch = bodies[i].text;
-			const gchar * tmp;
+			const gchar * begin = bodies[i];
+			const gchar * end;
 
-			/* article [2..n] trim the headers out */
-			if (pch!=NULL && i!=0) {
-				tmp = strstr (pch, "\n\n");
-				if (tmp != NULL)
-					pch = tmp;
-				while (isspace((int)*pch))
-					++pch;
+			/* article 1: keep headers.  all others: drop. */
+			if (begin != NULL) {
+				const gchar * eoh = strstr (begin, "\n\n");
+				if (eoh != NULL) {
+					eoh += 2;
+					if (begin!=NULL && i==0) {
+						memcpy (full+offset, begin, (eoh-begin));
+						offset += (eoh-begin);
+					}
+				}
+				begin = eoh;
 			}
 
 			/* all articles strip out leading space */
-			while (isspace((int)*pch))
-				++pch;
+			while (isspace((int)*begin))
+				++begin;
 
-			/* all articles strip out trailing space after last linefeed */
-			tmp = pch + strlen(pch) - 1;
-			while (tmp>pch && isspace((int)*tmp))
-				--tmp;
-			if (tmp[1]=='\n')
-				++tmp;
+			/* if there's a 'cut here', skip everything leading up to it */
+			if (1) {
+				const gchar * cut = pan_stristr (begin, "cut here");
+				if (cut != NULL)
+					skip_next_token (cut, '\n', &begin);
+			}
 
-			/* do the copying */
-			if (pch!=NULL) {
-				const size_t len = 1 + (tmp-pch);
-				memcpy (full+offset, pch, len);
-				offset += len;
+			/* if there's a second 'cut here', cut everything after it */
+			end = begin + strlen(begin);
+			if (1) {
+				const gchar * cut = pan_stristr (begin, "cut here");
+				if (cut != NULL) {
+					while (cut!=begin && *cut!='\n')
+						--cut;
+					end = cut+1;
+				}
+			}
+
+			/* if there's a body... */
+			if (end != begin)
+			{
+				/* all articles strip out trailing space */
+				--end;
+				while (end!=begin && isspace((int)*end))
+					--end;
+
+				/* copy the body */
+				if (begin!=NULL)
+				{
+					const size_t len = 1 + (end-begin);
+					memcpy (full+offset, begin, len);
+					offset += len;
+					memcpy (full+offset, "\n", 1);
+					offset += 1;
+				}
 			}
 		}
 
@@ -187,9 +221,10 @@ create_big_article (GPtrArray * articles)
 
 	/* cleanup */
 	for (i=0; i!=articles->len; ++i)
-		g_free (bodies[i].text);
+		g_free (bodies[i]);
 	g_free (bodies);
 
+	debug_exit ("create_big_article");
 	return full;
 }
 
@@ -209,6 +244,7 @@ get_decodable_parts (GPtrArray       * articles,
                      GPtrArray      * fillme_array)
 {
 	gchar * text;
+	debug_enter ("get_decodable_parts");
 
 	/* sanity check */
 	g_return_if_fail (articles!=NULL);
@@ -216,10 +252,12 @@ get_decodable_parts (GPtrArray       * articles,
 	/* get the message */
 	text = create_big_article (articles);
 	*setme_msg = pan_g_mime_parser_construct_message (text);
+	g_free (text);
+
+	/* build the array of decodable parts */
 	g_mime_message_foreach_part (*setme_msg, get_array_of_decodable_parts_func, fillme_array);
 
-	/* cleanup */
-	g_free (text);
+	debug_exit ("get_decodable_parts");
 }
 
 /**
@@ -235,8 +273,10 @@ decode_article (const decode_data * dd)
 	GPtrArray * attachments;
 	GSList * p;
 	gint i = 0;
+	gboolean use_mutex;
 	gboolean success = TRUE;
 	Article * first;
+	debug_enter ("decode_article");
 
 	/* sanity clause */
 	g_return_if_fail (dd!=NULL);
@@ -255,8 +295,11 @@ decode_article (const decode_data * dd)
 	   the reason we use mutexes here is that decoding a big mp3 takes
 	   up so _much_ memory that running two at once makes my system
 	   swap to its knees. */
-	status_item_emit_status_va (dd->item, _("Waiting to decode \"%s\""), first->subject);
-	g_static_mutex_lock (&mutex);
+	use_mutex = g_slist_length (dd->articles) > 3;
+	if (use_mutex) {
+		status_item_emit_status_va (dd->item, _("Waiting to decode \"%s\""), first->subject);
+		g_static_mutex_lock (&mutex);
+	}
 
 	/* let the client know what we're doing */
 	status_item_emit_status_va (dd->item, _("Processing \"%s\""), first->subject);
@@ -313,7 +356,7 @@ decode_article (const decode_data * dd)
 				filename,
 				(dd->group ? dd->group->name : "Unknown"),
 				first->subject, i+1);
-			log_add (msg);
+			log_add (LOG_IMPORTANT, msg);
 			debug0 (DEBUG_DECODE, msg);
 			g_free (msg);
 		}
@@ -331,12 +374,14 @@ decode_article (const decode_data * dd)
 	}
 
 	/* update the node */
-	article_remove_flag (first, STATE_DECODE_QUEUED);
-	article_add_flag (first, (success?STATE_DECODED:STATE_DECODE_FAILED));
+	articles_remove_flag (&first, 1, STATE_DECODE_QUEUED);
+	articles_add_flag (&first, 1, (success?STATE_DECODED:STATE_DECODE_FAILED));
 
 	/* cleanup */
 	g_ptr_array_free (articles, TRUE);
 	g_mime_message_destroy (mm);
 	g_ptr_array_free (attachments, TRUE);
-	g_static_mutex_unlock (&mutex);
+	if (use_mutex)
+		g_static_mutex_unlock (&mutex);
+	debug_exit ("decode_article");
 }
