@@ -32,13 +32,13 @@
 #include <gnome.h>
 #include <gdk_imlib.h>
 
-#include "dialogs.h"
-#include "globals.h"
-#include "pan.h"
-#include "prefs.h"
-#include "util.h"
+#include <pan/globals.h>
+#include <pan/pan.h>
+#include <pan/prefs.h>
+#include <pan/util.h>
 
-#include "xpm/pan_druid.xpm"
+#include <pan/dialogs/dialogs.h>
+#include <pan/xpm/pan_druid.xpm>
 
 typedef struct
 {
@@ -141,6 +141,7 @@ druid_finish (GnomeDruidPage  * page,
 		server->max_connections = 4;
 		server->idle_secs_before_timeout = 180;
 		server_add_new_server (server);
+		gnome_config_set_string ("/Pan/State/Server", profile_name);
 	}
 
 	/* save user stats */
@@ -150,7 +151,6 @@ druid_finish (GnomeDruidPage  * page,
 	gnome_config_set_string ("/Pan/Mail/smtp_port", smtp_port);
 	gnome_config_set_string ("/Pan/Mail/smtp_address", smtp_server);
 	mail_server_address = g_strdup (smtp_server);
-
 	
 	/* start Pan proper */
 	gnome_config_sync ();
@@ -420,7 +420,7 @@ druid_server_page (Druid *ddata, DruidChkBoxTemp *chkbox)
  	gtk_table_attach (GTK_TABLE (table), label, 0, 2, 3, 4,
 	 		  GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);
 
-	ddata->auth_toggle = gtk_check_button_new_with_label (_("My server requires me to log on."));
+	ddata->auth_toggle = gtk_check_button_new_with_label (_("My server requires a password."));
 	chkbox->radio_button = ddata->auth_toggle;
  	gtk_table_attach (GTK_TABLE (table), chkbox->radio_button, 0, 2, 4, 5,
 	 		  GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);
