@@ -21,8 +21,8 @@
 #ifndef __DIALOGS_H__
 #define __DIALOGS_H__
 
-#include "group.h"
-#include "rule.h"
+#include <pan/group.h>
+#include <pan/rule.h>
 
 void dialog_about	        (GtkWidget * window);
 
