@@ -6,10 +6,10 @@
 
 #include <ctype.h>
 
-#include "dialogs.h"
-#include "rules_callbacks.h"
-#include "rules_interface.h"
-#include "support.h"
+#include <pan/dialogs/dialogs.h>
+#include <pan/dialogs/rules_callbacks.h>
+#include <pan/dialogs/rules_interface.h>
+#include <pan/dialogs/support.h>
 
 #include <pan/group.h>
 #include <pan/globals.h>
@@ -1297,13 +1297,17 @@ apply_now_thread (void * data)
 	for (i=0; i!=groups->len; ++i)
 	{
 		Group * g = GROUP(g_ptr_array_index(groups,i));
+		GPtrArray * articles;
 
 		status_item_emit_status_va (status, _("Loading group `%s'"), group_get_readable_name(g));
 		group_ref_articles (g, NULL);
 		group_thread_if_needed (g, NULL);
 
 		status_item_emit_status_va (status, _("Applying rule `%s' to Group `%s'"), rule->name, group_get_readable_name(g));
-		rule_apply (rule, group_get_articles(g));
+
+		articles = group_get_article_array (g);
+		rule_apply (rule, articles);
+		g_ptr_array_free (articles, TRUE);
 
 		status_item_emit_status_va (status, _("Saving group `%s'"), group_get_readable_name(g));
 		group_unref_articles (g, NULL);
