@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,6 +35,6 @@ void     file_headers_save             (Group * group, StatusItem * status);
 
 
  
-long     get_string_offset             (GHashTable * hash, const gchar * str, FILE * fp, glong * pos);
+long     get_string_offset             (GHashTable * hash, GMemChunk * long_chunk, const gchar * str, FILE * fp, glong * pos);
 
 #endif
