@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,13 +37,11 @@ typedef struct
 {
 	pthread_t       main_t;
 	GtkWidget     * window;	        /* main window for Pan */
-	GtkWidget     * appbar;	        /* ??? */
 	GtkWidget     * group_clist;	/* ??? */
 	GtkWidget     * article_ctree;	/* ??? */
 	GtkWidget     * widget;		/* ??? */
 	GtkWidget     * text;		/* ??? */
 	GSList        * serverlist;
-	GSList	      * eventlog;
 	int             viewmode;
 	gboolean        mute;		/* if true, outgoing
 					   posts/mail are sent to
