@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -311,7 +311,7 @@ gnksa_split_from (const char * from,
 	}
 
 	end = &work [ strlen(work) - 1 ];
-	debug2 (DEBUG_SOCKET_OUTPUT, "work: [%s], end: [%s]", work, end);
+	debug2 (DEBUG_GNKSA, "work: [%s], end: [%s]", work, end);
 	if (*end == '>') /* Charles Kerr <charles@superpimp.org> */
 	{
 		const char * tmp;
@@ -447,7 +447,7 @@ gnksa_check_domain (const char * domain)
 	char ** labels;
 	const char * top;
 
-	debug1 (DEBUG_SOCKET_OUTPUT, "gnksa_check_domain: [%s]", domain);
+	debug1 (DEBUG_GNKSA, "gnksa_check_domain: [%s]", domain);
 
 	/* check for domain literal */
 	if (*domain == '[')
@@ -487,7 +487,7 @@ gnksa_check_domain (const char * domain)
 
 	/* check existence of the toplevel domain */
 	top = labels[label_qty-1];
-	debug1 (DEBUG_SOCKET_OUTPUT, "top level domain: [%s]", top);
+	debug1 (DEBUG_GNKSA, "top level domain: [%s]", top);
 	switch ((int)strlen(top))
 	{
 		case 1:
@@ -674,7 +674,7 @@ gnksa_do_check_from (const char   * from,
 	int retval;
 	char * address;
 	char * realname;
-	debug1 (DEBUG_SOCKET_OUTPUT, "From: [%s]", from);
+	debug1 (DEBUG_GNKSA, "From: [%s]", from);
 
 	/* split from */
 	retval = gnksa_split_from (from,
@@ -684,8 +684,8 @@ gnksa_do_check_from (const char   * from,
 
 	address = *alloc_and_setme_address;
 	realname = *alloc_and_setme_realname;
-	debug1 (DEBUG_SOCKET_OUTPUT, "Name: [%s]", (realname?realname:"NULL"));
-	debug1 (DEBUG_SOCKET_OUTPUT, "Address: [%s]", (address?address:"NULL"));
+	debug1 (DEBUG_GNKSA, "Name: [%s]", (realname?realname:"NULL"));
+	debug1 (DEBUG_GNKSA, "Address: [%s]", (address?address:"NULL"));
 
 	/* check address */
 	if (address != NULL) {
@@ -693,7 +693,7 @@ gnksa_do_check_from (const char   * from,
 			retval = gnksa_check_address (address);
 	}
 
-	debug1 (DEBUG_SOCKET_OUTPUT, "GNKSA [%d]", retval);
+	debug1 (DEBUG_GNKSA, "GNKSA [%d]", retval);
 	return retval;
 }
 
