@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -148,7 +148,7 @@ group_destructor (PanObject * o)
 	if (g->chunk != NULL)
 		g_string_chunk_free (g->chunk);
 	if (g->_articles != NULL) {
-		g_ptr_array_free (g->_articles, TRUE);
+		g_hash_table_destroy (g->_articles);
 		g->_articles = NULL;
 	}
 
@@ -282,7 +282,8 @@ group_set_article_read_qty (Group * group, gint article_read_qty)
 void
 group_inc_article_read_qty (Group * group, gint inc)
 {
-	group_set_article_read_qty (group, group->article_read_qty + inc);
+	if (inc != 0)
+		group_set_article_read_qty (group, group->article_read_qty + inc);
 }
 
 void
@@ -342,7 +343,12 @@ group_set_is_folder (Group * group, gboolean folder)
 ****
 ***/
 
-/* empty the group, delete the group database file */
+static gboolean
+real_group_empty_ghrfunc (gpointer key, gpointer value, gpointer data)
+{
+	pan_object_unref (PAN_OBJECT(value));
+	return TRUE;
+}
 static void
 real_group_empty (Group * group, gboolean clear_counts)
 {
@@ -356,18 +362,15 @@ real_group_empty (Group * group, gboolean clear_counts)
 	file_headers_destroy (group);
 
 	/* wipe out all the articles... */
-	if (group->_articles != NULL) {
-		pan_g_ptr_array_foreach (group->_articles, (GFunc)pan_object_unref, NULL);
-		g_ptr_array_set_size (group->_articles, 0);
-	}
-
+	if (group->_articles != NULL)
+		g_hash_table_foreach_remove (group->_articles, real_group_empty_ghrfunc, NULL);
 	group_set_article_qty (group, 0);
 	group_set_article_read_qty (group, 0);
 
+	/* if user wants to clear the range, do that too */
 	if (clear_counts)
 		group_set_article_range (group, 0, 0);
 
-
 	/* flush the flags */
 	if (group->flags & GROUP_SUBSCRIBED)
 		group_set_flags (group, GROUP_SUBSCRIBED);
@@ -448,27 +451,6 @@ group_empty_dialog (Group *group)
 ***  ARTICLE LIST
 **/
 
-static int
-compare_pgchar_pparticle_msgid (const void * a, const void *b )
-{
-	const gchar * msgid_a = (const gchar*) a;
-	const gchar * msgid_b = (*(const Article**)b)->message_id;
-	pan_warn_if_fail (is_nonempty_string(msgid_a));
-	pan_warn_if_fail (is_nonempty_string(msgid_b));
-	return pan_strcmp (msgid_a, msgid_b);
-}
-static int
-compare_particle_pparticle_msgid (const void * a, const void * b)
-{
-	const gchar * msgid_a = ((const Article*)a)->message_id;
-	return compare_pgchar_pparticle_msgid (msgid_a, b);
-}
-static int
-compare_pparticle_pparticle_msgid (const void * a, const void * b)
-{
-	return compare_particle_pparticle_msgid (*(const Article**)a, b);
-}
-
 static void
 group_add_articles_ghfunc (gpointer key, gpointer val, gpointer data)
 {
@@ -492,10 +474,15 @@ group_add_articles_ghfunc (gpointer key, gpointer val, gpointer data)
 void
 group_add_article (Group * group, Article * article)
 {
-	GPtrArray * a = g_ptr_array_new ();
+	GPtrArray * a;
+	debug_enter ("group_add_article");
+
+	a = g_ptr_array_new ();
 	g_ptr_array_add (a, article);
 	group_add_articles (group, a, NULL, NULL, NULL);
 	g_ptr_array_free (a, TRUE);
+
+	debug_exit ("group_add_article");
 }
 
  
@@ -527,22 +514,21 @@ group_add_articles_remove_unused (Group            * group,
 }
 
 void
-group_add_articles (Group       * group,
-                    GPtrArray   * articles,
-                    StatusItem  * status,
-                    GPtrArray   * used,
-                    GPtrArray   * ignored)
+group_add_articles (Group             * group,
+                    const GPtrArray   * articles,
+                    StatusItem        * status,
+                    GPtrArray         * used,
+                    GPtrArray         * ignored)
 {
-	gint read_qty;
-	guint i, j;
+	gint old_qty;
+	gint new_read_qty;
+	guint i;
 	gulong low;
 	gulong high;
-	gboolean changed = FALSE;
 	gboolean tmp_ref;
-	GPtrArray * old_art;
-	GPtrArray * new_art;
-	GPtrArray * merged;
-	GHashTable * remove_from_old = NULL;
+	GPtrArray * tmp_used;
+	GHashTable * ours;
+	debug_enter ("group_add_articles");
 
 	/* sanity checks */
 	g_return_if_fail (group!=NULL);
@@ -559,189 +545,158 @@ group_add_articles (Group       * group,
 		group_ref_articles (group, status);
 
 	/**
-	***  Remove the articles from their old groups;
-	***  do this in batches for efficiency.
-	***  This is faster but makes the code a lot uglier.
+	***  Remove the articles from their old groups.
+	***  Batching the removal makes the code uglier but
+	***  it runs faster.
 	**/
 
-	for (i=0; i!=articles->len; ++i)
+	if (1)
 	{
-		Article * article = ARTICLE(g_ptr_array_index(articles,i));
-		Group * oldgroup = article->group;
-		if (oldgroup!=NULL && oldgroup!=group)
+		GHashTable * remove_from_old = NULL;
+
+		for (i=0; i!=articles->len; ++i)
 		{
+			Article * article = ARTICLE(g_ptr_array_index(articles,i));
+			Group * oldgroup = article->group;
 			gboolean array_existed;
 			GPtrArray * to_remove;
 
+			if (oldgroup==NULL || oldgroup==group)
+				continue;
+
 			/* ensure the hash table exists */
 			if (remove_from_old == NULL)
-				remove_from_old = g_hash_table_new (
-					g_direct_hash, g_direct_equal);
+				remove_from_old = g_hash_table_new (g_direct_hash, g_direct_equal);
 
-			/* see if any articles are queued for removing
-			   from that group */
-			to_remove = g_hash_table_lookup (remove_from_old,
-			                                 oldgroup);
+			/* add this array to the array of articles to remove from its group */
+			to_remove = g_hash_table_lookup (remove_from_old, oldgroup);
 			array_existed = to_remove != NULL;
 			if (!array_existed)
 				to_remove = g_ptr_array_new ();
 			g_ptr_array_add (to_remove, article);
+
+			/* we hash the 'remove these' array by group */
 			if (!array_existed)
-				g_hash_table_insert (remove_from_old,
-				                     oldgroup,
-				                     to_remove);
+				g_hash_table_insert (remove_from_old, oldgroup, to_remove);
+		}
+
+		if (remove_from_old != NULL)
+		{
+			g_hash_table_foreach (remove_from_old, group_add_articles_ghfunc, NULL);
+			g_hash_table_destroy (remove_from_old);
 		}
-	}
-	if (remove_from_old != NULL) {
-		g_hash_table_foreach (remove_from_old,
-		                      group_add_articles_ghfunc,
-		                      NULL);
-		g_hash_table_destroy (remove_from_old);
-		remove_from_old = NULL;
 	}
 
 	/**
-	***  Now that the articles don't belong to another group,
-	***  add them to this one.  We do this by sorting the incoming
-	***  articles and merging two sorted lists.
+	***  Get a handle on our current stats...
 	**/
 
-	old_art = group_get_articles (group);
-	new_art = pan_g_ptr_array_dup (articles);
-	pan_g_ptr_array_sort (new_art, compare_pparticle_pparticle_msgid);
-	merged = g_ptr_array_new ();
-	pan_g_ptr_array_reserve (merged, new_art->len + old_art->len);
-	if (used != NULL)
-		pan_g_ptr_array_reserve (used, used->len + new_art->len);
-	if (ignored != NULL)
-		pan_g_ptr_array_reserve (ignored, ignored->len + new_art->len);
-
-	/* now merge the two lists */
 	low = group->article_low;
 	high = group->article_high;
-	for (i=j=0; i!=new_art->len || j!=old_art->len; )
+	ours = group_get_articles (group);
+	old_qty = (int) g_hash_table_size (ours);
+
+	/**
+	***  Now try to add the articles that the client passed in
+	**/
+
+	new_read_qty = 0;
+	tmp_used = g_ptr_array_new ();
+	pan_g_ptr_array_reserve (tmp_used, articles->len);
+	for (i=0; i!=articles->len; ++i)
 	{
-		Article * addme = NULL;
+		Article * a = ARTICLE(g_ptr_array_index(articles,i));
 
-		if (i==new_art->len)
+		if (g_hash_table_lookup (ours, a->message_id) == NULL)
 		{
-			addme = g_ptr_array_index (old_art, j++);
+			a->group = group;
+			g_hash_table_insert (ours, (gpointer)a->message_id, a);
+			g_ptr_array_add (tmp_used, a);
+
+			if (a->number<low)
+				low  = a->number;
+			if (a->number>high)
+				high = a->number;
+			if (newsrc_is_article_read (group_get_newsrc(group), a->number))
+				++new_read_qty;
 		}
-		else if (j==old_art->len)
+		else if (ignored != NULL)
 		{
-			addme = g_ptr_array_index (new_art, i++);
-			changed = TRUE;
-			if (used != NULL)
-				g_ptr_array_add (used, addme);
-		}
-		else
-		{
-			Article * a1 = ARTICLE(g_ptr_array_index(new_art,i));
-			Article * a2 = ARTICLE(g_ptr_array_index(old_art,j));
-			const int comp = pan_strcmp (a1->message_id, a2->message_id);
-
-			if (comp<0) {
-				addme = a1;
-				changed = TRUE;
-				++i;
-				if (used != NULL)
-					g_ptr_array_add (used, addme);
-			} else {
-				addme = a2;
-				++j;
-				if (!comp && a1!=a2) {
-					if (ignored != NULL)
-						g_ptr_array_add (ignored, a1);
-					++i;
-				}
-			}
-		}
-
-		if (addme != NULL)
-		{
-			addme->group = group;
-
-			if (addme->number < low)
-				low = addme->number;
-			if (addme->number > high)
-				high = addme->number;
-
-			g_ptr_array_add (merged, addme);
+			g_ptr_array_add (ignored, a);
 		}
 	}
 
-	/* update the article range */
-	if (high > group->article_high || low<group->article_low)
-		group_set_article_range (group, low, high);
+	/**
+	***  Now update our stats if articles were added
+	**/
 
-	/* update the group 'article' field */
-	group->articles_dirty = TRUE;
-	pan_g_ptr_array_assign (old_art, merged->pdata, merged->len);
-	group_set_article_qty (group, old_art->len);
+	if (tmp_used->len != 0)
+	{
+		const Group * const_group = group;
 
-	/* update the group 'article read' field */
-	read_qty = 0;
-	for (i=0; i!=old_art->len; ++i)
-		if (article_is_read (ARTICLE(g_ptr_array_index(old_art,i))))
-			++read_qty;
-	group_set_article_read_qty (group, read_qty);
+		/* maybe update the article range */
+		if (high!=group->article_high || low!=group->article_low)
+			group_set_article_range (group, low, high);
 
-	/* let everyone know that articles were just added */
-	if (changed) {
+		/* update the group article stats */
 		group->articles_threading_dirty = TRUE;
-		pan_callback_call (group_get_articles_added_callback(), group, articles);
+		group_set_article_qty (group, g_hash_table_size(ours));
+		if (old_qty != 0)
+			new_read_qty += group->article_read_qty;
+		group_set_article_read_qty (group, new_read_qty);
+		group_set_articles_dirty (group, TRUE);
+		grouplist_update_groups (&const_group, 1);
+
+		pan_callback_call (group_get_articles_added_callback(), group, tmp_used);
 	}
 
-#if 0
-	/* folders are for permanent storage, so re-save *NOW*.
-	   This is especially important for pan.sendlater. */
-	if (group_is_folder(group))
-		file_headers_save (group, status);
-#endif
+	if (used != NULL)
+		pan_g_ptr_array_assign (used, tmp_used->pdata, tmp_used->len);
 
 	/* cleanup */
-	g_ptr_array_free (merged, TRUE);
-	g_ptr_array_free (new_art, TRUE);
+	g_ptr_array_free (tmp_used, TRUE);
 	if (tmp_ref)
 		group_unref_articles (group, status);
+	debug_exit ("group_add_articles");
+}
+
+void
+group_set_articles_dirty (Group * group, gboolean dirty)
+{
+	g_return_if_fail (group!=NULL);
+
+	group->articles_dirty = dirty;
 }
 
 void
 group_thread_if_needed (Group * group, StatusItem * status)
 {
+	debug_enter ("group_thread_if_needed");
+
 	g_return_if_fail (group != NULL);
 
-	if (group->articles_threading_dirty && group->_articles!=NULL && group->_articles->len)
+	if (group->articles_threading_dirty && group->_articles!=NULL && g_hash_table_size(group->_articles)!=0)
 	{
-		thread_articles (group->_articles, status);
-		check_multipart_articles (group->_articles);
+		GPtrArray * a;
+
+		a = group_get_article_array (group);
+		thread_articles (a, status);
+		check_multipart_articles (a);
 		group->articles_threading_dirty = FALSE;
+
+		g_ptr_array_free (a, TRUE);
 	}
+
+	debug_exit ("group_thread_if_needed");
 }
 
  
 gpointer
 group_get_article_by_message_id (Group * group, const gchar * message_id)
 {
-	int index = 0;
-	GPtrArray * a;
-	gpointer retval = NULL;
-	gboolean exact_match = FALSE;
-
 	g_return_val_if_fail (group!=NULL, NULL);
 	g_return_val_if_fail (is_nonempty_string(message_id), NULL);
-
-	a = group_get_articles (group);
-        index = lower_bound (message_id,
-		             a->pdata,
-		             a->len,
-		             sizeof(gpointer),
-	                     compare_pgchar_pparticle_msgid,
-		             &exact_match);
-	if (exact_match)
-		retval = g_ptr_array_index (a, index);
-
-	return retval;
+	return g_hash_table_lookup (group_get_articles(group), message_id);
 }
 
 void
@@ -754,15 +709,14 @@ group_remove_article (Group * group, Article * article)
 }
 
 void
-group_remove_articles (Group * group, GPtrArray * articles)
+group_remove_articles (Group * group, const GPtrArray * articles)
 {
 	guint i;
-	guint old_len;
-	int removed_read = 0;
-	int removed_total = 0;
-	GPtrArray * a;
-	GPtrArray * tmp;
-	GPtrArray * removed;
+	int removed_read;
+	int removed_total;
+	GPtrArray * tmp_remove;
+	GHashTable * ours;
+	debug_enter ("group_remove_articles");
 
 	/* sanity checks */
 	g_return_if_fail (group != NULL);
@@ -772,110 +726,97 @@ group_remove_articles (Group * group, GPtrArray * articles)
 		g_assert (ARTICLE(g_ptr_array_index(articles,i))->group==group);
 
 	/* let everyone know that articles are going to be removed */
-	pan_callback_call (group_get_articles_removed_callback(), group, articles);
+	tmp_remove = pan_g_ptr_array_dup ((GPtrArray*)articles);
+	sort_articles ((Article**)tmp_remove->pdata, tmp_remove->len, ARTICLE_SORT_MSG_ID, TRUE);
+	pan_callback_call (group_get_articles_removed_callback(), group, tmp_remove);
 
 	/* is this the last gasp for these articles? */
-	for (i=0; i!=articles->len; ++i) {
-		Article * article = ARTICLE(g_ptr_array_index(articles,i));
-		if (PAN_OBJECT(article)->ref_count == 1) { /* about to die */
+	for (i=0; i!=tmp_remove->len; ++i) {
+		Article * article = ARTICLE(g_ptr_array_index(tmp_remove,i));
+		if (PAN_OBJECT(article)->ref_count == 1) /* about to die */
 			article_isolate (article);
-			acache_delete (article->message_id);
-		}
 	}
 
-	a = group_get_articles (group);
-	old_len = a->len;
+	/**
+	***  Try to remove the articles
+	**/
 
-	/* sort the articles being removed for lower_bound used below */
-	qsort (articles->pdata,
-	       articles->len,
-	       sizeof(gpointer),
-	       compare_pparticle_pparticle_msgid);
-
-	/* build a tmp array of all the articles not being removed */
-	tmp = g_ptr_array_new ();
-	removed = g_ptr_array_new ();
-	pan_g_ptr_array_reserve (tmp, a->len);
-	for (i=0; i!=a->len; ++i)
+	ours = group_get_articles (group);
+	removed_total = 0;
+	removed_read = 0;
+	for (i=0; i!=tmp_remove->len; ++i)
 	{
-		Article * article = ARTICLE(g_ptr_array_index(a,i));
-		gboolean do_remove = FALSE;
-		lower_bound (article,
-		             articles->pdata,
-		             articles->len,
-		             sizeof(gpointer),
-		             compare_particle_pparticle_msgid,
-		             &do_remove);
-		if (do_remove)
+		Article * a = ARTICLE(g_ptr_array_index(tmp_remove,i));
+		Article * our_a = ARTICLE(g_hash_table_lookup (ours, a->message_id));
+
+		if (our_a != NULL)
 		{
+			g_hash_table_remove (ours, a->message_id);
+
 			++removed_total;
-			if (article_is_read(article))
+			if (article_is_read (our_a))
 				++removed_read;
 
-			g_ptr_array_add (removed, article);
-		}
-		else
-		{
-			g_ptr_array_add (tmp, article);
+			our_a->group = NULL;
+			pan_object_unref (PAN_OBJECT(our_a));
 		}
 	}
 
-	g_message (_("Removed %d articles from group %s"),
-	           removed_total,
-		   group->name);
+	/**
+	***  Update the stats
+	**/
 
-	/* free the old articles */
-	for (i=0; i!=removed->len; ++i) {
-		Article * a = ARTICLE(g_ptr_array_index(removed,i));
-		a->group = NULL;
-		pan_object_unref (PAN_OBJECT(a));
-	}
-
-	/* update the group array */
-	pan_g_ptr_array_assign (a, tmp->pdata, tmp->len);
-
-	/* if articles were removed, update our stats */
-	if (a->len != old_len)
+	if (removed_total != 0)
 	{
-		const Group * const_group = (const Group*) group;
-		group->articles_dirty = TRUE;
+		const Group * const_group = group;
+
+		/* update the articles stats */
+		group->articles_threading_dirty = TRUE;
 		group->article_qty -= removed_total;
 		group->article_read_qty -= removed_read;
-		group->articles_threading_dirty = TRUE;
+		group_set_articles_dirty (group, TRUE);
 		grouplist_update_groups (&const_group, 1);
-		if (group_is_folder (group))
-			file_headers_save (group, NULL);
 	}
 
 	/* cleanup */
-	g_ptr_array_free (tmp, TRUE);
-	g_ptr_array_free (removed, TRUE);
+	g_ptr_array_free (tmp_remove, TRUE);
+	debug_exit ("group_remove_articles");
 }
 
 void
 group_expire_old_articles (Group * g)
 {
 	guint i;
+	GPtrArray * tmp;
 	GPtrArray * removeme;
+	debug_enter ("group_expire_old_articles");
 
 	g_return_if_fail (g != NULL);
-	g_return_if_fail (g->_articles != NULL);
-
-	removeme = g_ptr_array_new ();
 
 	/* find old articles to remove */
-	for (i=0; i!=g->_articles->len; ++i) {
-		Article * a = ARTICLE(g_ptr_array_index(g->_articles,i));
+	tmp = group_get_article_array (g);
+	removeme = g_ptr_array_new ();
+	pan_g_ptr_array_reserve (removeme, tmp->len);
+	for (i=0; i!=tmp->len; ++i) {
+		Article * a = ARTICLE(g_ptr_array_index(tmp,i));
 		if ((a->number<g->article_low) || (a->number>g->article_high))
 			g_ptr_array_add (removeme, a);
 	}
 
 	/* remove the old articles */
 	if (removeme->len != 0)
+	{
+		log_add_va (LOG_INFO, _("Expired %u articles from `%s'"),
+			removeme->len,
+			group_get_readable_name(g));
+
 		group_remove_articles (g, removeme);
+	}
 
 	/* cleanup */
+	g_ptr_array_free (tmp, TRUE);
 	g_ptr_array_free (removeme, TRUE);
+	debug_exit ("group_expire_old_articles");
 }
 
 
@@ -886,32 +827,43 @@ group_expire_old_articles (Group * g)
 void
 group_ref_articles (Group * group, StatusItem * status)
 {
+	debug_enter ("group_ref_articles");
+
+	/* sanity clause */
 	g_return_if_fail (group != NULL);
 	g_return_if_fail (group->_articles_refcount >= 0);
 
+	/* if we're up from zero, load the headers */
 	if (++group->_articles_refcount == 1)
 		file_headers_load (group, status);
 
-g_message ("group %s refcount inc to %d", group->name, group->_articles_refcount);
-
-	group_set_article_qty (group, group->_articles->len);
+	/*g_message ("group %s refcount inc to %d", group->name, group->_articles_refcount);*/
+	group_set_article_qty (group, g_hash_table_size(group->_articles));
+	debug_exit ("group_ref_articles");
 }
 
+static void
+group_unref_articles_ghfunc (gpointer key, gpointer val, gpointer data)
+{
+	pan_object_unref (PAN_OBJECT(val));
+}
 void
 group_unref_articles (Group * group, StatusItem * status)
 {
+	debug_enter ("group_unref_articles");
+
+	/* sanity clause */
 	g_return_if_fail (group != NULL);
 	g_return_if_fail (group->_articles_refcount > 0);
 
+	/* if down to zero, save & flush the articles */
 	if (!--group->_articles_refcount)
 	{
 		file_headers_save (group, status);
 
 		/* free the articles */
-		pan_g_ptr_array_foreach (group->_articles,
-		                         (GFunc)pan_object_unref,
-		                         NULL);
-		g_ptr_array_free (group->_articles, TRUE);
+		g_hash_table_foreach (group->_articles, group_unref_articles_ghfunc, NULL);
+		g_hash_table_destroy (group->_articles);
 		group->_articles = NULL;
 
 		/* free the memory used by those articles */
@@ -922,11 +874,10 @@ group_unref_articles (Group * group, StatusItem * status)
 		}
 	}
 
-g_message ("group %s refcount dec to %d", group->name, group->_articles_refcount);
+	/*g_message ("group %s refcount dec to %d", group->name, group->_articles_refcount);*/
+	debug_exit ("group_unref_articles");
 }
 
-
-
 /**
 ***
 **/
@@ -936,6 +887,9 @@ folder_get_by_name (const gchar * foldername)
 {
 	Server * server;
 
+	/* sanity clause */
+	g_return_val_if_fail (is_nonempty_string(foldername), NULL);
+
 	/* get the folder server... */
 	server = server_get_by_name (INTERNAL_SERVER_NAME);
 	g_return_val_if_fail (server!=NULL, NULL);
@@ -953,7 +907,7 @@ group_get_readable_name (const Group * group)
 	if (group == NULL)
 	{
 		pan_warn_if_reached ();
-		retval = "(No Group)";
+		retval = _("(No Group)");
 	}
 	else
 	{
@@ -975,6 +929,8 @@ group_chunk_string (Group         * group,
 {
 	gchar * retval = NULL;
 
+	g_return_val_if_fail (group!=NULL, NULL);
+
 	if (group->chunk == NULL)
 		group->chunk = g_string_chunk_new (8192);
 
@@ -993,7 +949,9 @@ group_set_string_field (Group         * group,
 	g_return_if_fail (group->server!=NULL);
 	g_return_if_fail (field_ptr!=NULL);
 
-	*field_ptr = server_chunk_string (group->server, chunkme_replacement);
+	*field_ptr = chunkme_replacement==NULL
+		? NULL
+		: server_chunk_string (group->server, chunkme_replacement);
 }
 
 
@@ -1009,6 +967,7 @@ new_folder_cb (gchar* string, gpointer data)
 	Group * group = NULL;
 	Server * server = server_get_by_name (INTERNAL_SERVER_NAME);
 	gchar * tmp;
+	debug_enter ("new_folder_cb");
 
 	/* user hit cancel */
 	if (string == NULL)
@@ -1047,6 +1006,7 @@ new_folder_cb (gchar* string, gpointer data)
 
 	/* cleanup */
 	g_free (tmp);
+	debug_exit ("new_folder_cb");
 }
 
 void
@@ -1062,16 +1022,22 @@ group_new_folder_dialog (void)
 void
 group_download_all_articles (Group * group)
 {
-	Task * task;
+	Task * task = NULL;
+	GPtrArray * a = NULL;
 
 	g_return_if_fail (group != NULL);
 	g_return_if_fail (group->_articles != NULL);
 
 	group_ref_articles (group, NULL);
-	task = TASK(task_bodies_new (group, group->_articles));
+	a = group_get_article_array (group);
+	if (a->len)
+		task = TASK(task_bodies_new (group, a));
 	if (task != NULL)
 		queue_add (task);
 	group_unref_articles (group, NULL);
+
+	/* cleanup */
+	g_ptr_array_free (a, TRUE);
 }
 
 Newsrc*
@@ -1092,21 +1058,28 @@ group_has_newsrc (const Group * group)
 	return group->_newsrc != NULL;
 }
 
-GPtrArray*
+GHashTable*
 group_get_articles (Group * group)
 {
 	g_return_val_if_fail (group!=NULL, NULL);
 
 	if (group->_articles == NULL)
-		group->_articles = g_ptr_array_new ();
+		group->_articles = g_hash_table_new (g_str_hash, g_str_equal);
 
 	return group->_articles;
 }
 
-gboolean
-group_articles_reffed (const Group * group)
+GPtrArray*
+group_get_article_array (Group * group)
 {
-	g_return_val_if_fail (group!=NULL, FALSE);
-	return group->_articles_refcount > 0; 
-}
+	GHashTable * hash;
+	GPtrArray * retval;
 
+	g_return_val_if_fail (group!=NULL, NULL);
+
+	hash = group_get_articles (group);
+	retval = g_ptr_array_new ();
+	pan_hash_to_ptr_array (hash, retval);
+
+	return retval;
+}
