@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,7 +49,9 @@ enum
         STATE_FILTER_SAVED                      = (1<<8),
         STATE_FILTER_QUEUED                     = (1<<9),
         STATE_FILTER_IDLE                       = (1<<10),
-	STATE_FILTER_NEW			= (1<<11)
+	STATE_FILTER_NEW			= (1<<11),
+	STATE_FILTER_CACHED			= (1<<12),
+	STATE_FILTER_NOT_CACHED			= (1<<13)
 };
 
 #define GROUP(x) ((Group*)(x))
@@ -68,6 +70,7 @@ typedef struct _Group
 
         guint          articles_dirty           : 1;     /* do the articles need saving? */
 	guint          articles_threading_dirty : 1;
+	gchar          permission;                       /* (read only) 'y'=posting, 'n'=nopost, 'm'=moderated */
 	gint8          flags;                            /* (read only) new, subscribed, etc */
 	gint8          sort_style;                       /* (read only) hint for sorting in UI */
 	guint16        state_filter;                     /* (read only) for filtering articles */
@@ -76,7 +79,6 @@ typedef struct _Group
 	gulong         article_low;                      /* (read only) low article number */
 	gulong         article_high;                     /* (read only) high article number */
 	gulong         article_high_old;                 /* (read only) previous high article */
-	gchar          permission;                       /* (read only) 'y'=posting, 'n'=nopost, 'm'=moderated */
 
 	Server       * server;
 	const gchar  * name;                             /* alt.what.is.my.name */
@@ -88,7 +90,7 @@ typedef struct _Group
 	/* PRIVATE */
 	gint           _articles_refcount;
 	Newsrc       * _newsrc;                          /* tracks read/unread */
-	GPtrArray    * _articles;                        /* articles */
+	GHashTable   * _articles;                        /* articles */
 	gchar        * _one_big_chunk;                   /* used by loading local headers */
 }
 Group;
@@ -185,7 +187,7 @@ void      group_add_article              (Group            * group,
                                           struct _Article  * article);
 
 void      group_add_articles             (Group            * group,
-                                          GPtrArray        * articles,
+                                          const GPtrArray  * articles,
                                           StatusItem       * status_or_null,
 					  GPtrArray        * used,
 					  GPtrArray        * ignored);
@@ -198,7 +200,7 @@ void      group_remove_article           (Group            * group,
                                           struct _Article  * article);
 
 void      group_remove_articles          (Group            * group,
-                                          GPtrArray        * articles);
+                                          const GPtrArray  * articles);
 
 void      group_ref_articles             (Group            * group,
                                           StatusItem       * item);
@@ -217,17 +219,20 @@ void      group_thread_if_needed         (Group            * group,
 
 const gchar* group_chunk_string (Group * group, const gchar * chunkme);
 
+void      group_set_articles_dirty       (Group            * group,
+                                          gboolean           dirty);
+
 Newsrc*   group_get_newsrc               (Group            * group);
 
 gboolean  group_has_newsrc               (const Group      * group);
 
-GPtrArray* group_get_articles            (Group            * group);
+GHashTable* group_get_articles           (Group            * group);
+
+GPtrArray* group_get_article_array       (Group            * group);
 
 gpointer   group_get_article_by_message_id (Group          * group,
                                             const gchar    * message_id);
 
-gboolean   group_articles_reffed         (const Group      * group);
-
 
 /**
 ***  PUBLIC - STATIC
