@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,14 +45,12 @@
 #include "xpm/folder.xpm"
 #include "xpm/envelope.xpm"
 
-
 static void grouplist_update_ui_thread (void* data);
 static void group_menu_popup (GdkEventButton * event);
-
 static void grouplist_filter_changed_cb (void);
+static void grouplist_refresh_filter (void);
 
 extern GtkWidget   * ttips;
-
 static GnomePixmap * folder_pixmap           = NULL;
 static GnomePixmap * group_pixmap            = NULL;
 
@@ -62,7 +60,15 @@ static GtkWidget   * folder_clist_menu       = NULL;
 static Server      * my_server               = NULL;
 static GtkWidget   * grouplist_mode_menu     = NULL;
 static GtkWidget   * groupname_filter_entry  = NULL;
+static gchar       * groupname_filter_string = NULL;
 
+/**
+ * This lock must be gotten _before_ the gui lock, and released
+ * _after_ the gui lock is released.  Otherwise deadlock may occur.
+ */
+static GStaticMutex  grouplist_ui_mutex      = G_STATIC_MUTEX_INIT;
+
+PanCallback * grouplist_server_set_callback = NULL;
 PanCallback * grouplist_group_selection_changed = NULL;
 
 static gchar*
@@ -95,12 +101,6 @@ grouplist_get_server (void)
 	return my_server;
 }
 
-int
-grouplist_get_view_mode (void)
-{
-	return group_mode;
-}
-
 static void
 group_dialog_clicked_cb (GnomeDialog * dialog, int num, Server * server)
 {
@@ -113,21 +113,23 @@ group_dialog_clicked_cb (GnomeDialog * dialog, int num, Server * server)
 void
 grouplist_set_server (Server * server)
 {
-	g_return_if_fail (server != NULL);
+	Server * old_server = my_server;
+	debug_enter ("grouplist_set_server");
 
-	/* nothing to do */
+	g_return_if_fail (server != NULL);
 	if (my_server == server)
 		return;
 
 	/* set the active list */
 	my_server = server;
+	pan_callback_call (grouplist_server_set_callback, my_server, NULL);
 
-	/* Clear the ui */
-	pan_lock ();
-	gtk_clist_clear (GTK_CLIST(Pan.group_clist));
-	pan_unlock ();
+	if (grouplist_get_visible_server() == old_server) {
+		pan_lock ();
+		gtk_clist_clear (GTK_CLIST(Pan.group_clist));
+		pan_unlock ();
+	}
 
-	/* already loaded the groups */
 	if (my_server->groups->len) {
 		grouplist_update_ui (my_server);
 		return;
@@ -161,6 +163,8 @@ grouplist_set_server (Server * server)
 
 		g_free (s);
 	}
+
+	debug_exit ("grouplist_set_server");
 }
 
 /*****
@@ -198,6 +202,7 @@ grouplist_selection_changed_cb (gpointer call_object,
 {
 	GPtrArray * groups = (GPtrArray*)call_arg;
 	Group * single_group = NULL;
+	debug_enter ("grouplist_selection_changed_cb");
 
 	/* load the specified group */
 	if (groups!=NULL && groups->len==1)
@@ -217,12 +222,14 @@ grouplist_selection_changed_cb (gpointer call_object,
 	button_click_count = -1;
 	mb = -1;
 
+	debug_exit ("grouplist_selection_changed_cb");
 	return 0;
 }
 
 static void
 grouplist_button_press (GtkWidget *widget, GdkEventButton * event)
 {
+	debug_enter ("grouplist_button_press");
 	switch (event->button)
 	{
 		case 1:
@@ -234,9 +241,17 @@ grouplist_button_press (GtkWidget *widget, GdkEventButton * event)
 				button_click_count = 1;
 			break;
 		case 3:
+#if 0
+			if (grouplist_get_selected_group() == NULL) {
+				gint row=0, col=0;
+				if (gtk_clist_get_selection_info(GTK_CLIST(widget), event->x, event->y, &row, &col))
+					gtk_clist_select_row (GTK_CLIST(widget), row, 0);
+			}
+#endif
 			group_menu_popup (event);
 			break;
 	}
+	debug_exit ("grouplist_button_press");
 }
 
 static void
@@ -244,6 +259,7 @@ grouplist_key_press (GtkWidget     * widget,
                      GdkEventKey   * event,
                      gpointer        user_data)
 {
+	debug_enter ("grouplist_key_press");
 	switch (event->keyval)
 	{
 		case GDK_Up: case GDK_Down:
@@ -254,6 +270,7 @@ grouplist_key_press (GtkWidget     * widget,
 		default:
 			button_click_count = -1;
 	} 
+	debug_exit ("grouplist_key_press");
 }
 
 static gboolean select_callback_pending = FALSE;
@@ -261,12 +278,17 @@ static gboolean select_callback_pending = FALSE;
 static gint
 select_row_cb_idle (gpointer data)
 {
-	GPtrArray * groups = grouplist_get_selected_groups ();
+	GPtrArray * groups;
+	debug_enter ("select_row_cb_idle");
+
+       	groups = grouplist_get_selected_groups ();
 	pan_callback_call (grouplist_group_selection_changed,
 	                   Pan.group_clist,
 			   groups);
 	select_callback_pending = FALSE;
 	g_ptr_array_free (groups, TRUE);
+
+	debug_exit ("select_row_cb_idle");
 	return FALSE;
 }
 static void
@@ -275,11 +297,15 @@ select_row_cb (GtkWidget  * widget,
                int          col,
                GdkEvent   * event)
 {
+	debug_enter ("select_row_cb");
+
 	if (!select_callback_pending)
 	{
 		select_callback_pending = TRUE;
 		gtk_idle_add (select_row_cb_idle, NULL);
 	}
+
+	debug_exit ("select_row_cb");
 }
 
 /*****
@@ -289,7 +315,10 @@ select_row_cb (GtkWidget  * widget,
 GPtrArray*
 grouplist_get_subscribed_groups (void)
 {
-	GPtrArray * a = g_ptr_array_new ();
+	GPtrArray * a;
+	debug_enter ("grouplist_get_subscribed_groups");
+       
+	a = g_ptr_array_new ();
 
 	if (my_server != NULL)
 	{
@@ -303,6 +332,7 @@ grouplist_get_subscribed_groups (void)
 		}
 	}
 
+	debug_exit ("grouplist_get_subscribed_groups");
 	return a;
 }
 
@@ -310,12 +340,14 @@ void
 grouplist_set_selected_groups (const GPtrArray * a)
 {
 	GtkCList * clist;
+	debug_enter ("grouplist_set_selected_groups");
 
 	if (a != NULL)
 	{
 		guint i;
 
 		clist = GTK_CLIST(Pan.group_clist);
+		g_static_mutex_lock (&grouplist_ui_mutex);
 		pan_lock ();
 		gtk_clist_freeze (clist);
 		gtk_clist_unselect_all (clist);
@@ -333,7 +365,10 @@ grouplist_set_selected_groups (const GPtrArray * a)
 		}
 		gtk_clist_thaw (clist);
 		pan_unlock ();
+		g_static_mutex_unlock (&grouplist_ui_mutex);
 	}
+
+	debug_exit ("grouplist_set_selected_groups");
 }
 
 Group*
@@ -342,6 +377,7 @@ grouplist_get_selected_group (void)
 	GList * l;
 	GtkCList * clist;
 	Group * retval = NULL;
+	debug_enter ("grouplist_get_selected_group");
 
 	pan_lock ();
 	clist = GTK_CLIST(Pan.group_clist);
@@ -350,6 +386,7 @@ grouplist_get_selected_group (void)
 		retval = GROUP(gtk_clist_get_row_data(clist, GPOINTER_TO_INT(l->data)));
 	pan_unlock ();
 
+	debug_exit ("grouplist_get_selected_group");
 	return retval;
 }
 
@@ -359,6 +396,7 @@ grouplist_get_selected_groups (void)
 	GtkCList * clist = GTK_CLIST(Pan.group_clist);
 	GPtrArray * retval = g_ptr_array_new ();
 	GList * l = NULL;
+	debug_enter ("grouplist_get_selected_groups");
 
 	/* get the group selected items; otherwise, 
 	   use the articlelist's group */
@@ -372,6 +410,7 @@ grouplist_get_selected_groups (void)
 	}
 	pan_unlock ();
 
+	debug_exit ("grouplist_get_selected_groups");
 	return retval;
 }
 
@@ -382,31 +421,41 @@ grouplist_get_selected_groups (void)
 void
 grouplist_remove_row (const Group *group)
 {
+	debug_enter ("grouplist_remove_row");
 	if (group->server == my_server)
 	{
 		GtkCList *list = GTK_CLIST(Pan.group_clist);
 		int row;
 
+		g_static_mutex_lock (&grouplist_ui_mutex);
 		pan_lock();
 		row = gtk_clist_find_row_from_data (list, (gpointer)group);
 		if (row != -1)
 			gtk_clist_remove (list, row);
 		pan_unlock();
+		g_static_mutex_unlock (&grouplist_ui_mutex);
 	}
+	debug_exit ("grouplist_remove_row");
 }
 static void
 grouplist_update_selected_rows (void)
 {
-	GPtrArray * groups = grouplist_get_selected_groups ();
+	GPtrArray * groups;
+	debug_enter ("grouplist_update_selected_rows");
+
+       	groups = grouplist_get_selected_groups ();
 	if (groups->len)
 		grouplist_update_groups ((const Group**)groups->pdata, groups->len);
 	g_ptr_array_free (groups, TRUE);
+
+	debug_exit ("grouplist_update_selected_rows");
 }
 static void
 grouplist_process_selected (int type)
 {
 	gint i;
 	GPtrArray * groups;
+	debug_enter ("grouplist_process_selected");
 
 	/* find out which group(s) we're to operate on */
 	groups = grouplist_get_selected_groups ();
@@ -436,15 +485,11 @@ grouplist_process_selected (int type)
 				break;
 
 			case 3: /* download all */
-				queue_add (TASK(task_headers_new(group,
-				                                 HEADERS_ALL,
-				                                 FALSE)));
+				queue_add (TASK(task_headers_new(group, HEADERS_ALL)));
 				break;
 
 			case 4: /* download new */
-				queue_add (TASK(task_headers_new(group,
-				                                 HEADERS_NEW,
-				                                 FALSE)));
+				queue_add (TASK(task_headers_new(group, HEADERS_NEW)));
 				break;
 
 			case 5: /* clean/empty */
@@ -469,6 +514,7 @@ grouplist_process_selected (int type)
 
 	/* cleanup */
 	g_ptr_array_free (groups, TRUE);
+	debug_exit ("grouplist_process_selected");
 }
 
 void
@@ -495,12 +541,17 @@ grouplist_selected_properties (void)
 void
 grouplist_selected_update_count_info (void)
 {
-	GPtrArray * a = grouplist_get_selected_groups ();
+	GPtrArray * a;
+	debug_enter ("grouplist_selected_update_count_info");
+
+       	a = grouplist_get_selected_groups ();
 	if (a->len)
 		queue_add (TASK(task_group_count_new (
 			(Group**)a->pdata, a->len)));
 	else
 		g_ptr_array_free (a, TRUE);
+
+	debug_exit ("grouplist_selected_update_count_info");
 }
 
 void
@@ -508,54 +559,16 @@ grouplist_selected_mark_read (void)
 {
 	GPtrArray * a = grouplist_get_selected_groups ();
 	guint i;
+	debug_enter ("grouplist_selected_mark_read");
+
         for (i=0; i<a->len; ++i)
 	{
 		Group * g = GROUP(g_ptr_array_index(a,i));
 		group_mark_all_read (g, TRUE);
 	}
 	g_ptr_array_free (a, TRUE);
-}
 
-/*****
-******
-*****/
-
-static void
-grouplist_selected_rename_cb (gchar* string, gpointer data)
-{
-	if (is_nonempty_string(string))
-	{
-		Group * group = GROUP(data);
-		const Group * const_group = (const Group*) group;
-		group_set_string_field (group, &group->readable_name, string);
-		group_set_dirty (group);
-		grouplist_update_groups (&const_group, 1);
-	}
-}
-void
-grouplist_selected_rename (void)
-{
-	Group * group = grouplist_get_selected_group ();
-	if (group != NULL)
-	{
-		gchar * fmt = group_is_folder(group)
-			? _("Rename Folder \"%s\"")
-			: _("Rename Group \"%s\"");
-		gchar * prompt = g_strdup_printf (fmt,
-			group_get_readable_name(group));
-
-		pan_lock ();
-		gnome_request_dialog (FALSE,
-		                      prompt,
-		                      group_get_readable_name(group),
-			              128,
-			              grouplist_selected_rename_cb,
-			              group,
-			              GTK_WINDOW(Pan.window));
-		pan_unlock ();
-
-		g_free (prompt);
-	}
+	debug_exit ("grouplist_selected_mark_read");
 }
 
 /*****
@@ -568,6 +581,7 @@ delete_groups_dialog_clicked_cb (GnomeDialog      * dialog,
                                  gpointer           user_data)
 {
 	GPtrArray * a = (GPtrArray*) user_data;
+	debug_enter ("delete_groups_dialog_clicked_cb");
 
 	/* if user clicked yes; destroy the groups */
 	if (button == 0)
@@ -594,6 +608,8 @@ delete_groups_dialog_clicked_cb (GnomeDialog      * dialog,
 	pan_lock();
 	gtk_widget_destroy (GTK_WIDGET(dialog));
 	pan_unlock();
+
+	debug_exit ("delete_groups_dialog_clicked_cb");
 }
 
 
@@ -601,7 +617,10 @@ delete_groups_dialog_clicked_cb (GnomeDialog      * dialog,
 void
 grouplist_selected_destroy (void)
 {
-	GPtrArray * a = grouplist_get_selected_groups ();
+	GPtrArray * a;
+	debug_enter ("grouplist_selected_destroy");
+
+       	a = grouplist_get_selected_groups ();
 	if (a->len != 0)
 	{
 		guint i;
@@ -653,6 +672,8 @@ grouplist_selected_destroy (void)
 		gtk_widget_show_all (dialog);
 		pan_unlock ();
 	}
+
+	debug_exit ("grouplist_selected_destroy");
 }
 
 /*****
@@ -670,6 +691,7 @@ grouplist_start_task_headers (GPtrArray * a,
                               gboolean bodies)
 {
 	guint i;
+	debug_enter ("grouplist_start_task_headers");
 
 	/* sample all the selected groups */
 	for (i=0; i!=a->len; ++i)
@@ -677,21 +699,25 @@ grouplist_start_task_headers (GPtrArray * a,
 		Group * group = GROUP(g_ptr_array_index(a,i));
 		PanObject * o = bodies
 			? task_headers_new_with_bodies (group, type)
-			: task_headers_new (group, type, FALSE);
+			: task_headers_new (group, type);
 		queue_add (TASK(o));
 	}
 
 	/* refresh the ui */
 	grouplist_update_selected_rows ();
+
+	debug_exit ("grouplist_start_task_headers");
 }
 
 void
 grouplist_selected_download_dialog (void)
 {
 	guint i;
-	GPtrArray * a = grouplist_get_selected_groups ();
+	GPtrArray * a;
+	debug_enter ("grouplist_selected_download_dialog");
 
 	/* sample all the selected groups */
+       	a = grouplist_get_selected_groups ();
 	for (i=0; i!=a->len; ++i) {
 		Group * group = GROUP(g_ptr_array_index(a,i));
 		if (!group_is_folder(group))
@@ -703,63 +729,104 @@ grouplist_selected_download_dialog (void)
 
 	/* cleanup */
 	g_ptr_array_free (a, TRUE);
+	debug_exit ("grouplist_selected_download_dialog");
 }
 
 void
 grouplist_selected_download_sample (void)
 {
-	GPtrArray * a = grouplist_get_selected_groups ();
+	GPtrArray * a;
+	debug_enter ("grouplist_selected_download_sample");
+
+       	a = grouplist_get_selected_groups ();
 	grouplist_start_task_headers (a, HEADERS_SAMPLE, FALSE);
 	g_ptr_array_free (a, TRUE);
+
+	debug_exit ("grouplist_selected_download_sample");
 }
 void
 grouplist_selected_download_sample_bodies (void)
 {
-	GPtrArray * a = grouplist_get_selected_groups ();
+	GPtrArray * a;
+	debug_enter ("grouplist_selected_download_sample_bodies");
+
+       	a = grouplist_get_selected_groups ();
 	grouplist_start_task_headers (a, HEADERS_SAMPLE, TRUE);
 	g_ptr_array_free (a, TRUE);
+
+	debug_exit ("grouplist_selected_download_sample_bodies");
 }
 void
 grouplist_selected_download_all (void)
 {
-	GPtrArray * a = grouplist_get_selected_groups ();
+	GPtrArray * a;
+	debug_enter ("grouplist_selected_download_all");
+
+       	a = grouplist_get_selected_groups ();
 	grouplist_start_task_headers (a, HEADERS_ALL, FALSE);
 	g_ptr_array_free (a, TRUE);
+
+	debug_exit ("grouplist_selected_download_all");
 }
 void
 grouplist_selected_download_all_bodies (void)
 {
-	GPtrArray * a = grouplist_get_selected_groups ();
+	GPtrArray * a;
+	debug_enter ("grouplist_selected_download_all_bodies");
+
+       	a = grouplist_get_selected_groups ();
 	grouplist_start_task_headers (a, HEADERS_ALL, TRUE);
 	g_ptr_array_free (a, TRUE);
+
+	debug_exit ("grouplist_selected_download_all_bodies");
 }
 void
 grouplist_selected_download_new (void)
 {
-	GPtrArray * a = grouplist_get_selected_groups ();
+	GPtrArray * a;
+	debug_enter ("grouplist_selected_download_new");
+
+       	a = grouplist_get_selected_groups ();
 	grouplist_start_task_headers (a, HEADERS_NEW, FALSE);
 	g_ptr_array_free (a, TRUE);
+
+	debug_exit ("grouplist_selected_download_new");
 }
 void
 grouplist_selected_download_new_bodies (void)
 {
-	GPtrArray * a = grouplist_get_selected_groups ();
+	GPtrArray * a;
+	debug_enter ("grouplist_selected_download_new_bodies");
+
+       	a = grouplist_get_selected_groups ();
 	grouplist_start_task_headers (a, HEADERS_NEW, TRUE);
 	g_ptr_array_free (a, TRUE);
+
+	debug_exit ("grouplist_selected_download_new_bodies");
 }
 void
 grouplist_subscribed_download_new (void)
 {
-	GPtrArray * a = grouplist_get_subscribed_groups ();
+	GPtrArray * a;
+	debug_enter ("grouplist_subscribed_download_new");
+
+       	a = grouplist_get_subscribed_groups ();
 	grouplist_start_task_headers (a, HEADERS_NEW, FALSE);
 	g_ptr_array_free (a, TRUE);
+
+	debug_exit ("grouplist_subscribed_download_new");
 }
 void
 grouplist_subscribed_download_new_bodies (void)
 {
-	GPtrArray * a = grouplist_get_subscribed_groups ();
+	GPtrArray * a;
+	debug_enter ("grouplist_subscribed_download_new_bodies");
+
+       	a = grouplist_get_subscribed_groups ();
 	grouplist_start_task_headers (a, HEADERS_NEW, TRUE);
 	g_ptr_array_free (a, TRUE);
+
+	debug_exit ("grouplist_subscribed_download_new_bodies");
 }
 
 /*****
@@ -811,6 +878,7 @@ static void
 grouplist_set_view_mode (GtkWidget *widget, gpointer data)
 {
 	GtkWidget *menu = NULL;
+	debug_enter ("grouplist_set_view_mode");
 
 	group_mode = GPOINTER_TO_INT (data);
 	grouplist_update_ui (grouplist_get_visible_server());
@@ -837,6 +905,8 @@ grouplist_set_view_mode (GtkWidget *widget, gpointer data)
         gtk_option_menu_set_menu (GTK_OPTION_MENU(grouplist_mode_menu), menu);
         gtk_object_unref (GTK_OBJECT(menu));
         pan_unlock ();
+
+	debug_exit ("grouplist_set_view_mode");
 }
 
 typedef struct
@@ -886,6 +956,12 @@ grouplist_mode_menu_create (void)
 ******
 *****/
 
+void
+grouplist_shutdown_module (void)
+{
+	gnome_config_set_int ("/Pan/State/group_mode", group_mode);
+}
+
 gpointer
 grouplist_create (void)
 {
@@ -903,7 +979,7 @@ grouplist_create (void)
 	titles[4] = _("Description");
 
        	toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL,GTK_TOOLBAR_TEXT);
-
+	grouplist_server_set_callback = pan_callback_new ();
 	grouplist_group_selection_changed = pan_callback_new ();
 
 	pan_callback_add (grouplist_group_selection_changed,
@@ -943,6 +1019,8 @@ grouplist_create (void)
 	groupname_filter_entry = w = gtk_entry_new ();
 	gtk_signal_connect (GTK_OBJECT (w), "activate",
 			    GTK_SIGNAL_FUNC(grouplist_filter_changed_cb), w);
+	gtk_signal_connect (GTK_OBJECT (w), "focus-out-event",
+			    GTK_SIGNAL_FUNC(grouplist_filter_changed_cb), w);
 	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w,
 		_("Type in a group search string and press ENTER.  "
 		  "Wildcards are allowed."), "");
@@ -1008,7 +1086,7 @@ grouplist_create (void)
 static void
 grouplist_filter_changed_cb (void)
 {
-	grouplist_update_ui (grouplist_get_visible_server());
+	grouplist_refresh_filter ();
 }
 
 /*****
@@ -1025,14 +1103,13 @@ void
 grouplist_load (Server * server)
 {
 	StatusItem * status = NULL;
+	debug_enter ("grouplist_load");
 
 	/* entry assertions */
 	g_return_if_fail (server!=NULL);
 
 	/* create a new status item to get load messages */
 	status = STATUS_ITEM(status_item_new(grouplist_load_describe));
-	pan_object_ref(PAN_OBJECT(status));
-	pan_object_sink(PAN_OBJECT(status));
 	status_item_set_active (status, TRUE);
 
 	/* load the groups... */
@@ -1042,6 +1119,8 @@ grouplist_load (Server * server)
 	status_item_set_active (status, FALSE);
 	pan_object_unref(PAN_OBJECT(status));
 	status = NULL;
+
+	debug_exit ("grouplist_load");
 }
 
 /*****
@@ -1116,21 +1195,21 @@ grouplist_update_describe (const StatusItem *item)
 static void
 grouplist_update_ui_thread (void* data)
 {
-	gchar * s;
-	gchar * pch;
 	gchar * title;
-	Server * server = grouplist_get_visible_server ();
+	Server * server;
 	GtkCList * clist = GTK_CLIST(Pan.group_clist);
-	GPtrArray * sel = grouplist_get_selected_groups ();
+	GPtrArray * sel;
 	StatusItem *item = NULL;
 	int i;
+	debug_enter ("grouplist_update_ui_thread");
 
+       	server = grouplist_get_visible_server ();
 	g_return_if_fail (server != NULL);
 
+       	sel = grouplist_get_selected_groups ();
+
         /* create a new status item to get sort/thread messages */
 	item = STATUS_ITEM(status_item_new(grouplist_update_describe));
-	pan_object_ref(PAN_OBJECT(item));
-	pan_object_sink(PAN_OBJECT(item));
 	status_item_set_active (item, TRUE);
 
 	status_item_emit_init_steps (item, server->groups->len);
@@ -1145,20 +1224,8 @@ grouplist_update_ui_thread (void* data)
 		default:               title = _("Bug in Code");        break;
 	}
 
-	/* wrap the key in wildcards to make it a substring search...
-	   unless there are already wildcards present, indicating the
-	   user already knows what he wants */
-        pch = gtk_editable_get_chars (GTK_EDITABLE(groupname_filter_entry), 0, -1);
-	pch = g_strstrip (pch);
-	if (strchr(pch,'*'))
-		s = g_strdup (pch);
-	else if (*pch)
-		s = g_strdup_printf("*%s*",pch);
-	else
-		s = NULL;
-	replace_gstr (&pch, NULL);
-
 	/* update the list */
+	g_static_mutex_lock (&grouplist_ui_mutex);
 	pan_lock ();
 	gtk_clist_freeze (clist);
 	gtk_clist_clear (clist);
@@ -1169,10 +1236,13 @@ grouplist_update_ui_thread (void* data)
 	{
 		Group * group = GROUP(g_ptr_array_index (server->groups, i));
 
-		status_item_emit_next_step_inside_lock (item);
+		pan_unlock ();
+		status_item_emit_next_step (item);
+		pan_lock ();
+
 		if (!should_be_in_grouplist (server, group))
 			continue;
-		if (is_nonempty_string(s) && fnmatch(s,group->name,0))
+		if (is_nonempty_string(groupname_filter_string) && fnmatch(groupname_filter_string,group->name,PAN_CASEFOLD))
 			continue;
 
 		grouplist_add_group_nolock (group);
@@ -1180,6 +1250,7 @@ grouplist_update_ui_thread (void* data)
 	}
 	gtk_clist_thaw (clist);
 	pan_unlock();
+	g_static_mutex_unlock (&grouplist_ui_mutex);
 
 	/* update the selection */
 	grouplist_set_selected_groups (sel);
@@ -1189,20 +1260,24 @@ grouplist_update_ui_thread (void* data)
 	pan_object_unref(PAN_OBJECT(item));
 
 	/* cleanup */
-	g_free (s);
 	g_ptr_array_free (sel, TRUE);
+	debug_exit ("grouplist_update_ui_thread");
 }
 
 void
-grouplist_update_ui (Server *server)
+grouplist_update_ui (Server * server)
 {
-	if (server == grouplist_get_visible_server())
+	debug_enter ("grouplist_update_ui");
+
+	if (server!=NULL && server==grouplist_get_visible_server())
 	{
 		pthread_t thread;
-		g_return_if_fail (server != NULL);
+
 		pthread_create (&thread, NULL, (void*)grouplist_update_ui_thread, server);
 		pthread_detach (thread);
 	}
+
+	debug_exit ("grouplist_update_ui");
 }
 
 /***
@@ -1258,24 +1333,58 @@ grouplist_update_group_nolock (const Group * group)
 		gtk_clist_set_text (list, row, 2, buf);
 	}
 }
+
+static gint
+grouplist_update_groups_idle (gpointer data)
+{
+	guint i;
+	GPtrArray * a = (GPtrArray*)data;
+	GtkCList * list = GTK_CLIST (Pan.group_clist);
+	debug_enter ("grouplist_update_groups_idle");
+
+	g_static_mutex_lock (&grouplist_ui_mutex);
+	pan_lock_unconditional ();
+	gtk_clist_freeze (list);
+	for (i=0; i!=a->len; ++i)
+		grouplist_update_group_nolock (GROUP(g_ptr_array_index(a,i)));
+	gtk_clist_thaw (list);
+	pan_unlock_unconditional ();
+	g_static_mutex_unlock (&grouplist_ui_mutex);
+
+	/* cleanup */
+	g_ptr_array_free (a, TRUE);
+	debug_exit ("grouplist_update_groups_idle");
+	return 0;
+}
+
 void
 grouplist_update_groups (const Group ** groups, int group_qty)
 {
 	gint i;
-	GtkCList * list = GTK_CLIST (Pan.group_clist);
+	GPtrArray * a;
+	Server * server;
+	debug_enter ("grouplist_update_groups");
 
 	g_return_if_fail (groups != NULL);
 	g_return_if_fail (group_qty >= 1);
 
-	/* update the list... */
-	pan_lock ();
-	if (group_qty>1)
-		gtk_clist_freeze (list);
+	/* build an array of groups that we might need to update */
+	a = g_ptr_array_new ();
+	server = grouplist_get_visible_server ();
 	for (i=0; i<group_qty; ++i)
-		grouplist_update_group_nolock (groups[i]);
-	if (group_qty>1)
-		gtk_clist_thaw (list);
-	pan_unlock ();
+		if (groups[i]!=NULL && groups[i]->server==server)
+			g_ptr_array_add (a, (gpointer)groups[i]);
+
+	/* if we have any that we should update, queue it */
+	if (a->len != 0) {
+		pan_lock ();
+		gtk_idle_add (grouplist_update_groups_idle, a);
+		pan_unlock ();
+	} else {
+		g_ptr_array_free (a, TRUE);
+	}
+
+	debug_exit ("grouplist_update_groups");
 }
 
 /***
@@ -1283,11 +1392,10 @@ grouplist_update_groups (const Group ** groups, int group_qty)
 ****
 ***/
 
-
 static void
-grouplist_internal_select_row (int row,
-                               gboolean activate,
-                               gboolean lock)
+grouplist_internal_select_row (gint      row,
+                               gboolean  activate,
+                               gboolean  lock)
 {
         GtkCList* list = GTK_CLIST(Pan.group_clist);
 
@@ -1400,16 +1508,15 @@ grouplist_activate_next_group (void)
 static void
 group_menu_popup (GdkEventButton *event)
 {
-	gboolean selected = grouplist_get_selected_group != NULL;
+	gboolean selected = grouplist_get_selected_group() != NULL;
 	GList * l = NULL;
+	debug_enter ("group_menu_popup");
 
 	if (group_mode == GROUP_FOLDERS) /* We are dealing with folders */
 	{
 		pan_lock ();
 		l=GTK_MENU_SHELL(folder_clist_menu)->children;
 		/* l is new folder */
-		l=l->next; /* rename group/folder */
-		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
 		l=l->next; /* delete folder */
 		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
 
@@ -1422,9 +1529,9 @@ group_menu_popup (GdkEventButton *event)
 	else /* We must be dealing with real groups */
 	{
 		pan_lock ();
-		l=GTK_MENU_SHELL(group_clist_menu)->children; 
-		/* l is download dialog */
-		gtk_widget_set_sensitive (GTK_WIDGET(l->data), TRUE);
+
+		l=GTK_MENU_SHELL(group_clist_menu)->children;  /* download */
+		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
 		l=l->next; /* l is separator */
 		l=l->next; /* l is get new headers in subscribed */
 		gtk_widget_set_sensitive (GTK_WIDGET(l->data), TRUE);
@@ -1460,8 +1567,6 @@ group_menu_popup (GdkEventButton *event)
 		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
 		l=l->next; /* empty group */
 		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
-		l=l->next; /* rename group */
-		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
 		l=l->next; /* properties */
 		gtk_widget_set_sensitive (GTK_WIDGET(l->data), selected);
 
@@ -1471,4 +1576,36 @@ group_menu_popup (GdkEventButton *event)
 				event->button, event->time);
 		pan_unlock ();
 	} 
+
+	debug_exit ("group_menu_popup");
+}
+
+static void
+grouplist_refresh_filter (void)
+{
+	gchar * new_filter;
+        gchar * pch;
+
+	pan_lock ();
+	pch  = gtk_editable_get_chars (GTK_EDITABLE(groupname_filter_entry), 0, -1);
+	pan_unlock ();
+
+	/* build the new filter string */
+	new_filter = NULL;
+	if (is_nonempty_string (pch)) {
+		if (strchr(pch,'*') != NULL)
+			new_filter = g_strdup (pch);
+		else 
+			new_filter = g_strdup_printf("*%s*",pch);
+	}
+
+	/* if it differs from the old filter string, refresh */
+	if (pan_strcmp (new_filter, groupname_filter_string)) {
+		replace_gstr (&groupname_filter_string, g_strdup(new_filter));
+		grouplist_update_ui (grouplist_get_visible_server());
+	}
+
+	/* cleanup */
+	g_free (pch);
+	g_free (new_filter);
 }
