@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,14 +28,13 @@
 #include "server.h"
 
 extern PanCallback * grouplist_group_selection_changed;
+extern PanCallback * grouplist_server_set_callback;
 
 extern void grouplist_select_next_unread_group     (void);
 extern void grouplist_select_next_group            (void);
 extern void grouplist_activate_next_unread_group   (void);
 extern void grouplist_activate_next_group          (void);
 
-extern int  grouplist_get_view_mode         (void);
-
 extern void grouplist_update_ui             (Server * server);
 extern void grouplist_update_groups         (const Group ** groups, int group_qty);
 extern void grouplist_process_selcted	    (int type);
@@ -51,7 +50,6 @@ extern void grouplist_selected_download_sample_bodies  (void);
 extern void grouplist_selected_download_bodies         (void);
 extern void grouplist_selected_download_dialog         (void);
 extern void grouplist_selected_mark_read               (void);
-extern void grouplist_selected_rename                  (void);
 extern void grouplist_selected_update_count_info       (void);
 
 extern void grouplist_selected_subscribe               (void);
@@ -74,6 +72,8 @@ extern Server *grouplist_get_server (void);
 extern GPtrArray* grouplist_get_selected_groups (void);
 extern Group* grouplist_get_selected_group (void);
 extern GPtrArray* grouplist_get_subscribed_groups (void);
+
 extern gpointer grouplist_create (void);
+extern void grouplist_shutdown_module (void);
 
 #endif /* __GROUPLIST_H__ */
