@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -97,7 +97,7 @@ gui_notebook_construct (void)
 	gtk_notebook_set_tab_border (GTK_NOTEBOOK (notebook), 2);
 
 	gui_restore_column_widths (Pan.group_clist, "group");
-	gui_restore_column_widths (Pan.article_ctree, "articlelist");
+	gui_restore_column_widths (Pan.article_ctree, "threads");
 
 	if (Pan.viewmode == GUI_PANED)
 	{
