@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -157,7 +157,7 @@ gui_paned_construct (void)
 			gtk_paned_add1 (GTK_PANED(v1), w[0]);
 			gtk_paned_add2 (GTK_PANED(v1), v2);
 			gtk_paned_set_position (GTK_PANED(v1), vpos);
-			gtk_paned_set_position (GTK_PANED(v2), vpos+50);
+			gtk_paned_set_position (GTK_PANED(v2), hpos);
 			gtk_box_pack_start (GTK_BOX(contents_vbox), v1, TRUE, TRUE, 0);
 			break;
 		}
@@ -179,7 +179,7 @@ gui_paned_construct (void)
 
 	Pan.viewmode = GUI_PANED;
 	gui_restore_column_widths (Pan.group_clist, "group");
-	gui_restore_column_widths (Pan.article_ctree, "articlelist");
+	gui_restore_column_widths (Pan.article_ctree, "threads");
 	gtk_widget_show_all (GTK_WIDGET(contents_vbox));
 
 	gtk_widget_unref (groups_vbox);
