@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,42 +21,50 @@
 #include <config.h>
 #include <gnome.h>
 
+#include <ctype.h>
 #include <sys/time.h>          
 
-#include "article-find.h"
-#include "article-thread.h"
-#include "articlelist.h"
-#include "dialogs/dialogs.h"
-#include "dialogs/rules_interface.h"
-#include "globals.h"
-#include "gnksa.h"
-#include "gui.h"
-#include "gui-notebook.h"
-#include "gui-paned.h"
-#include "grouplist.h"
-#include "log.h"
-#include "message-send.h"
-#include "message-window.h"
-#include "pan.h"
-#include "prefs.h"
-#include "print.h"
-#include "status-item.h"
-#include "status-item-view.h"
-#include "save.h"
-#include "text.h"
-#include "task-manager.h"
-#include "util.h"
-#include "queue.h"
+#include <pan/article-find.h>
+#include <pan/article-thread.h>
+#include <pan/articlelist.h>
+#include <pan/debug.h>
+#include <pan/file-grouplist.h>
+#include <pan/file-headers.h>
+#include <pan/globals.h>
+#include <pan/gnksa.h>
+#include <pan/gui.h>
+#include <pan/gui-notebook.h>
+#include <pan/gui-paned.h>
+#include <pan/grouplist.h>
+#include <pan/log.h>
+#include <pan/message-send.h>
+#include <pan/message-window.h>
+#include <pan/pan.h>
+#include <pan/prefs.h>
+#include <pan/print.h>
+#include <pan/status-item.h>
+#include <pan/status-item-view.h>
+#include <pan/save.h>
+#include <pan/text.h>
+#include <pan/task-manager.h>
+#include <pan/util.h>
+#include <pan/queue.h>
+#include <pan/dialogs/dialogs.h>
+#include <pan/dialogs/rules_interface.h>
+#include <pan/xpm/envelope.xpm>
+#include <pan/xpm/envelope_new.xpm>
+#include <pan/xpm/caution.xpm>
+#include <pan/xpm/inform.xpm>
 
 GtkTooltips * ttips = NULL;
 GdkColormap * cmap = NULL;
 
-#include "xpm/envelope.xpm"
-#include "xpm/envelope_new.xpm"
-
 static unsigned int gui_refresh_timeout_id = 0;
 static double KBps = 0.0;
-static GtkWidget * taskbar;
+static GtkWidget * taskbar = NULL;
+static GtkWidget * error_button = NULL;
+static GtkWidget * status_ok_label = NULL;
+static GtkWidget * status_error_label = NULL;
 
 #define VIEW_QTY 3
 typedef struct {
@@ -79,6 +87,7 @@ static int thread_changed_cb                    (gpointer, gpointer, gpointer);
 static int gui_articlelist_sort_changed_cb      (gpointer, gpointer, gpointer);
 static int gui_text_show_all_headers_changed_cb (gpointer, gpointer, gpointer);
 static int status_item_active_changed_cb        (gpointer, gpointer, gpointer);
+static void turn_off_status_item_views (void);
 
 static void
 server_import_newsrc_dialog_menu (void)
@@ -96,6 +105,53 @@ server_export_newsrc_dialog_menu (void)
 }
 
 
+
+static gchar*
+save_setup_describe (const StatusItem * item)
+{
+	return g_strdup (_("Saving Pan's current state"));
+}
+static void
+save_setup_thread (void* unused)
+{
+	GSList * l;
+	StatusItem * status;
+	debug_enter ("save_setup");
+
+	/* show the user what's going on */
+	status = STATUS_ITEM(status_item_new(save_setup_describe));
+        status_item_set_active (status, TRUE);
+
+	/* save everything */
+	for (l=Pan.serverlist; l!=NULL; l=l->next) {
+		guint i;
+		Server * server = SERVER(l->data);
+		status_item_emit_status_va (status, "%s - %d groups",
+			server->name,
+			(int)server->groups->len);
+		file_grouplist_save (server, status);
+		for (i=0; i<server->groups->len; ++i) {
+			Group * group = GROUP(g_ptr_array_index(server->groups,i));
+			if (group->articles_dirty)
+				file_headers_save (group, status);
+		}
+	}
+
+	/* cleanup */
+        status_item_set_active (status, FALSE);
+	pan_object_unref (PAN_OBJECT(status));
+	log_add (LOG_IMPORTANT, _("Saved settings to disk."));
+	debug_exit ("save_setup");
+}
+static void
+save_setup (gpointer data)
+{
+	pthread_t thread;
+	pthread_create (&thread, NULL, (void*)save_setup_thread, NULL);
+	pthread_detach (thread);
+}
+
+
 static GnomeUIInfo file_menu[] =
 {
 	{
@@ -134,6 +190,17 @@ static GnomeUIInfo file_menu[] =
 		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE,
 		'\0', 0, NULL
 	},
+	GNOMEUIINFO_SEPARATOR,
+	{
+		GNOME_APP_UI_ITEM,
+		N_("Save Setup"),
+		N_("Save Setup"),
+		save_setup, NULL,
+		NULL,
+		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE,
+		'\0', 0, NULL
+	},
+
 	GNOMEUIINFO_SEPARATOR,
 	GNOMEUIINFO_MENU_PRINT_ITEM (print_cb, NULL),
 	GNOMEUIINFO_SEPARATOR,
@@ -469,19 +536,26 @@ pan_manual_url (void)
 static void
 send_feedback_cb (GtkWidget * w)
 {
-	gchar * from = get_default_author_from ();
-	gchar * msg_id = gnksa_generate_message_id ("superpimp.org");
-	Group * sendlater = folder_get_by_name (PAN_SENDLATER);
-	Article * a = article_new (sendlater);
+	gchar * from;
+	gchar * msg_id;
+	Group * sendlater;
+	Article * a;
+	debug_enter ("send_feedback_cb");
+
+       	from = get_default_author_from ();
+       	msg_id = gnksa_generate_message_id ("superpimp.org");
+       	sendlater = folder_get_by_name (PAN_SENDLATER);
+       	a = article_new (sendlater);
 
 	article_set_header (a, HEADER_SUBJECT, "Pan " VERSION " Feedback", DO_CHUNK);
 	article_set_header (a, PAN_MAIL_TO, "pan@superpimp.org", DO_CHUNK);
-	article_set_header (a, HEADER_FROM, from, DO_CHUNK);
 	article_set_header (a, HEADER_MESSAGE_ID, msg_id, DO_CHUNK);
+	article_set_author_from_header (a, from);
 	message_edit_window (NULL, a);      
 
 	g_free (msg_id);
 	g_free (from);
+	debug_exit ("send_feedback_cb");
 }
 
 static void
@@ -652,11 +726,14 @@ static GnomeUIInfo article_sort_menu[] =
 	GNOMEUIINFO_END
 };
 
+static gboolean filter_dampen_feedback_loop = FALSE;
+
 static void
 article_filter_cb (GtkCheckMenuItem* item, gpointer data)
 {
 	const guint flag = GPOINTER_TO_UINT(data);
-	articlelist_poke_state_filter (flag, item->active);
+	if (!filter_dampen_feedback_loop)
+		articlelist_poke_state_filter (flag, item->active);
 }
 
 static GnomeUIInfo article_filter_menu[] =
@@ -751,6 +828,25 @@ static GnomeUIInfo article_filter_menu[] =
 	GNOMEUIINFO_END
 };
 
+static gint
+log_button_clicked (gpointer menu, gpointer data)
+{
+	GtkWidget * old_child;
+	GtkWidget * new_child = status_ok_label;
+
+	pan_lock ();
+	if (((old_child = GTK_BIN(error_button)->child)) != new_child) {
+		gtk_container_remove (GTK_CONTAINER(error_button), old_child);
+		gtk_container_add (GTK_CONTAINER(error_button), new_child);
+		gtk_widget_show (new_child);
+	}
+	pan_unlock ();
+
+	dialog_log_viewer ();
+
+	return 0;
+}
+
 static void
 zoom_page_cb (gpointer menu, gpointer data)
 {
@@ -837,7 +933,7 @@ static GnomeUIInfo view_menu[] =
 		thread_articles_cb, NULL,
 		NULL,
 		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BLANK,
-		'\0', 0, NULL
+		'T', 0, NULL
 	},
 	{
 		/* 9 */
@@ -903,7 +999,7 @@ static GnomeUIInfo view_menu[] =
 		GNOME_APP_UI_ITEM,
 		N_("_Log Viewer"),
 		N_("Open up the Log Viewer."),
-		dialog_log_viewer, NULL, NULL,
+		log_button_clicked, NULL, NULL,
 		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP,
 		'L', GDK_CONTROL_MASK, NULL
 	},
@@ -1045,12 +1141,6 @@ GnomeUIInfo group_menu [] =
 		N_("Remove all articles from the selected group(s)."),
 		grouplist_selected_empty
 	},
-	{
-		GNOME_APP_UI_ITEM,
-		N_("Rename..."),
-		N_("Rename the selected group."),
-		grouplist_selected_rename
-	},
 	{
 		GNOME_APP_UI_ITEM,
 		N_("_Properties..."),
@@ -1070,12 +1160,6 @@ GnomeUIInfo folder_menu [] =
 		N_("Create a new folder."),
 		group_new_folder_dialog
 	},
-	{
-		GNOME_APP_UI_ITEM,
-		N_("Rename Folder..."),
-		N_("Rename the selected folder."),
-		grouplist_selected_rename
-	},
 	{
 		GNOME_APP_UI_ITEM,
 		N_("Delete Folder..."),
@@ -1183,7 +1267,7 @@ gui_create_appbar (GtkWidget* window)
 	gtk_container_add (GTK_CONTAINER(frame), w);
 	gtk_box_pack_start (GTK_BOX(appbar), frame, FALSE, FALSE, 0);
 
-	/* remainder */
+	/* status item views */
 	taskbar = gtk_table_new (1, VIEW_QTY, TRUE);
 	for (i=0; i<VIEW_QTY; ++i) {
 		GtkWidget * w = status_item_view_new ();
@@ -1192,6 +1276,18 @@ gui_create_appbar (GtkWidget* window)
 	}
 	gtk_box_pack_start (GTK_BOX(appbar), taskbar, TRUE, TRUE, 0);
 
+	/* error button */
+	status_ok_label = gnome_pixmap_new_from_xpm_d (inform_xpm);
+	status_error_label = gnome_pixmap_new_from_xpm_d (caution_xpm);
+	gtk_object_ref (GTK_OBJECT(status_ok_label));
+	gtk_object_ref (GTK_OBJECT(status_error_label));
+	error_button = w = gtk_button_new ();
+	gtk_container_add (GTK_CONTAINER(w), status_ok_label);
+	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("Open the Status Log"), "");
+	gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
+	gtk_signal_connect (GTK_OBJECT(w), "clicked", (GtkSignalFunc)log_button_clicked, NULL);
+	gtk_box_pack_start (GTK_BOX(appbar), w, FALSE, FALSE, 0);
+
 	gnome_app_set_statusbar (GNOME_APP(window), appbar);
 	pan_unlock();
 
@@ -1279,23 +1375,54 @@ text_fill_body_changed_cb (gpointer call_obj,
 	return 0;
 }
 
+static int
+log_list_appended_cb (gpointer call_obj,
+                      gpointer call_arg,
+                      gpointer user_data)
+{
+	LogEntry * e = (LogEntry*) call_obj;
+	if (e->severity== LOG_ERROR)
+	{
+		GtkWidget * old_child;
+		GtkWidget * new_child = status_error_label;
+		pan_lock ();
+		if (((old_child = GTK_BIN(error_button)->child)) != new_child) {
+			gtk_container_remove (GTK_CONTAINER(error_button), old_child);
+			gtk_container_add (GTK_CONTAINER(error_button), new_child);
+			gtk_widget_show (new_child);
+		}
+		pan_unlock ();
+	}
+	return 0;
+}
 
 void
 gui_shutdown (void)
 {
-	gint x = 0, y = 0;
-	gint width = 640, height = 400;
+	gint i;
+	gchar * pch;
+	GtkWidget * w[3];
+	debug_enter ("gui_shutdown");
 
 	/**
 	***  Stop updating the UI
 	**/
 
-	pan_callback_remove (text_get_fill_body_changed_callback(),
-	                     text_fill_body_changed_cb,
-	                     NULL);
 	pan_callback_remove (status_item_get_active_callback(),
 	                     status_item_active_changed_cb,
 	                     NULL);
+	turn_off_status_item_views ();
+
+	/**
+	***  Stop updating the rest of the UI
+	**/
+
+	pan_callback_remove (log_get_entry_added_callback(),
+	                     log_list_appended_cb,
+			     NULL);
+	pan_callback_remove (text_get_fill_body_changed_callback(),
+	                     text_fill_body_changed_cb,
+	                     NULL);
 	pan_callback_remove (articlelist_state_filter_changed,
 	                     gui_message_state_filter_changed_cb,
 	                     NULL);
@@ -1309,11 +1436,11 @@ gui_shutdown (void)
 	                     update_menus_cb,
 	                     NULL);
 	pan_callback_remove (grouplist_group_selection_changed,
-	                     update_menus_cb,
-	                     NULL);
+	                     update_menus_cb, NULL);
+	pan_callback_remove (grouplist_server_set_callback,
+	                     update_menus_cb, NULL);
 	pan_callback_remove (articlelist_group_changed,
-	                     update_menus_cb,
-	                     NULL);
+	                     update_menus_cb, NULL);
 	pan_callback_remove (articlelist_selection_changed,
 	                     update_menus_cb,
 	                     NULL);
@@ -1330,12 +1457,9 @@ gui_shutdown (void)
 	gui_refresh_timeout_id = 0;
 
 	/**
-	***  Save window/clist positioning
+	***  Save settings
 	**/
 
-	gdk_window_get_position (Pan.window->window, &x, &y);
-	gdk_window_get_size (Pan.window->window, &width, &height);
-
 	gnome_config_set_bool ("/Pan/General/do_wrap",
 	                       text_get_wrap());
 	gnome_config_set_bool ("/Pan/Display/Show_All_Headers_In_Body",
@@ -1343,23 +1467,52 @@ gui_shutdown (void)
 	gnome_config_set_bool ("/Pan/Display/Mute_Quoted_text",
 	                       text_get_mute_quoted());
 
-	gnome_config_set_int ("/Pan/Geometry/pan_x", x);
-	gnome_config_set_int ("/Pan/Geometry/pan_y", y);
-	gnome_config_set_int ("/Pan/Geometry/width", width);
-	gnome_config_set_int ("/Pan/Geometry/height", height);
 	gnome_config_set_int ("/Pan/State/viewmode", Pan.viewmode);
-	gnome_config_set_int ("/Pan/State/group_mode", grouplist_get_view_mode());
+	grouplist_shutdown_module ();
 
-	if (Pan.viewmode == GUI_NOTEBOOK) {
-		gnome_config_set_int ("/Pan/State/page", gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)));
-	}
-	else if (Pan.viewmode == GUI_PANED) {
-		gnome_config_set_int ("/Pan/Geometry/vpaned", articlelist_ctree->allocation.height);
-		gnome_config_set_int ("/Pan/Geometry/hpaned", groups_vbox->allocation.width);
+	switch (Pan.viewmode)
+	{
+		case GUI_NOTEBOOK:
+			gnome_config_set_int ("/Pan/State/page", gtk_notebook_get_current_page (GTK_NOTEBOOK(notebook)));
+			break;
+
+		case GUI_PANED:
+			pch = layout_str + 1;
+			for (i=0; i<=2; ++i, ++pch)
+			{
+				if (tolower(*pch)=='g')
+					w[i] = groups_vbox;
+				else if (tolower(*pch)=='t')
+					w[i] = articlelist_ctree;
+				else
+					w[i] = text_box;
+			}
+			switch(*layout_str)
+			{
+				case '2':
+					gnome_config_set_int ("/Pan/Geometry/vpaned", w[0]->allocation.height);
+					gnome_config_set_int ("/Pan/Geometry/hpaned", w[1]->allocation.width);
+					break;
+				case '4':
+					gnome_config_set_int ("/Pan/Geometry/vpaned", w[1]->allocation.height);
+					gnome_config_set_int ("/Pan/Geometry/hpaned", w[0]->allocation.width);
+					break;
+				case '5':
+					gnome_config_set_int ("/Pan/Geometry/vpaned", w[0]->allocation.height);
+					gnome_config_set_int ("/Pan/Geometry/hpaned", w[1]->allocation.height);
+					break;
+				case '3':
+				case '1':
+				default:
+					gnome_config_set_int ("/Pan/Geometry/vpaned", w[0]->allocation.height);
+					gnome_config_set_int ("/Pan/Geometry/hpaned", w[0]->allocation.width);
+					break;
+			}
 	}
 
 	gui_save_column_widths (Pan.group_clist, "group");
-	gui_save_column_widths (Pan.article_ctree, "articlelist");
+	gui_save_column_widths (Pan.article_ctree, "threads");
+	gui_save_window_size (Pan.window, "main_window");
 	gnome_config_sync();
 
 	/**
@@ -1368,6 +1521,7 @@ gui_shutdown (void)
 
 	articlelist_set_group (NULL);
 	set_current_article (NULL);       
+	debug_exit ("gui_shutdown");
 }
 
 
@@ -1386,6 +1540,7 @@ update_menus_idle (gpointer data)
 	const Group * sendlater;
 	gboolean sendlater_empty;
 	gboolean posted_here;
+	debug_enter ("update_menus_idle");
 
 	article = articlelist_get_selected_article ();
 	have_article = article!=NULL;
@@ -1461,7 +1616,7 @@ update_menus_idle (gpointer data)
 	menu = group_menu;
 	i = -1;
 	++i; /* i is download dialog */
-	gtk_widget_set_sensitive (menu[i].widget, TRUE);
+	gtk_widget_set_sensitive (menu[i].widget, have_group);
 	++i; /* i is separator */
 	++i; /* i is new headers for subscribed */
 	gtk_widget_set_sensitive (menu[i].widget, TRUE);
@@ -1495,8 +1650,6 @@ update_menus_idle (gpointer data)
 	gtk_widget_set_sensitive (menu[i].widget, have_group);
 	++i; /* i is clear group */
 	gtk_widget_set_sensitive (menu[i].widget, have_group);
-	++i; /* i is rename group */
-	gtk_widget_set_sensitive (menu[i].widget, have_group);
 	++i; /* i is group properties */
 	gtk_widget_set_sensitive (menu[i].widget, have_group);
 	
@@ -1504,8 +1657,6 @@ update_menus_idle (gpointer data)
 	i = -1;
 	++i; /* i is new folder */
 	gtk_widget_set_sensitive (menu[i].widget, TRUE);
-	++i; /* i is rename folder or group */
-	gtk_widget_set_sensitive (menu[i].widget, have_group);
 	++i; /* i is delete folder */
 	gtk_widget_set_sensitive (menu[i].widget, have_folder);
 
@@ -1538,7 +1689,7 @@ update_menus_idle (gpointer data)
 	/* update file menu */
 	menu = file_menu;
 	i = -1;
-	++i; /* save article */
+	++i; /* save article as */
 	gtk_widget_set_sensitive (menu[i].widget, have_article);
 	++i; /* save binary attachment */
 	gtk_widget_set_sensitive (menu[i].widget, have_article);
@@ -1547,16 +1698,21 @@ update_menus_idle (gpointer data)
 	++i; /* open binary attachment */
 	gtk_widget_set_sensitive (menu[i].widget, have_article);
 	++i; /* separator */
+	++i; /* save setup */
+	gtk_widget_set_sensitive (menu[i].widget, TRUE);
+	++i; /* separator */
 	++i; /* print article */
 	gtk_widget_set_sensitive (menu[i].widget, have_article);
 	++i; /* separator */
 	++i; /* exit */
 
+
 	pan_unlock_unconditional ();
 
 	/* update the title */
 	gui_set_title (alist_group);
 
+	debug_exit ("update_menus_idle");
 	return 0;
 }
 static int
@@ -1604,6 +1760,7 @@ gui_articlelist_sort_changed_cb (
 	int sort = GPOINTER_TO_INT(call_arg);
 	int i = 0;
 	GtkRadioMenuItem* w = NULL;
+	debug_enter ("gui_articlelist_sort_changed_cb");
 
 	switch (sort) {
 		case ARTICLE_SORT_AUTHOR: i=0; break;
@@ -1624,6 +1781,7 @@ gui_articlelist_sort_changed_cb (
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(w), TRUE);
 	dampen_feedback_sort = FALSE;
 
+	debug_exit ("gui_articlelist_sort_changed_cb");
 	return 0;
 }
 
@@ -1632,51 +1790,40 @@ static gint
 gui_message_state_filter_changed_idle (gpointer data)
 {
 	guint filter = GPOINTER_TO_UINT (data);
+	debug_enter ("gui_message_state_filter_changed_idle");
 
 	pan_lock_unconditional();
+	filter_dampen_feedback_loop = TRUE;
 
-	gtk_check_menu_item_set_active (
-		GTK_CHECK_MENU_ITEM(article_filter_menu[0].widget),
-		filter & STATE_FILTER_NEW);
-	gtk_check_menu_item_set_active (
-		GTK_CHECK_MENU_ITEM(article_filter_menu[1].widget),
-		filter & STATE_FILTER_UNREAD);
-	gtk_check_menu_item_set_active (
-		GTK_CHECK_MENU_ITEM(article_filter_menu[2].widget),
-		filter & STATE_FILTER_READ);
-
-	gtk_check_menu_item_set_active (
-		GTK_CHECK_MENU_ITEM(article_filter_menu[4].widget),
-		filter & STATE_FILTER_WATCHED);
-	gtk_check_menu_item_set_active (
-		GTK_CHECK_MENU_ITEM(article_filter_menu[5].widget),
-		filter & STATE_FILTER_IGNORED);
-	gtk_check_menu_item_set_active (
-		GTK_CHECK_MENU_ITEM(article_filter_menu[6].widget),
-		filter & STATE_FILTER_NORMAL_RANK);
-
-	gtk_check_menu_item_set_active (
-		GTK_CHECK_MENU_ITEM(article_filter_menu[8].widget),
-		filter & STATE_FILTER_COMPLETE_BINARIES);
-	gtk_check_menu_item_set_active (
-		GTK_CHECK_MENU_ITEM(article_filter_menu[9].widget),
-		filter & STATE_FILTER_INCOMPLETE_BINARIES);
-	gtk_check_menu_item_set_active (
-		GTK_CHECK_MENU_ITEM(article_filter_menu[10].widget),
-		filter & STATE_FILTER_NONBINARIES);
-
-	gtk_check_menu_item_set_active (
-		GTK_CHECK_MENU_ITEM(article_filter_menu[12].widget),
-		filter & STATE_FILTER_SAVED);
-	gtk_check_menu_item_set_active (
-		GTK_CHECK_MENU_ITEM(article_filter_menu[13].widget),
-		filter & STATE_FILTER_QUEUED);
-	gtk_check_menu_item_set_active (
-		GTK_CHECK_MENU_ITEM(article_filter_menu[14].widget),
-		filter & STATE_FILTER_IDLE);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(article_filter_menu[0].widget),
+	                                filter & STATE_FILTER_NEW);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(article_filter_menu[1].widget),
+	                                filter & STATE_FILTER_UNREAD);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(article_filter_menu[2].widget),
+	                                filter & STATE_FILTER_READ);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(article_filter_menu[4].widget),
+	                                filter & STATE_FILTER_WATCHED);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(article_filter_menu[5].widget),
+	                                filter & STATE_FILTER_IGNORED);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(article_filter_menu[6].widget),
+	                                filter & STATE_FILTER_NORMAL_RANK);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(article_filter_menu[8].widget),
+	                                filter & STATE_FILTER_COMPLETE_BINARIES);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(article_filter_menu[9].widget),
+	                                filter & STATE_FILTER_INCOMPLETE_BINARIES);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(article_filter_menu[10].widget),
+	                                filter & STATE_FILTER_NONBINARIES);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(article_filter_menu[12].widget),
+	                                filter & STATE_FILTER_SAVED);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(article_filter_menu[13].widget),
+	                                filter & STATE_FILTER_QUEUED);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(article_filter_menu[14].widget),
+	                                filter & STATE_FILTER_IDLE);
 
+	filter_dampen_feedback_loop = FALSE;
 	pan_unlock_unconditional ();
 
+	debug_exit ("gui_message_state_filter_changed_idle");
 	return 0;
 }
 static gint
@@ -1700,6 +1847,7 @@ void
 gui_construct (const gchar * geometry)
 {
 	GtkWidget * w;
+	debug_enter ("gui_construct");
 
 	memset (views, '\0', sizeof(ViewStruct)*VIEW_QTY);
 	ttips = gtk_tooltips_new();
@@ -1711,49 +1859,30 @@ gui_construct (const gchar * geometry)
 	gtk_signal_connect (GTK_OBJECT(Pan.window), "destroy",
 			    GTK_SIGNAL_FUNC (pan_shutdown), NULL);
 
-	gtk_widget_realize (Pan.window);
-
 	/** geometry parsing **/
 	/* The following chunk of code will allow command line arguments
 	 * to override the config. So if this was invoked as part of a 
 	 * session we 'Do the right thing' :)
 	 */
-	if (geometry != NULL)
-	  {
-	    gint x, y, w, h;
-	    if (gnome_parse_geometry (geometry, &x, &y, &w, &h) )
-	      {
-		if (x != -1)
-		  gtk_widget_set_uposition (Pan.window, x, y);
-		if (w != -1)
-		  gtk_window_set_default_size (GTK_WINDOW (Pan.window), w, h);
-	      }
-	    else
-	      {
-		g_error (_("Unable to parse the geometry string \"%s\""), geometry);
-	      }
-	  }
+	if (geometry == NULL)
+	{
+		gui_restore_window_size (Pan.window, "main_window");
+	}
 	else
-	  {                  /* Lets see if the config has a geometry */
-	    gint w, h;
-
-	    w = gnome_config_get_int ("/Pan/Geometry/width");
-	    h = gnome_config_get_int ("/Pan/Geometry/height");
-	    if (w!=0 && h!= 0)
-		    gtk_window_set_default_size (GTK_WINDOW (Pan.window), w, h);
-	    else
-		    gtk_window_set_default_size (GTK_WINDOW (Pan.window), 640, 400);
-
-/* While this is nice for some people, it's typically much better to let
- * the window manager do the position saving and restoring... we have similar
- * code at the bottom of this function...
-
-	    x = gnome_config_get_int ("/Pan/Geometry/pan_x");
-	    y = gnome_config_get_int ("/Pan/Geometry/pan_y");
-	    if (x != 0 && y != 0)
-	      gtk_widget_set_uposition (Pan.window, x, y);
-*/
-	  }
+	{
+		gint x, y, w, h;
+		if (gnome_parse_geometry (geometry, &x, &y, &w, &h) )
+		{
+			if (x != -1)
+				gtk_widget_set_uposition (Pan.window, x, y);
+			if (w != -1)
+				gtk_window_set_default_size (GTK_WINDOW (Pan.window), w, h);
+		}
+		else
+		{
+			g_error (_("Unable to parse the geometry string \"%s\""), geometry);
+		}
+	}
 
 /* Groups vbox */
 	groups_vbox = gtk_vbox_new (FALSE, 0);
@@ -1786,6 +1915,10 @@ gui_construct (const gchar * geometry)
 
         gui_set_layout (gnome_config_get_int ( "Pan/State/viewmode=1" ));
 
+	pan_callback_add (log_get_entry_added_callback(),
+	                  log_list_appended_cb,
+			  NULL);
+
 	pan_callback_add (status_item_get_active_callback(),
 	                  status_item_active_changed_cb,
 	                  NULL);
@@ -1807,14 +1940,14 @@ gui_construct (const gchar * geometry)
 	                  show_all_headers_changed_cb,
 	                  NULL);
 	pan_callback_add (text_get_mute_quoted_changed_callback(),
-	                  mute_quoted_text_changed_cb,
-	                  NULL);
+	                  mute_quoted_text_changed_cb, NULL);
 	pan_callback_add (current_article_changed,
-			  update_menus_cb,
-	                  NULL);
+			  update_menus_cb, NULL);
 	pan_callback_add (grouplist_group_selection_changed,
-			  update_menus_cb,
-			  NULL);
+			  update_menus_cb, NULL);
+	pan_callback_add (grouplist_server_set_callback,
+	                  update_menus_cb,
+	                  NULL);
 	pan_callback_add (articlelist_group_changed,
 			  update_menus_cb,
 			  NULL);
@@ -1848,6 +1981,7 @@ gui_construct (const gchar * geometry)
 	gui_set_title (NULL);
 
 	gui_refresh_timeout_id = gtk_timeout_add (2192, gui_refresh_timeout_cb, NULL); 
+	debug_exit ("gui_construct");
 }
 
 
@@ -1897,6 +2031,7 @@ static void
 gui_page_change (int change)
 {
 	int cur_page;
+	debug_enter ("gui_page_change");
 
 	if (Pan.viewmode == GUI_PANED)
 		cur_page = gui_paned_get_current_pane ();
@@ -1983,6 +2118,8 @@ gui_page_change (int change)
 
 	if (Pan.viewmode == GUI_NOTEBOOK)
 		gtk_notebook_set_page (GTK_NOTEBOOK (notebook), cur_page);
+
+	debug_exit ("gui_page_change");
 }
 
 static void
@@ -2008,6 +2145,7 @@ gui_set_title_idle (gpointer data)
 	GString * s = g_string_new (NULL);
 	const gchar * servername = NULL;
 	const gchar * groupname = NULL;
+	debug_enter ("gui_set_title_idle");
 
 	/* look for active server & group */
 	if (group == NULL) {
@@ -2034,6 +2172,7 @@ gui_set_title_idle (gpointer data)
 	/* cleanup */
 	g_string_free (s, TRUE);
 
+	debug_exit ("gui_set_title_idle");
 	return 0;
 }
 void
@@ -2063,7 +2202,10 @@ gui_key_press_cb (GtkWidget      * widget,
 			break;
 
 		case GDK_Delete:
-			articlelist_selected_delete ();
+			if (event->state & GDK_SHIFT_MASK)
+				articlelist_selected_delete_thread ();
+			else
+				articlelist_selected_delete ();
 			break;
 
 		case GDK_Return:
@@ -2082,13 +2224,10 @@ gui_key_press_cb (GtkWidget      * widget,
 	return TRUE;
 }
 
+/**
+***
+**/
 
-/*---[ gui_restore_column_widths ]------------------------------------
- * @clist:
- * @type:
- *
- * 
- *--------------------------------------------------------------------*/
 void
 gui_restore_column_widths (GtkWidget     * clist,
                            const gchar   * type)
@@ -2098,6 +2237,7 @@ gui_restore_column_widths (GtkWidget     * clist,
 	const int cols = list->columns;
 	int * widths = g_new (int, cols);
 	GString * buf = g_string_new (NULL);
+	debug_enter ("gui_restore_column_widths");
 
 	/* get width from config... */
 	for (i=0; i!=cols; ++i) {
@@ -2115,6 +2255,7 @@ gui_restore_column_widths (GtkWidget     * clist,
 	/* cleanup */
 	g_string_free (buf, TRUE);
 	g_free (widths);
+	debug_exit ("gui_restore_column_widths");
 }
 
 void
@@ -2136,6 +2277,74 @@ gui_save_column_widths (GtkWidget    * clist,
 	gnome_config_sync ();
 }
 
+/**
+***
+**/
+
+void
+gui_restore_window_size (GtkWidget     * widget,
+                         const gchar   * key)
+{
+	gint x, y, w, h;
+	GString * buf;
+	gint screen_w = gdk_screen_width ();
+	gint screen_h = gdk_screen_height ();
+	const gint fuzz = 10;
+
+	/* sanity clause */
+	g_return_if_fail (widget!=NULL);
+	g_return_if_fail (!GTK_WIDGET_REALIZED(widget));
+	g_return_if_fail (is_nonempty_string(key));
+
+	/* get the dimensions */
+	buf = g_string_new (NULL);
+	g_string_sprintf (buf, "/Pan/Geometry/%s_x", key);
+	x = gnome_config_get_int (buf->str);
+	g_string_sprintf (buf, "/Pan/Geometry/%s_y", key);
+	y = gnome_config_get_int (buf->str);
+	g_string_sprintf (buf, "/Pan/Geometry/%s_width", key);
+	w = gnome_config_get_int (buf->str);
+	g_string_sprintf (buf, "/Pan/Geometry/%s_height", key);
+	h = gnome_config_get_int (buf->str);
+
+	/* move & resize */
+	if ((0<x && x<screen_w-fuzz) && (0<y && y<screen_h-fuzz))
+		gtk_widget_set_uposition (widget, x, y);
+	if (w>0 && h>0)
+		gtk_window_set_default_size (GTK_WINDOW(widget), w, h);
+
+	/* cleanup */
+	g_string_free (buf, TRUE);
+}
+ 
+void
+gui_save_window_size (GtkWidget     * widget,
+                      const gchar   * key)
+{
+	GString * buf;
+	gint x, y, w, h;
+
+	g_return_if_fail (widget!=NULL);
+	g_return_if_fail (widget->window!=NULL);
+	g_return_if_fail (is_nonempty_string(key));
+
+	buf = g_string_new (NULL);
+	gdk_window_get_position (widget->window, &x, &y);
+	gdk_window_get_size (widget->window, &w, &h);
+
+	g_string_sprintf (buf, "/Pan/Geometry/%s_x", key);
+	gnome_config_set_int (buf->str, x);
+	g_string_sprintf (buf, "/Pan/Geometry/%s_y", key);
+	gnome_config_set_int (buf->str, y);
+	g_string_sprintf (buf, "/Pan/Geometry/%s_width", key);
+	gnome_config_set_int (buf->str, w);
+	g_string_sprintf (buf, "/Pan/Geometry/%s_height", key);
+	gnome_config_set_int (buf->str, h);
+
+	g_string_free (buf, TRUE);
+	gnome_config_sync ();
+}
+
 /*****
 ******
 *****/
@@ -2144,8 +2353,8 @@ static int connection_size = -1;
 
 static gboolean _is_connecting = FALSE;
 
-static void
-gui_refresh_connection_label_nolock (void)
+static gint
+gui_refresh_connection_label_idle (gpointer data)
 {
 	char str[128];
 
@@ -2156,8 +2365,12 @@ gui_refresh_connection_label_nolock (void)
 	else
 		sprintf (str, _("Offline"));
 
+	pan_lock_unconditional ();
 	if (GTK_IS_LABEL(connection_qty_label))
 		gtk_label_set_text (GTK_LABEL(connection_qty_label), str);
+	pan_unlock_unconditional ();
+
+	return 0;
 }
 
 void
@@ -2168,7 +2381,7 @@ gui_set_connecting_flag (gboolean is_connecting)
 		_is_connecting = is_connecting;
 
 		pan_lock ();
-		gui_refresh_connection_label_nolock ();
+		gtk_idle_add (gui_refresh_connection_label_idle, NULL);
 		pan_unlock ();
 	}
 }
@@ -2179,9 +2392,10 @@ gui_set_connection_size (int size)
 	if (size != connection_size)
 	{
 		connection_size = size;
+		log_add_va (LOG_INFO, _("News server connection count: %d"), size);
 
 		pan_lock ();
-		gui_refresh_connection_label_nolock ();
+		gtk_idle_add (gui_refresh_connection_label_idle, NULL);
 		pan_unlock ();
 	}
 }
@@ -2229,7 +2443,7 @@ gui_refresh_timeout_cb (gpointer user_data)
 	}
 
 	pan_lock_unconditional ();
-	gui_refresh_connection_label_nolock ();
+	gtk_idle_add (gui_refresh_connection_label_idle, NULL);
 	pan_unlock_unconditional ();
 
 	return 1;
@@ -2258,6 +2472,7 @@ static void
 gui_add_status_item (StatusItem * item)
 {
 	int i;
+	debug_enter ("gui_add_status_item");
 
 	pan_lock ();
 
@@ -2270,12 +2485,14 @@ gui_add_status_item (StatusItem * item)
 	}
 
 	pan_unlock ();
+	debug_exit ("gui_add_status_item");
 }
 
 static void
 gui_remove_status_item (StatusItem * item)
 {
 	int i;
+	debug_enter ("gui_remove_status_item");
 
 	pan_lock ();
 
@@ -2289,6 +2506,27 @@ gui_remove_status_item (StatusItem * item)
 	}
 
 	pan_unlock();
+	debug_exit ("gui_remove_status_item");
+}
+
+static void
+turn_off_status_item_views (void)
+{
+	gint i;
+	debug_enter ("turn_off_status_item_views");
+
+	pan_lock ();
+	for (i=0; i<VIEW_QTY; ++i) {
+		StatusItem * item = views[i].status_item;
+		StatusItemView * view = views[i].view;
+		if (item != NULL) {
+			status_item_view_set_item (STATUS_ITEM_VIEW(view), NULL);
+			views[i].status_item = NULL;
+		}
+	}
+	pan_unlock ();
+
+	debug_exit ("turn_off_status_item_views");
 }
 
 static int
@@ -2298,10 +2536,14 @@ status_item_active_changed_cb (gpointer call_obj,
 {
 	StatusItem * item = STATUS_ITEM(call_obj);
 	gboolean is_active = GPOINTER_TO_INT(call_arg) != 0;
+	debug_enter ("status_item_active_changed_cb");
+
 	if (is_active)
 		gui_add_status_item (item);
 	else
 		gui_remove_status_item (item);
+
+	debug_exit ("status_item_active_changed_cb");
 	return 0;
 }
 
