@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -63,9 +63,13 @@ void gui_page_set (int page, GtkWidget *focus_item);
 
 void gui_set_title (const Group*);
 
-void gui_restore_column_widths (GtkWidget * clist, const gchar * type);
+void gui_restore_column_widths    (GtkWidget * clist, const gchar * key);
 
-void gui_save_column_widths (GtkWidget * clist, const gchar * type);
+void gui_save_column_widths       (GtkWidget * clist, const gchar * key);
+
+void gui_restore_window_size      (GtkWidget * window, const gchar * key);
+
+void gui_save_window_size         (GtkWidget * window, const gchar * key);
 
 void gui_select_all (void);
 
