@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,8 +21,41 @@
 #ifndef __LOG_H__
 #define __LOG_H__
 
-extern void log_add    (const char *);
+#include <time.h>
+#include <glib.h>
+#include "pan-callback.h"
+
+void log_init (void);
+
+void log_shutdown_module (void);
+
+typedef enum
+{
+	LOG_ERROR,
+	LOG_IMPORTANT,
+	LOG_INFO
+}
+LogSeverity;
+
+void log_add    (LogSeverity, const gchar *);
+
+void log_add_va (LogSeverity, const gchar *, ...);
+
+typedef struct
+{
+	time_t date;
+	gchar * message;
+	LogSeverity severity;
+}
+LogEntry;
+
+GPtrArray * log_get_entries (void);
+
+void log_clear (void);
+
+PanCallback* log_get_entry_added_callback (void);
+
+PanCallback* log_get_entry_list_changed_callback (void);
 
-extern void log_add_va (const char *, ...);
 
 #endif /* __LOG_H__ */
