@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 
 #include <gmime/gmime-utils.h>
 
+#include "debug.h"
 #include "globals.h"
 #include "gnksa.h"
 #include "grouplist.h"
@@ -66,14 +67,15 @@ static gint send_article_now (StatusItem*, PanSocket*, Server*, SendStruct*);
 ***/
 
 static GString *
-build_nntp_header (GString *postme,
-                   Article * article,
-                   Server * server,
-                   const gchar * subject_append,
-		   gboolean force_new_message_id)
+build_nntp_header (GString        * postme,
+                   Article        * article,
+                   Server         * server,
+                   const gchar    * subject_append,
+		   gboolean         force_new_message_id)
 {
 	gchar * tmp;
 	const gchar * pch;
+	debug_enter ("build_nntp_header");
 
 	/* From: */
 	tmp = article_get_author_str (article);
@@ -150,7 +152,7 @@ build_nntp_header (GString *postme,
 	/* Mime -- this hardcoding will go away soon; this is a stopgap */
 	g_string_append (postme, "Mime-Version: 1.0\r\n");
 	g_string_append (postme, "Content-Type: text/plain; charset=ISO-8859-1\r\n");
-	g_string_append (postme, "Content-Tranfer-Encoding: 8bit\r\n");
+	g_string_append (postme, "Content-Transfer-Encoding: 8bit\r\n");
 
 	/* Extra headers: */
 	if (1) {
@@ -175,6 +177,7 @@ build_nntp_header (GString *postme,
 	/* separator that goes between headers and body */
 	g_string_sprintfa (postme, "\r\n");
 
+	debug_exit ("build_nntp_header");
 	return (postme);
 }
 
@@ -187,15 +190,17 @@ build_nntp_header (GString *postme,
 static GPtrArray*
 build_nntp_message (Server * server, Article * article)
 {
-	GPtrArray * bodies = g_ptr_array_new();
+	GPtrArray * bodies;
 	gchar * tmp; /* *append; */
 	GString * body;
 	GString * postme;
+	debug_enter ("build_nntp_message");
 
 	/* sanity checks... */
 	g_return_val_if_fail (article!=NULL, NULL);
 	g_return_val_if_fail (article_get_header(article,HEADER_NEWSGROUPS), 0);
 
+       	bodies = g_ptr_array_new();
 #if 0
 	if (article_has_attachment (article))
 	{
@@ -295,6 +300,7 @@ build_nntp_message (Server * server, Article * article)
 #if 0
 	}
 #endif
+	debug_exit ("build_nntp_message");
 	return bodies;
 }
 
@@ -330,19 +336,28 @@ task_send_describe (const StatusItem * item)
 static void
 task_send_dtor (gpointer data)
 {
-	TaskFuncPtr * task = (TaskFuncPtr*) data;
-	SendStruct * send = (SendStruct*) task->user_data;
+	TaskFuncPtr * task;
+	SendStruct * send;
+	debug_enter ("task_send_dtor");
 
+       	task = (TaskFuncPtr*) data;
+       	send = (SendStruct*) task->user_data;
 	group_unref_articles (send->original_group, NULL);
 	g_free (send);
+
+	debug_exit ("task_send_dtor");
 }
 
 static int
 task_send_run (TaskFuncPtr* task, gpointer user_data)
 {
-	SendStruct * data = (SendStruct*) user_data;
-	Article * article = data->article;
+	SendStruct * data;
+	Article * article;
 	gint status;
+	debug_enter ("task_send_run");
+
+       	data = (SendStruct*) user_data;
+       	article = data->article;
 
 	/* try to send the message */
 	status = send_article_now (STATUS_ITEM(task),
@@ -365,7 +380,7 @@ task_send_run (TaskFuncPtr* task, gpointer user_data)
 		if (text->len != 0) {
 			g_string_append (text, _("Your message was saved in the folder ``pan.sendlater''"));
 			pan_error_dialog (text->str);
-			g_message (text->str);
+			log_add (LOG_ERROR, text->str);
 		}
 
 		g_string_free (text, TRUE);
@@ -384,6 +399,7 @@ task_send_run (TaskFuncPtr* task, gpointer user_data)
 	 * the message in pan.sendlater anyway.  This way the user
 	 * can try to correct the problem before resending.
 	 */
+	debug_exit ("task_send_run");
 	return status==0 ? TASK_SUCCESS : TASK_FAIL_HOPELESS;
 }
 
@@ -399,10 +415,17 @@ send_article_now (StatusItem           * item,
                   SendStruct           * data)
 {
 	gint retval = 0;
-	Article * article = data->article;
-	const gchar * subject = article->subject;
-	const gchar * mail_to = data->do_mail ? article_get_header (article, PAN_MAIL_TO) : NULL;
-	const gchar * post_to = data->do_post ? article_get_header (article, HEADER_NEWSGROUPS) : NULL;
+	Article * article;
+	const gchar * subject;
+	const gchar * mail_to;
+	const gchar * post_to;
+	debug_enter ("send_article_now");
+
+	/* initialize variables */
+       	article = data->article;
+       	subject = article->subject;
+       	mail_to = data->do_mail ? article_get_header (article, PAN_MAIL_TO) : NULL;
+       	post_to = data->do_post ? article_get_header (article, HEADER_NEWSGROUPS) : NULL;
 
 	/* mail the message */
 	if (is_nonempty_string(mail_to))
@@ -422,12 +445,12 @@ send_article_now (StatusItem           * item,
 		/* mail the message */
 		if (!val)
 		{
-			log_add_va (_("E-mail Message \"%s\" sent."), subject);
+			log_add_va (LOG_IMPORTANT, _("E-mail Message \"%s\" sent."), subject);
 			data->do_mail = FALSE;
 		}
 		else
 		{
-			log_add_va (_("E-mail Message \"%s\" not sent."), subject);
+			log_add_va (LOG_ERROR, _("E-mail Message \"%s\" not sent."), subject);
 			retval |= SMTP_FAILED;
 		}
 
@@ -452,11 +475,11 @@ send_article_now (StatusItem           * item,
 
 			/* post the part */
 			if (nntp_post(item,sock,a) == TASK_SUCCESS) {
-				log_add_va (_("NNTP Article \"%s\" (%d of %d) posted."),
+				log_add_va (LOG_IMPORTANT, _("NNTP Article \"%s\" (%d of %d) posted."),
 					subject, i+1, post_me->len);
 				data->do_post = FALSE;
 			} else {
-				log_add_va (_("NNTP Article \"%s\" (%d of %d) not posted."),
+				log_add_va (LOG_ERROR, _("NNTP Article \"%s\" (%d of %d) not posted."),
 					subject, i+1, post_me->len);
 				retval |= NNTP_FAILED;
 			}
@@ -467,6 +490,7 @@ send_article_now (StatusItem           * item,
 		g_ptr_array_free (post_me, TRUE);
 	}
 
+	debug_exit ("send_article_now");
 	return retval;
 }
 
@@ -484,6 +508,7 @@ queue_article_for_posting (Server     * server,
 {
 	GPtrArray * errors;
 	GoodnessLevel goodness = OKAY;
+	debug_enter ("queue_article_for_posting");
 
 	/* sanity check */
 	g_return_val_if_fail (server!=NULL, 1);
@@ -522,6 +547,7 @@ queue_article_for_posting (Server     * server,
 	/* cleanup */
 	pan_g_ptr_array_foreach (errors, (GFunc)g_free, NULL);
 	g_ptr_array_free (errors, TRUE);
+	debug_exit ("queue_article_for_posting");
 	return goodness==OKAY ? 0 : 1;
 }
 
@@ -532,12 +558,15 @@ flush_sendlater_articles (void)
 	guint i;
         Group * folder;
 	GPtrArray * articles;
+	debug_enter ("flush_sendlater_articles");
 
+	/* find the sendlater folder */
        	folder = folder_get_by_name (PAN_SENDLATER);
 	g_return_if_fail (folder!=NULL);
 
+	/* load the articles for that group */
 	group_ref_articles (folder, NULL);
-	articles = group_get_articles (folder);
+	articles = group_get_article_array (folder);
 
 	for (i=0; i!=articles->len; ++i)
 	{
@@ -556,6 +585,8 @@ flush_sendlater_articles (void)
 		queue_article_for_posting (s, a, TRUE);
 	}
 
-	articles = NULL;
+	/* cleanup */
+	g_ptr_array_free (articles, TRUE);
 	group_unref_articles (folder, NULL);
+	debug_exit ("flush_sendlater_articles");
 }
