@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #include <unistd.h>
 
 #include "articlelist.h"
+#include "debug.h"
 #include "globals.h"
 #include "gnksa.h"
 #include "grouplist.h"
@@ -54,7 +55,6 @@ static void wrap_cb                        (GtkWidget*, MessageWindow*);
 static void save_cb                        (GtkWidget*, MessageWindow*);
 static void send_now_cb                    (GtkWidget*, MessageWindow*);
 static void send_later_cb                  (GtkWidget*, MessageWindow*);
-static void window_destroy_cb              (GtkWidget*, MessageWindow*);
 static void message_window_save_cb         (GtkWidget*, MessageWindow*);
 static void message_window_cut_cb          (GtkWidget*, MessageWindow*);
 static void message_window_copy_cb         (GtkWidget*, MessageWindow*);
@@ -71,6 +71,25 @@ static GtkWidget * extra_headers_page      (MessageWindow*);
 static GtkWidget * create_body_pane_nolock (MessageWindow*);
 /*static void message_edit_external (GtkWidget *widget, Message *msg);*/
 
+/***
+****  SHUTDOWN
+***/
+
+static gint
+window_delete_event_cb (GtkWidget * w, GdkEvent * e, gpointer data)
+{
+	MessageWindow * mw = (MessageWindow*) data;
+	gui_save_window_size (mw->window, "message_window");
+	return FALSE;
+}
+static void
+window_destroy_cb (GtkWidget * window, MessageWindow * mw)
+{
+	if (mw->article != NULL)
+		group_unref_articles (mw->article->group, NULL);
+	g_free (mw);
+}
+
 /***
 ****  EXPERIMENTAL WRAP CODE
 ***/
@@ -437,7 +456,7 @@ create_reply_body (Article * article)
 	g_free (pch);
 
 	/* wrap what we've got so far */
-	pch = fill_body (out->str);
+	pch = fill_body (out->str, wrap_column);
 	g_string_assign (out, pch);
 	g_free (pch);
 
@@ -512,8 +531,10 @@ static void
 rot13_cb (GtkWidget * widget, MessageWindow * mw)
 {
 	GtkEditable * e = GTK_EDITABLE(mw->body);
-	gint start = e->selection_start_pos;
-	gint end = e->selection_end_pos;
+	gint a = e->selection_start_pos;
+	gint b = e->selection_end_pos;
+	gint start = MIN (a, b);
+	gint end = MAX (a, b);
 
 	if (end > start)
 	{
@@ -536,7 +557,7 @@ wrap_cb (GtkWidget *widget, MessageWindow * mw)
 	pan_unlock ();
 
 	/* fill the text, and update the body */
-	new_body = fill_body (body);
+	new_body = fill_body (body, wrap_column);
 	g_free (body);
 	update_body_pane (mw->body, new_body, FALSE);
 	g_free (new_body);
@@ -1035,13 +1056,8 @@ populate_post_info_pane (MessageWindow *mw)
 	}
 	else /* a reply */
 	{
-		Group *  group = NULL;
 		gchar * p;
-
-		/* figure out which group to use */
-		if (a!=NULL) group = a->group;
-		if (group==NULL) group = articlelist_get_group ();
-		if (group==NULL) group = grouplist_get_selected_group ();
+		gboolean default_newsgroups = TRUE;
 
 		/**
 		***  BODY
@@ -1116,22 +1132,14 @@ populate_post_info_pane (MessageWindow *mw)
 				else /* explicit followup groups */
 				{
 					gtk_entry_set_text (GTK_ENTRY(mw->newsgroups), followup);
+					default_newsgroups = FALSE;
 				}
 			}
 			else if (is_nonempty_string(newsgroups)) /* explicit groups */
 			{
 				gtk_entry_set_text (GTK_ENTRY(mw->newsgroups), newsgroups);
+				default_newsgroups = FALSE;
 			}
-			else /* implicit group */
-			{
-				if (group!=NULL && !group_is_folder(group))
-					gtk_entry_set_text (GTK_ENTRY(mw->newsgroups), group->name);
-			}
-		}
-		else if (is_posting(mw->type))
-		{
-			if (group!=NULL && !group_is_folder(group))
-				gtk_entry_set_text (GTK_ENTRY(mw->newsgroups), group->name);
 		}
 
 		/* are we sending mail? */
@@ -1150,6 +1158,36 @@ populate_post_info_pane (MessageWindow *mw)
 				}
 			}
 		}
+
+		/* no explicit newsgroup specified for this post,
+		 * so let's guess where the user might want to post */
+		if (default_newsgroups && is_posting(mw->type))
+		{
+			GString * str = g_string_new (NULL);
+
+			if (a!=NULL && a->group!=NULL)
+				g_string_sprintfa (str, "%s,", a->group->name);
+			else {
+				gint i;
+				Group * thread_group = articlelist_get_group ();
+				GPtrArray * ggroups = grouplist_get_selected_groups ();
+				if (ggroups->len<2 && thread_group!=NULL)
+					g_string_sprintfa (str, "%s,", thread_group->name);
+				else for (i=0; i<ggroups->len; ++i) {
+					Group * g = GROUP(g_ptr_array_index(ggroups,i));
+					if (g!=NULL && !group_is_folder(g))
+						g_string_sprintfa (str, "%s,", g->name);
+				}
+				g_ptr_array_free (ggroups, TRUE);
+			}
+
+			if (str->len != 0) {
+				g_string_truncate (str, str->len-1); /* zotz last comma */
+				gtk_entry_set_text (GTK_ENTRY(mw->newsgroups), str->str);
+			}
+
+			g_string_free (str, TRUE);
+		}
 	}
 }
 
@@ -1294,7 +1332,8 @@ update_body_pane_nolock (GtkWidget    * body_widget,
 				g_string_erase (line, line->len-1, 1);
 
 			/* add linefeed */
-			g_string_append_c (line, '\n');
+			if (is_nonempty_string (march))
+				g_string_append_c (line, '\n');
 
 			/* insert this line */
 			gtk_text_insert (text, NULL, pFG, NULL, line->str, line->len);
@@ -1330,10 +1369,6 @@ message_post_window_create (MessageWindow *mw)
 
 	pan_lock();
 	mw->window = gnome_app_new ("Post", "New Message");
-	gtk_window_set_default_size (GTK_WINDOW (mw->window), 550, 410);
-	gtk_window_set_policy (GTK_WINDOW(mw->window), TRUE, TRUE, TRUE);
-	gtk_signal_connect (GTK_OBJECT(mw->window), "destroy",
-	                    GTK_SIGNAL_FUNC(window_destroy_cb), mw);
 	gnome_app_create_menus_with_data (GNOME_APP (mw->window),
 	                                  message_post_main_menu, mw);
 	gnome_app_create_toolbar_with_data (GNOME_APP (mw->window),
@@ -1417,14 +1452,10 @@ message_read_window_create (MessageWindow *mw)
 	mw->window = gnome_app_new ("Read", mw->article->subject);
 	vbox = gtk_vbox_new (FALSE, 0);
 	gnome_app_set_contents (GNOME_APP (mw->window), vbox);
-	gtk_window_set_default_size (GTK_WINDOW (mw->window), 550, 410);
-	gtk_window_set_policy (GTK_WINDOW(mw->window), TRUE, TRUE, TRUE);
 	gnome_app_create_menus_with_data (GNOME_APP(mw->window),
 	                                  message_read_main_menu, mw);
 	gnome_app_create_toolbar_with_data (GNOME_APP(mw->window),
 	                                    message_read_toolbar, mw);
-	gtk_signal_connect (GTK_OBJECT(mw->window), "destroy",
-	                    GTK_SIGNAL_FUNC(window_destroy_cb), mw);
 	pan_unlock();
 
 	/* create read info pane */
@@ -1583,10 +1614,6 @@ message_edit_window_create (MessageWindow *mw)
 
 	pan_lock ();
 	mw->window = gnome_app_new ("Edit", mw->article->subject);
-	gtk_window_set_default_size (GTK_WINDOW (mw->window), 550, 410);
-	gtk_window_set_policy (GTK_WINDOW(mw->window), TRUE, TRUE, TRUE);
-	gtk_signal_connect (GTK_OBJECT(mw->window), "destroy",
-	                    GTK_SIGNAL_FUNC(window_destroy_cb), mw);
 	gnome_app_create_menus_with_data (GNOME_APP(mw->window),
 	                                  message_edit_main_menu,
 	                                  mw);
@@ -1613,17 +1640,10 @@ message_reply_window_create (MessageWindow *mw)
 
 	pan_lock();
 	mw->window = gnome_app_new ("Reply", title);
-	gtk_window_set_default_size (GTK_WINDOW(mw->window), 550, 410);
-	gtk_window_set_policy (GTK_WINDOW(mw->window), TRUE, TRUE, TRUE);
-	gtk_signal_connect (
-		GTK_OBJECT(mw->window), "destroy",
-		GTK_SIGNAL_FUNC(window_destroy_cb), mw);
-	gnome_app_create_menus_with_data (
-		GNOME_APP(mw->window),
-		message_post_main_menu, mw);
-	gnome_app_create_toolbar_with_data (
-		GNOME_APP(mw->window),
-		message_post_toolbar, mw);
+	gnome_app_create_menus_with_data (GNOME_APP(mw->window),
+	                                  message_post_main_menu, mw);
+	gnome_app_create_toolbar_with_data (GNOME_APP(mw->window),
+	                                    message_post_toolbar, mw);
 	pan_unlock();
 
 	w = create_post_info_pane (mw);
@@ -1678,9 +1698,20 @@ message_window_new (Article * article, MessageWindowType type)
 
 	if (mw->window != NULL)
 	{
-		gtk_window_set_transient_for (GTK_WINDOW(mw->window),
-		                              GTK_WINDOW(Pan.window));
+		pan_lock ();
+		gtk_window_set_default_size (GTK_WINDOW (mw->window), 550, 410);
+		gtk_signal_connect (GTK_OBJECT(mw->window), "delete_event",
+		                    GTK_SIGNAL_FUNC(window_delete_event_cb), mw);
+		gtk_signal_connect (GTK_OBJECT(mw->window), "destroy",
+		                    GTK_SIGNAL_FUNC(window_destroy_cb), mw);
+		gtk_window_set_policy (GTK_WINDOW(mw->window), TRUE, TRUE, TRUE);
+		pan_unlock ();
+
+		gui_restore_window_size (mw->window, "message_window");
+
+		pan_lock ();
 		gtk_widget_show_all (mw->window);
+		pan_unlock ();
 	}
 }
 
@@ -1828,6 +1859,8 @@ populate_article_from_mw (Article * article, const MessageWindow * mw)
 	gint i;
 	gchar ** sd;
 	gchar * pch;
+	gchar * author_addr;
+	gchar * author_real;
 
 	/* date */
 	article->date = time(0);
@@ -1837,7 +1870,17 @@ populate_article_from_mw (Article * article, const MessageWindow * mw)
 
 	/* author */
 	pch = get_text_from_editable (mw->from);
-	article_set_author_from_header (article, pch);
+	gnksa_do_check_from (pch, &author_addr, &author_real);
+	if (is_nonempty_string (author_real))
+		gnksa_strip_realname (author_real);
+	article->author_addr = is_nonempty_string (author_addr)
+		? group_chunk_string (article->group, author_addr)
+		: NULL;
+	article->author_real = is_nonempty_string (author_real)
+		? group_chunk_string (article->group, author_real)
+		: NULL;
+	g_free (author_addr);
+	g_free (author_real);
 	g_free (pch);
 
 	/* message-id */
@@ -1949,10 +1992,13 @@ build_article_from_window (const MessageWindow * mw)
 {
 	Group * folder;
 	Article * article;
-g_message ("build_article_from_window");
+	debug_enter ("build_article_from_window");
+
 	folder  = folder_get_by_name (PAN_SENDLATER);
 	article = article_new (folder);
 	populate_article_from_mw (article, mw);
+
+	debug_exit ("build_article_from_window");
 	return article;
 }
 
@@ -1965,7 +2011,6 @@ save_cb (GtkWidget * button, MessageWindow * mw)
 	GPtrArray * headers;
 
 	/* update the article fields from changes made in the gui */
-g_message ("save_cb");
 	tmp = article_new (a->group);
 	populate_article_from_mw (tmp, mw);
 	a->author_addr = tmp->author_addr;
@@ -1992,14 +2037,6 @@ g_message ("save_cb");
 	message_window_destroy (mw);
 }
 
-static void
-window_destroy_cb (GtkWidget * window, MessageWindow * mw)
-{
-	if (mw->article != NULL)
-		group_unref_articles (mw->article->group, NULL);
-	g_free (mw);
-}
-
 typedef enum
 {
 	SEND_NOW,
@@ -2011,9 +2048,13 @@ static void
 post (MessageWindow * mw, SendMode mode)
 {
 	Article * article;
-	Server * server = grouplist_get_server ();
-	GPtrArray * errors = g_ptr_array_new ();
+	Server * server;
+	GPtrArray * errors;
 	GoodnessLevel goodness;
+	debug_enter ("post");
+
+       	server = grouplist_get_server ();
+       	errors = g_ptr_array_new ();
 
 	/* Get the article pointer, either through repopulating the
 	 * current article (if we're editing an existing message) or
@@ -2060,6 +2101,8 @@ post (MessageWindow * mw, SendMode mode)
 	g_ptr_array_free (errors, TRUE);
 	if (goodness!=OKAY && mw->type!=EDIT_ORIGINAL)
 		group_remove_article (folder_get_by_name(PAN_SENDLATER), article);
+
+	debug_exit ("post");
 }
 
 static void
@@ -2117,7 +2160,6 @@ message_window_save_ok_clicked (GtkWidget * widget, GtkWidget * file_entry)
 		}
 		else
 		{
-			g_message ("fopen() failed to save message");
 		}
 	}
 
