@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include <gmime/gmime-utils.h>
 
 #include "acache.h"
+#include "debug.h"
 #include "gnksa.h"
 #include "log.h"
 #include "nntp.h"
@@ -66,6 +67,7 @@ nntp_get_response (PanSocket     * socket,
 	gint val;
 	gint retval;
 	const gchar * msg = NULL;
+	debug_enter ("nntp_get_response");
 
 	val = pan_socket_getline (socket, &msg);
 	if (val)
@@ -83,6 +85,7 @@ nntp_get_response (PanSocket     * socket,
 		retval = atoi (msg);
 	}
 
+	debug_exit ("nntp_get_response");
 	return retval;
 }
 
@@ -94,6 +97,7 @@ nntp_command (StatusItem           * status,
 {
 	int val;
 	size_t len;
+	debug_enter ("nntp_command");
 
 	g_return_val_if_fail (sock!=NULL, -1);
 	g_return_val_if_fail (is_nonempty_string(command), -1);
@@ -171,6 +175,7 @@ nntp_disconnect (StatusItem   * status,
 {
 	const gchar * reply;
 	gint val;
+	debug_enter ("nntp_disconnect");
 
 	/* entry assertions */
 	g_return_val_if_fail (status!=NULL, TASK_FAIL_HOPELESS);
@@ -184,6 +189,7 @@ nntp_disconnect (StatusItem   * status,
 	}
 
 	/* disconnected succcessfully */
+	debug_exit ("nntp_disconnect");
 	return TASK_SUCCESS;
 }
 
@@ -203,17 +209,29 @@ nntp_handshake (StatusItem  * status,
 	g_return_val_if_fail (status!=NULL, TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (socket!=NULL, TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (setme_posting_ok!=NULL, TASK_FAIL_HOPELESS);
+	debug_enter ("nntp_handshake");
 
 	/* get the server's handshake message */
 	val = nntp_get_response (socket, &response);
-	if (val!=200 && val!=201) {
+	if (val==200 || val==201) { /* these values are expected */
+		val = TASK_SUCCESS;
+		log_add_va (LOG_INFO, _("Handshake: %s"), response);
+	} else {
 		status_item_emit_error_va (status, _("NNTP handshake failed: %s"), response);
-		return val==-1 ? TASK_FAIL : TASK_FAIL_HOPELESS;
+		log_add_va (LOG_ERROR, _("Handshake failed: %s"), response);
+		return val==-1 ? TASK_FAIL : TASK_FAIL_HOPELESS; 
 	}
 
 	/* can we post? */
 	*setme_posting_ok = val==200;
-	return TASK_SUCCESS;
+
+	/* yes, we also support authentication on demand, but it appears
+	   many servers are happier if we ask for authentication up-front. */
+	if (socket->need_auth)
+		val = nntp_authenticate (status, socket);
+
+	debug_exit ("nntp_handshake");
+	return val;
 }
 
 gint
@@ -222,14 +240,15 @@ nntp_authenticate (StatusItem     * status,
 {
 	gint val;
 	const gchar * response = NULL;
+	debug_enter ("nntp_authenticate");
 
 	/* entry assertions */
 	g_return_val_if_fail (status!=NULL, TASK_FAIL);
 	g_return_val_if_fail (sock!=NULL, TASK_FAIL);
-	g_return_val_if_fail (is_nonempty_string(sock->nntp_username), TASK_FAIL_HOPELESS);
+	g_return_val_if_fail (sock->nntp_username!=NULL, TASK_FAIL_HOPELESS);
 
 	/* set the username */
-	val = nntp_command_va (status, sock, &response, "AUTHINFO user %s", sock->nntp_username);
+	val = nntp_command_va (status, sock, &response, "AUTHINFO USER %s", sock->nntp_username);
 	if (val!=OK_AUTH && val!=NEED_AUTHDATA) {
 		if (status != NULL)
 			status_item_emit_error_va (status, _("Authentication failed: %s"), response);
@@ -243,7 +262,7 @@ nntp_authenticate (StatusItem     * status,
 				status_item_emit_error_va (status, _("Authentication failed: need a password"));
 			return TASK_FAIL;
 		}
-		val = nntp_command_va (status, sock, &response, "AUTHINFO pass %s", sock->nntp_password);
+		val = nntp_command_va (status, sock, &response, "AUTHINFO PASS %s", sock->nntp_password);
 	}
 
 	/* if we failed, emit an error */	
@@ -253,6 +272,7 @@ nntp_authenticate (StatusItem     * status,
 		return TASK_FAIL;
 	}
 
+	debug_exit ("nntp_authenticate");
 	return TASK_SUCCESS;
 }
 
@@ -263,6 +283,7 @@ nntp_can_post (StatusItem   * status,
 {
 	gint val;
 	const gchar * response = NULL;
+	debug_enter ("nntp_can_post");
 
 	/* entry assertions */
 	g_return_val_if_fail (status!=NULL, TASK_FAIL_HOPELESS);
@@ -300,6 +321,7 @@ nntp_set_group (StatusItem   * status,
 {
 	gint val;
 	const gchar * response = NULL;
+	debug_enter ("nntp_set_group");
 
 	/* sanity checks */
 	g_return_val_if_fail (sock!=NULL, TASK_FAIL);
@@ -313,7 +335,7 @@ nntp_set_group (StatusItem   * status,
 	/* change to that group */
 	val = nntp_command_va (status, sock, &response, "GROUP %s", group_name);
 	if (val != OK_GROUP) {
-		status_item_emit_error_va (status, _("Unable to set group: %s"), response);
+		status_item_emit_error_va (status, _("Unable to set group `%s': %s"), group_name, response);
 		return val==-1 ? TASK_FAIL : TASK_FAIL_HOPELESS;
 	}
 
@@ -333,6 +355,7 @@ nntp_post (StatusItem    * status,
 {
 	gint val;
 	const gchar * response;
+	debug_enter ("nntp_post");
 
 	/* entry assertions */
 	g_return_val_if_fail (sock!=NULL, TASK_FAIL);
@@ -357,25 +380,26 @@ nntp_post (StatusItem    * status,
 
 		/* post the article */
 		val = nntp_command_va (status, sock, &response, "%s\r\n.\r\n", msg);
-		if (val != 240) {
+		if (val == 240) {
+			log_add_va (LOG_INFO, _("Posting complete: %s"), response);
+		} else {
 			status_item_emit_error_va (status, "Posting failed: %s", response);
 			return val==-1 ? TASK_FAIL : TASK_FAIL_HOPELESS;
-		}
+		} 
 	}
 
 	/* if we made it this far, we succeeded! */
 	return TASK_SUCCESS;
 }
 
-/**
-***
-**/
+
 
 gint
 nntp_noop (StatusItem    * status,
            PanSocket     * socket)
 {
 	gboolean can_post = FALSE;
+	debug_enter ("nntp_noop");
 
 	/* entry assertions */
 	g_return_val_if_fail (status!=NULL, TASK_FAIL_HOPELESS);
@@ -424,13 +448,15 @@ nntp_article_download (StatusItem      * status,
                        Group           * group,
                        Article         * article,
                        const gboolean  * abort,
-                       gint              verbose)
+                       gint              verbose,
+                       gboolean          checkout)
 {
 	const gchar * response;
 	gboolean getting_headers;
 	gint retval;
 	gint val;
 	GString * buf;
+	debug_enter ("nntp_article_download");
 
 	/* sanity checks */
 	g_return_val_if_fail (status!=NULL, TASK_FAIL);
@@ -445,11 +471,15 @@ nntp_article_download (StatusItem      * status,
 
 	/* request the article from the server */
 	val = nntp_command_va (status, sock, &response, "ARTICLE %lu", article->number);
-	if (val != 220) {
-		status_item_emit_error_va (status, _("Getting article from server failed: %s"), response);
+	if (val != 220)
+	{
+		gchar * pch = g_strdup_printf (_("Getting article `%s' body failed: %s"), article->subject, response);
+		status_item_emit_error (status, pch);
+		g_free (pch);
+
 		if (val==-1)
 			return TASK_FAIL;
-		article_add_flag (article, STATE_ERROR);
+		articles_add_flag (&article, 1, STATE_ERROR);
 		return TASK_FAIL_HOPELESS;
 	}
 
@@ -514,12 +544,13 @@ nntp_article_download (StatusItem      * status,
 	}
 
 	/* save the message */
-	acache_set_message (article->message_id, buf->str);
+	if (retval == TASK_SUCCESS) {
+        	acache_set_message (article->message_id, buf->str, checkout);
+		articles_remove_flag (&article, 1, STATE_DOWNLOAD_FLAGGED);
+	}
+
 	g_string_free (buf, TRUE);
-
-	if (retval == TASK_SUCCESS)
-		article_remove_flag (article, STATE_DOWNLOAD_FLAGGED);
-
+	debug_exit ("nntp_article_download");
 	return retval;
 }
 
@@ -539,6 +570,7 @@ nntp_get_group_info (StatusItem       * status,
 {
 	gint val;
 	const gchar * response = NULL;
+	debug_enter ("nntp_get_group_info");
 
 	/* entry assertions */
 	g_return_val_if_fail (status!=NULL, TASK_FAIL);
@@ -552,7 +584,7 @@ nntp_get_group_info (StatusItem       * status,
 	/* change to that group */
 	val = nntp_command_va (status, sock, &response, "GROUP %s", group_name);
 	if (val != OK_GROUP) {
-		status_item_emit_error_va (status, _("Unable to set group: %s"), response);
+		status_item_emit_error_va (status, _("Unable to set group `%s': %s"), group_name, response);
 		return val==-1 ? TASK_FAIL : TASK_FAIL_HOPELESS;
 	}
 
@@ -567,14 +599,14 @@ nntp_get_group_info (StatusItem       * status,
 **/
 
 gint
-nntp_articlelist_download (StatusItem       * status,
-                           PanSocket        * sock,
-                           Group            * group,
-                           gulong             low,
-                           gulong             high,
-                           const gboolean   * abort,
-                           const char       * progress_fmt,
-                           GPtrArray        * setme)
+nntp_download_headers (StatusItem       * status,
+                       PanSocket        * sock,
+                       Group            * group,
+                       gulong             low,
+                       gulong             high,
+                       const gboolean   * abort,
+                       const char       * progress_fmt,
+                       GPtrArray        * setme)
 {
 	const char * buffer;
 	const char * response;
@@ -584,6 +616,7 @@ nntp_articlelist_download (StatusItem       * status,
 	gboolean read_status = 0;
 	gint result;
 	gint val;
+	debug_enter ("nntp_download_headers");
 
 	/* sanity checks */
 	g_return_val_if_fail (status!=NULL, TASK_FAIL_HOPELESS);
@@ -658,8 +691,9 @@ nntp_articlelist_download (StatusItem       * status,
 		/* subject */
 		tmp = get_next_token_str (pch, '\t', &pch);
 		replace_gstr (&tmp, g_mime_utils_8bit_header_decode(tmp));
-		if (is_nonempty_string(tmp))
-			article_set_header(a, HEADER_SUBJECT, tmp, DO_CHUNK);
+		if (!is_nonempty_string(tmp))
+			replace_gstr (&tmp, g_strdup(_("No Subject")));
+		article_set_header(a, HEADER_SUBJECT, tmp, DO_CHUNK);
 		g_free (tmp);
 
 		/* author */
@@ -711,35 +745,13 @@ nntp_articlelist_download (StatusItem       * status,
 		***  Validity Checking
 		**/
 
-		pch = article_get_header (a, HEADER_SUBJECT);
-		if (!is_nonempty_string(pch)) {
-			status_item_emit_error_va (status, _("Corrupt header skipped - no subject: %s"), buffer);
-			pan_object_unref (PAN_OBJECT(a));
-			continue;
-		}
-
 		pch = article_get_header (a, HEADER_MESSAGE_ID);
 		if (!is_nonempty_string(pch)) {
-			status_item_emit_error_va (status, _("Corrupt header skipped - no msg id: %s"), buffer);
+			status_item_emit_error_va (status, _("Corrupt header skipped - no Message Id: %s"), buffer);
 			pan_object_unref (PAN_OBJECT(a));
 			continue;
 		}
 
-		if (a->date == 0) {
-			status_item_emit_error_va (status, _("Corrupt header skipped - no date: %s"), buffer);
-			pan_object_unref (PAN_OBJECT(a));
-			continue;
-		}
-
-		tmp = article_get_author_str (a);
-		if (!is_nonempty_string(tmp)) {
-			status_item_emit_error_va (status, _("Corrupt header skipped - no author: %s"), buffer);
-			pan_object_unref (PAN_OBJECT(a));
-			g_free (tmp);
-			continue;
-		}
-		g_free (tmp);
-
 
 		/* Look for the (n/N) or [n/N] construct in subject lines,
 		 * starting at the end of the string and working backwards */
@@ -832,12 +844,15 @@ nntp_download_bodies (StatusItem       * status,
                       PanSocket        * sock,
                       const gboolean   * abort,
                       const GPtrArray  * const articles,
-                      gint             * index)
+                      gint             * index,
+                      gboolean           checkout,
+                      gboolean           abort_if_one_fails)
 {
 	gint result;
 	gint i;
 	gint lines_so_far;
 	gint lines_total;
+	debug_enter ("nntp_download_bodies");
 
 	/* sanity checks */
 	g_return_val_if_fail (status!=NULL, TASK_FAIL_HOPELESS);
@@ -874,41 +889,31 @@ nntp_download_bodies (StatusItem       * status,
 		const gchar * pch;
 		Article * article = ARTICLE(g_ptr_array_index(articles,*index));
 
-		status_item_emit_status_va (status,
-			_("Getting article %d of %d"),
-			1+*index, articles->len);
+		status_item_emit_status_va (status, _("Getting %d of %d: \"%s\""),
+		                                    1+*index, articles->len,
+		                                    article->subject);
 
-		/* download if we have to.  Abort if the download failed but
-		 * might succeed later.  Hopeless downloads silently fall
-		 * through; it's pointless to postpone the task for them. */
+		/* download if we have to. */
 		pch = article_get_header (article, HEADER_MESSAGE_ID);
-		if (!acache_has_message (pch))
+		if (acache_has_message (pch))
+		{
+			result = TASK_SUCCESS;
+
+			/* since it's cached locally, we can unflag */
+			articles_remove_flag (&article, 1, STATE_DOWNLOAD_FLAGGED);
+		}
+		else
 		{
 			result = nntp_article_download (status,
 			                                sock,
 			                                group,
 			                                article,
 			                                abort,
-			                                NNTP_VERBOSE_NEXT_STEP);
-		}
-		else
-		{
-			result = TASK_SUCCESS;
+			                                NNTP_VERBOSE_NEXT_STEP,
+			                                checkout);
 
-			/* since it's cached locally, we can unflag */
-			article_remove_flag (article, STATE_DOWNLOAD_FLAGGED);
-		}
-
-		switch (result)
-		{
-			case TASK_FAIL:
-				status_item_emit_status_va (status, _("Downloading article \"%s\" failed."), article->subject);
+			if (result!=TASK_SUCCESS && abort_if_one_fails)
 				return result;
-			case TASK_SUCCESS:
-			case TASK_FAIL_HOPELESS:
-				break;
-			default:
-				pan_warn_if_reached ();
 		}
 	}
 
@@ -926,6 +931,7 @@ nntp_cancel (StatusItem     * status,
 	const gchar * newsgroups;
 	gint retval;
 	const gchar * msg_id = article_get_header (article, HEADER_MESSAGE_ID);
+	debug_enter ("nntp_cancel");
 
 	/* sanity checks */
 	g_return_val_if_fail (sock != NULL, TASK_FAIL);
@@ -956,5 +962,6 @@ nntp_cancel (StatusItem     * status,
 	g_free (author);
 	g_string_free (msg, TRUE);
 
+	debug_exit ("nntp_cancel");
 	return retval;
 }
