@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -83,12 +83,13 @@ gint    nntp_get_group_info       (StatusItem           * status,
                                    gulong               * setme_high_num,
                                    const gboolean       * abort);
 
-gint    nntp_article_download     (StatusItem      * status,
-                                   PanSocket       * sock,
-                                   Group           * group,
-                                   Article         * article,
-                                   const gboolean  * abort_download,
-			           gint              nntp_verbosity);
+gint    nntp_article_download     (StatusItem           * status,
+                                   PanSocket            * sock,
+                                   Group                * group,
+                                   Article              * article,
+                                   const gboolean       * abort_download,
+			           gint                   nntp_verbosity,
+			           gboolean               checkout);
 
 /**
  * This will try to get headers for articles [low-high] from the specified
@@ -97,14 +98,14 @@ gint    nntp_article_download     (StatusItem      * status,
  * If verbose is true and the operation is successful,  the StatusItem will
  * emit low-high status_item_next_step signals.
  */
-gint nntp_articlelist_download (StatusItem       * item,
-                                PanSocket        * sock,
-                                Group            * group,
-                                gulong             low,
-                                gulong             high,
-                                const gboolean   * abort,
-			        const char       * progress_fmt,
-                                GPtrArray        * setme);
+gint nntp_download_headers (StatusItem       * item,
+                            PanSocket        * sock,
+                            Group            * group,
+                            gulong             low,
+                            gulong             high,
+                            const gboolean   * abort,
+			    const char       * progress_fmt,
+                            GPtrArray        * setme);
 
 /**
  * emits articles->len status_item_next_step signals.
@@ -114,7 +115,9 @@ gint nntp_download_bodies (StatusItem        * item,
                            PanSocket         * sock,
                            const gboolean    * abort,
                            const GPtrArray   * const articles,
-                           gint              * index);
+                           gint              * index,
+                           gboolean            checkout,
+                           gboolean            abort_if_one_fails);
 
 gint nntp_cancel (StatusItem            * item,
                   const Article         * article,
