@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,8 +37,13 @@
 #include "text.h"
 #include "thread-watch.h"
 #include "util.h"
+#include "util-wrap.h"
 #include "dialogs/dialogs.h"
 
+#ifdef GTKHTML_HAVE_GCONF
+#include <gconf/gconf-client.h>
+#endif
+
 static gint save_session (GnomeClient        * client,
 			  gint                 phase,
 			  GnomeSaveStyle       save_style,
@@ -63,12 +68,13 @@ static gint       debug_socket = 0;
 static gint       debug_socket_input = 0;
 static gint       debug_socket_output = 0;
 static gint       debug_queue = 0;
+static gint       debug_gnksa = 0;
 static gint       debug_newsrc = 0;
+static gint       debug_trace = 0;
 static gint       debug_decode = 0;
 static gint       debug_gui_locks = 0;
-static gint       debug_db = 0;
 static gint       debug_pan_object = 0;
-static gint       debug_everything = 0;
+static gint       debug_defaults = 0;
 static gint       mute = 0;
 static gint       regression_tests = 0;
 
@@ -76,13 +82,10 @@ struct poptOption options[] = {
 	{ "geometry", '\0', POPT_ARG_STRING, &geometry, 0,
 	  N_("Specify the geometry of the window."),
 	  N_("GEOMETRY") },
-
 	{ "mute", '\0', 0, &mute, 0,
 	  N_("Send outgoing messages to stdout instead of smtp/nntp servers for debugging purposes."), NULL },
 	{ "debug-acache", '\0', 0, &debug_acache, 0,
 	  N_("Turn on article cache debugging messages."), NULL },
-	{ "debug-db", '\0', 0, &debug_db, 0,
-	  N_("Turn on database debugging messages."), NULL },
 	{ "debug-socket-input", '\0', 0, &debug_socket_input, 0,
 	  N_("Turn on socket input debugging messages."), NULL },
 	{ "debug-socket-output", '\0', 0, &debug_socket_output, 0,
@@ -91,6 +94,8 @@ struct poptOption options[] = {
 	  N_("Turn on socket input/output debugging messages."), NULL },
 	{ "debug-sockets", '\0', 0, &debug_socket, 0,
 	  N_("Same as --debug-socket"), NULL },
+	{ "debug-trace", '\0', 0, &debug_trace, 0,
+	  N_("Trace function calls"), NULL },
 	{ "debug-queue", '\0', 0, &debug_queue, 0,
 	  N_("Turn on task scheduler debugging messages."), NULL },
 	{ "debug-decode", '\0', 0, &debug_decode, 0,
@@ -99,10 +104,12 @@ struct poptOption options[] = {
 	  N_("Turn on newsrc decoding debugging messages."), NULL },
 	{ "debug-gui-locks", '\0', 0, &debug_gui_locks, 0,
 	  N_("Turn on gui thread lock debugging messages."), NULL },
+	{ "debug-gnksa", '\0', 0, &debug_gnksa, 0,
+	  N_("Turn on debugging of GNKSA code."), NULL },
 	{ "debug-pan-object", '\0', 0, &debug_pan_object, 0,
 	  N_("Turn on pan object system debugging messages."), NULL },
-	{ "debug", '\0', 0, &debug_everything, 0,
-	  N_("Turn on ALL debugging messages."), NULL },
+	{ "debug", '\0', 0, &debug_defaults, 0,
+	  N_("Turn on most of the generally-useful debug flags.  A good first choice when preparing a bug report."), NULL },
 	{ "regression-tests", '\0', 0, &regression_tests, 0,
 	  N_("Run regression tests for the Pan programmers."), NULL },
 
@@ -126,8 +133,6 @@ pan_post_gtkmain_init (gpointer null)
 void
 pan_init (void)
 {
-	acache_init ();
-	Pan.eventlog = NULL;
 	gtk_idle_add (pan_post_gtkmain_init, NULL);
 }
 
@@ -159,6 +164,9 @@ main (int argc, char *argv[])
 #if GNOME_LIBS_VERSION>=60
 	gnome_window_icon_set_default_from_file (GNOME_ICONDIR "/pan.png");
 #endif
+#endif
+#ifdef GTKHTML_HAVE_GCONF
+	gconf_init (argc, argv, NULL);
 #endif
 
 	debug_flags = (debug_acache ? DEBUG_ACACHE : 0)
@@ -167,25 +175,31 @@ main (int argc, char *argv[])
 		| (debug_queue ? DEBUG_QUEUE : 0)
 		| (debug_decode ? DEBUG_DECODE : 0)
 		| (debug_newsrc ? DEBUG_NEWSRC : 0)
+		| (debug_gnksa ? DEBUG_GNKSA : 0)
+		| (debug_trace ? DEBUG_TRACE : 0)
 		| (debug_gui_locks ? DEBUG_LOCK : 0)
 		| (debug_pan_object ? DEBUG_PAN_OBJECT : 0)
-		| (debug_db ? DEBUG_DB : 0)
-		| (debug_everything ? ~0 : 0);
+		| (debug_defaults ? (DEBUG_TRACE|DEBUG_DECODE|DEBUG_ACACHE|DEBUG_SOCKET_INPUT|DEBUG_SOCKET_OUTPUT): 0);
 	set_debug_level (debug_flags);
 
-	printf (_("Pan %s started\n"), VERSION);
-	fflush (NULL);
+	default_incoming_name_real = _("Unknown");
+	default_incoming_name_addr = _("unknown@spammesenseless.net");
 
 
+	log_init ();
+	log_add_va (LOG_INFO, _("Pan %s Started"), VERSION);
+
 	if (regression_tests)
 	{
 		util_regression_tests ();
 		gnksa_regression_tests ();
+		wrap_regression_tests ();
 		regression_tests_summary ();
 		return 0;
 	}
 
 	prefs_init ();
+	acache_init ();
 
 	check_and_warn_if_version_change ();
 
@@ -242,12 +256,11 @@ pan_shutdown (void)
 	g_free (grouplist_font);
 
 	server_shutdown ();
-
 	acache_shutdown ();
-
 	bozo_shutdown_module ();
 	thread_shutdown_module ();
 	rule_manager_shutdown_module ();
+	log_shutdown_module ();
 
 	pan_object_bookkeeping ();
 
