@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 
 #include "acache.h"
 #include "articlelist.h"
+#include "debug.h"
 #include "globals.h"
 #include "gui.h"
 #include "gui-headers.h"
@@ -103,7 +104,7 @@ typedef struct
 	/* general */
 	GtkWidget * display_article_dates_in_local_time_cbutton;
 	GtkWidget * fetch_new_on_group_load_cbutton;
-	/* GtkWidget * break_thread_when_subject_changes_cbutton; */
+	GtkWidget * break_thread_when_subject_changes_cbutton;
 	GtkWidget * online_prompt_first_cbutton;
 	GtkWidget * online_without_asking_cbutton;
 	GtkWidget * online_never_cbutton;
@@ -142,6 +143,8 @@ typedef struct
 	GtkWidget * text_fg_gcp;
 	GtkWidget * text_bg_gcp;
 	GtkWidget * text_quoted_gcp;
+	GtkWidget * thread_date_entry;
+	GtkWidget * body_date_entry;
 
 	/* layout */
 	GtkWidget * layout_page;
@@ -199,17 +202,20 @@ static GSList * sprof_list = NULL; /* holds the profile editor GnomeDialogs */
 
 extern GtkTooltips * ttips;
 
-gchar * layout_str = NULL;
+gchar * thread_date_format                    = NULL;
+gchar * body_date_format                      = NULL;
+gchar * layout_str                            = NULL;
 gint wrap_column                              = 74;
 gint watched_thread_span_days                 = 30;
 gint mail_server_port                         = 0;
-/* gboolean break_thread_when_subject_changes    = FALSE; */
+gboolean break_thread_when_subject_changes    = FALSE;
 gboolean display_article_dates_in_local_time  = FALSE;
 gboolean hide_mpart_child_nodes               = TRUE;
 gboolean pan_mute                             = FALSE;
 gboolean auto_part_selection                  = TRUE;
 gboolean use_gdk_fontset_load                 = FALSE;
 gboolean fetch_new_on_group_load              = TRUE;
+gfloat acache_max_megs                        = 2.0;
 
 ListClickAction grouplist_mb1_action          = CLICK_ACTION_LOAD;
 ListClickAction grouplist_mb2_action          = CLICK_ACTION_SELECT;
@@ -229,6 +235,29 @@ gchar * external_editor = NULL;
 gchar * extra_posting_headers = NULL;
 gchar * mail_server_address = NULL;
 
+static const gchar*
+get_date_format_tooltip_string (void)
+{
+	return _("%a - abbreviated weekday name\n"
+	         "%A - full weekday name\n"
+	         "%b - abbreviated month name\n"
+	         "%B - full month name\n"
+	         "%c - local date & time\n"
+		 "%d - day of the month\n"
+		 "%H - hour (24-hour clock)\n"
+		 "%I - hour (12-hour clock)\n"
+		 "%j - day of the year (001-366)\n"
+		 "%m - month (01-12)\n"
+		 "%M - minute (00-59)\n"
+		 "%p - local equivalent of AM/PM\n"
+		 "%S - second (00-61)\n"
+		 "%x - local date\n"
+		 "%X - local time\n"
+		 "%y - two-digit year\n"
+		 "%Y - four-digit year\n"
+		 "%% - %");
+}
+
 #define connect_signal_to_prefs_changed(object,signal_name) \
 	gtk_signal_connect_object (GTK_OBJECT(object), signal_name, \
 	                           GTK_SIGNAL_FUNC(gnome_property_box_changed), \
@@ -378,8 +407,6 @@ static void
 prefs_apply_cb (GtkWidget   * box,
                 gint          page_num)
 {
-	int cache_old;
-	int cache_new;
 	gboolean tmp_bool = FALSE;
 	gboolean articlelist_changed = FALSE;
 	gboolean server_connections_changed = FALSE;
@@ -395,6 +422,18 @@ prefs_apply_cb (GtkWidget   * box,
 	 * gnome_config_set_int("/Pan/Articles/Expiration", 30);
 	 */
 
+	/* thread date format */
+	if (1) {
+		gchar * tmp = g_strdup (thread_date_format);
+		replace_string_and_config_from_editable (
+			&thread_date_format,
+			"/Pan/Display/Thread_Date_Format",
+			win->thread_date_entry);
+		if (pan_strcmp (tmp, thread_date_format))
+			articlelist_changed = TRUE;
+		g_free (tmp);
+	}
+
 	set_config_from_editable (
 		"/Pan/User/Full_Name",
 		win->user_fullname);
@@ -458,11 +497,24 @@ prefs_apply_cb (GtkWidget   * box,
 	articlelist_changed |= handle_color_picker ("killfile_color", &killfile_color, win->ignored_gcp);
 	articlelist_changed |= handle_color_picker ("watched_color", &watched_color, win->watched_gcp);
 
-	/* update the text window colors */
+	/* text window colors */
 	color_changed = FALSE;
 	color_changed |= handle_color_picker ("text_fg_color", &text_fg_color, win->text_fg_gcp);
 	color_changed |= handle_color_picker ("text_bg_color", &text_bg_color, win->text_bg_gcp);
 	color_changed |= handle_color_picker ("text_quoted_color", &text_quoted_color, win->text_quoted_gcp);
+
+	/* text window date format */
+	if (1) {
+		gchar * tmp = g_strdup (body_date_format);
+		replace_string_and_config_from_editable (
+			&body_date_format,
+			"/Pan/Display/Body_Date_Format",
+			win->body_date_entry);
+		if (pan_strcmp (tmp, body_date_format))
+			color_changed = TRUE;
+		g_free (tmp);
+	}
+
 	if (color_changed)
 	{
 		/* must use a new style* in order to update bg color */
@@ -514,17 +566,14 @@ prefs_apply_cb (GtkWidget   * box,
 	if (handle_font_picker (win->message_gfp,
 	                        &message_body_font,
 	                        "/Pan/Display/message_body_font")) {
-		widget_set_font (GTK_WIDGET(Pan.text), message_body_font);
-		text_refresh ();
+		text_set_font (message_body_font);
+
 	}
 
 	/* cache */
-	cache_old = gnome_config_get_int_with_default(
-		"/Pan/Cache/MaxSize=1242880", NULL);
-	cache_new = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(win->cache_megs_sb));
-	cache_new *= (1024*1024); /* convert to bytes */
-	if (cache_old != cache_new) {
-		gnome_config_set_int ("/Pan/Cache/MaxSize", cache_new);
+	if (1) {
+		acache_max_megs = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(win->cache_megs_sb));
+		gnome_config_set_float ("/Pan/Cache/MaxMegs", acache_max_megs);
 		acache_expire ();
 	}
 	gnome_config_set_bool ("/Pan/Cache/FlushOnExit",
@@ -608,7 +657,6 @@ prefs_apply_cb (GtkWidget   * box,
 	}
 
 
-#if 0	
 	tmp_bool = break_thread_when_subject_changes;
 	update_entry_and_bool_from_toggle_button (
 		&break_thread_when_subject_changes,
@@ -616,7 +664,6 @@ prefs_apply_cb (GtkWidget   * box,
 		win->break_thread_when_subject_changes_cbutton);
 	if (tmp_bool != break_thread_when_subject_changes)
 		articlelist_changed = TRUE;
-#endif
 
 
 	tmp_bool = display_article_dates_in_local_time;
@@ -694,11 +741,11 @@ prefs_apply_cb (GtkWidget   * box,
 				server_connections_changed = TRUE;
 			}
 
-			/* reserve nonleech */
+			/* reserve for bodies */
 			i = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(sc->reserve_tb));
-			if ((i!=0) != (server->reserve_nonleech!=0))
+			if ((i!=0) != (server->reserve_connection_for_bodies!=0))
 			{
-				server->reserve_nonleech = i;
+				server->reserve_connection_for_bodies = i;
 				server_connections_changed = TRUE;
 			}
 		}
@@ -750,7 +797,7 @@ prefs_apply_cb (GtkWidget   * box,
 	/* layout */
 	if (1) {
 		gchar * str = layout_get_new_string (win->layout_page);
-		if (strcmp (str, layout_str)) {
+		if (pan_strcmp (str, layout_str)) {
 			replace_gstr (&layout_str, g_strdup(str));
 			gnome_config_set_string ("/Pan/Display/Layout", layout_str);
 			gui_set_layout (GUI_PANED);
@@ -879,6 +926,7 @@ prefs_init (void)
 {
 	static gchar * static_tmp_dir = NULL;
 	gchar *pch = NULL;
+	debug_enter ("prefs_init");
 
 	/* get the layout string */
 	replace_gstr (&layout_str, gnome_config_get_string ("/Pan/Display/Layout"));
@@ -907,7 +955,6 @@ prefs_init (void)
 		ensure_trailing_slash (&download_dir);
 	else
 		download_dir = gnome_util_prepend_user_home ("/News/Pan/");
-	directory_check (download_dir);
 
 
 	/* (Set TMPDIR env var, so that uulib uses the right tmpdir) */
@@ -916,9 +963,11 @@ prefs_init (void)
 	replace_gstr (&static_tmp_dir, pch); /* purify says not to free pch */
 	pch = NULL;
 
+	acache_max_megs = gnome_config_get_float ("/Pan/Cache/MaxMegs=2.0");
+
 	/* general settings */
 
-	text_set_wrap (gnome_config_get_bool("/Pan/General/do_wrap=TRUE"));
+	text_set_wrap (gnome_config_get_bool("/Pan/General/do_wrap=FALSE"));
 	wrap_column = gnome_config_get_int ("/Pan/General/wrap_column=74");
 	online_prompt = gnome_config_get_bool ("/Pan/General/online_prompt");
 	use_gdk_fontset_load = gnome_config_get_bool ("/Pan/General/use_gdk_fontset_load=FALSE");
@@ -930,10 +979,9 @@ prefs_init (void)
 	articlelist_mb2_action = gnome_config_get_int ("/Pan/General/articlelist_mb2_action=0");
 
 	auto_part_selection = gnome_config_get_bool ("/Pan/General/auto_part_selection=TRUE");
-#if 0
-	break_thread_when_subject_changes = gnome_config_get_bool (
-		"/Pan/General/break_thread_when_subject_changes=FALSE");
-#endif
+
+	break_thread_when_subject_changes = gnome_config_get_bool ("/Pan/General/break_thread_when_subject_changes=FALSE");
+
 	display_article_dates_in_local_time = gnome_config_get_bool ("/Pan/General/display_article_dates_in_local_time=FALSE");
 	hide_mpart_child_nodes = gnome_config_get_bool ("/Pan/General/hide_mpart_child_nodes=TRUE");
 	going_online_preference = gnome_config_get_int ("/Pan/General/Going_Online=0");
@@ -942,7 +990,15 @@ prefs_init (void)
 	/* user settings */
 	replace_gstr (&extra_posting_headers, gnome_config_get_string ("/Pan/Posting/Extra_Headers"));
 	if (extra_posting_headers == NULL)
-		extra_posting_headers = g_strdup ("X-No-Productlinks: Yes");
+		extra_posting_headers = g_strdup ("");
+
+	replace_gstr (&thread_date_format, gnome_config_get_string ("/Pan/Display/Thread_Date_Format"));
+	if (!is_nonempty_string (thread_date_format))
+		replace_gstr (&thread_date_format, g_strdup ("%m.%d %H:%M"));
+
+	replace_gstr (&body_date_format, gnome_config_get_string ("/Pan/Display/Body_Date_Format"));
+	if (!is_nonempty_string (body_date_format))
+		replace_gstr (&body_date_format, g_strdup("%a, %d %b %Y %H:%M:%S"));
 
 	replace_gstr (&attribution_line, gnome_config_get_string ("/Pan/User/attribution_line"));
 	if (!attribution_line)
@@ -972,6 +1028,8 @@ prefs_init (void)
 	set_color (&text_fg_color,    "text_fg_color",        0,     0,     0);
 	set_color (&text_bg_color,    "text_bg_color",    65335, 65335, 65335);
 	set_color (&text_quoted_color,"text_quoted_color",    0,     0, 65335);
+
+	debug_exit ("prefs_init");
 }
 
 static void
@@ -1661,10 +1719,47 @@ prefs_text_page (void)
 	GtkWidget * vbox;
 	GtkWidget * w;
 	GtkWidget * v;
+	GtkWidget * h;
 
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER(vbox), GNOME_PAD_SMALL);
 
+	/**
+	***  Misc
+	**/
+
+	v = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
+	gtk_container_set_border_width (GTK_CONTAINER(v), GNOME_PAD_SMALL);
+
+	h = gtk_hbox_new (FALSE, 0);
+	w = gtk_check_button_new_with_label (_("Fill articles for reading at column"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), text_get_wrap());
+	connect_signal_to_prefs_changed (w, "toggled");
+	gtk_box_pack_start (GTK_BOX(h), w, FALSE, FALSE, 0);
+	win->wrap_column_tb = w;
+	adj = (GtkAdjustment *) gtk_adjustment_new (256.0, 1.0, 256.0, 1.0, 1.0, 0.0);
+	w = gtk_spin_button_new (adj, 0, 0);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON(w), wrap_column);
+	connect_signal_to_prefs_changed (adj, "value_changed");
+	gtk_box_pack_start (GTK_BOX(h), w, FALSE, FALSE, 0);
+	win->wrap_column_sb = w;
+	gtk_box_pack_start (GTK_BOX(v), h, FALSE, FALSE, 0);
+
+	h = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
+	w = gtk_label_new (_("Article date format:"));
+	gtk_box_pack_start (GTK_BOX(h), w, FALSE, FALSE, 0);
+	w = gtk_entry_new_with_max_length (128);
+	gtk_entry_set_text (GTK_ENTRY(w), body_date_format);
+	connect_signal_to_prefs_changed (w, "changed");
+	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, get_date_format_tooltip_string(), "");
+	gtk_box_pack_start (GTK_BOX(h), w, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX(v), h, FALSE, FALSE, 0);
+	win->body_date_entry = w;
+
+	frame = gtk_frame_new (_("Misc"));
+	gtk_container_add (GTK_CONTAINER(frame), v);
+	gtk_box_pack_start (GTK_BOX(vbox), frame, FALSE, FALSE, 0);
+
 	/**
 	***  Fonts
 	**/
@@ -1691,29 +1786,6 @@ prefs_text_page (void)
 	gtk_container_add (GTK_CONTAINER(frame), v);
 	gtk_box_pack_start (GTK_BOX(vbox), frame, FALSE, FALSE, GNOME_PAD_SMALL);
 
-	/**
-	***  Fill
-	**/
-
-	v = gtk_hbox_new (FALSE, 0);
-
-	w = gtk_check_button_new_with_label (_("Fill articles for reading at column"));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), text_get_wrap());
-	connect_signal_to_prefs_changed (w, "toggled");
-	gtk_box_pack_start (GTK_BOX(v), w, FALSE, FALSE, 0);
-	win->wrap_column_tb = w;
-
-	adj = (GtkAdjustment *) gtk_adjustment_new (256.0, 1.0, 256.0, 1.0, 1.0, 0.0);
-	w = gtk_spin_button_new (adj, 0, 0);
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON(w), wrap_column);
-	connect_signal_to_prefs_changed (adj, "value_changed");
-	gtk_box_pack_start (GTK_BOX(v), w, FALSE, FALSE, 0);
-	win->wrap_column_sb = w;
-
-	frame = gtk_frame_new (_("Text Wrapping"));
-	gtk_container_add (GTK_CONTAINER(frame), v);
-	gtk_box_pack_start (GTK_BOX(vbox), frame, FALSE, FALSE, GNOME_PAD_SMALL);
-
 	/**
 	***  Headers
 	**/
@@ -1778,6 +1850,7 @@ prefs_articlelist_page (void)
 	GtkWidget * frame;
 	GtkWidget * vbox;
 	GtkWidget * w;
+	GtkWidget * h;
 	GtkWidget * v;
 	GtkWidget * b;
 
@@ -1862,23 +1935,15 @@ prefs_articlelist_page (void)
 	***  General
 	**/
 
-	v = gtk_vbox_new (FALSE, 0);
+	v = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
+	gtk_container_set_border_width (GTK_CONTAINER(v), GNOME_PAD_SMALL);
 
-#if 0
 	/* break threads when subject changes? */
 	w = gtk_check_button_new_with_label (_("When a followup changes the subject, show as new thread"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), break_thread_when_subject_changes);
 	connect_signal_to_prefs_changed (w, "toggled");
 	gtk_box_pack_start (GTK_BOX(v), w, FALSE, FALSE, 0);
 	win->break_thread_when_subject_changes_cbutton = w;
-#endif
-
-	/* utc or local time */
-	w = gtk_check_button_new_with_label (_("Display article dates in local time instead of UTC"));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), display_article_dates_in_local_time);
-	connect_signal_to_prefs_changed (w, "toggled");
-	gtk_box_pack_start (GTK_BOX(v), w, FALSE, FALSE, 0);
-	win->display_article_dates_in_local_time_cbutton = w;
 
 	/* multipart: one node or all */
 	w = gtk_check_button_new_with_label (_("Show only the first part of complete multipart binary posts"));
@@ -1887,10 +1952,29 @@ prefs_articlelist_page (void)
 	gtk_box_pack_start (GTK_BOX(v), w, FALSE, FALSE, 0);
 	win->hide_mpart_child_nodes_cbutton = w;
 
-	frame = gtk_frame_new (_("Display"));
+	/* utc or local time */
+	w = gtk_check_button_new_with_label (_("Display article dates in local time instead of UTC"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), display_article_dates_in_local_time);
+	connect_signal_to_prefs_changed (w, "toggled");
+	gtk_box_pack_start (GTK_BOX(v), w, FALSE, FALSE, 0);
+	win->display_article_dates_in_local_time_cbutton = w;
+
+	/* date format */
+	h = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
+	w = gtk_label_new (_("Article date format:"));
+	gtk_box_pack_start (GTK_BOX(h), w, FALSE, FALSE, 0);
+	w = gtk_entry_new_with_max_length (128);
+	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, get_date_format_tooltip_string(), "");
+	gtk_entry_set_text (GTK_ENTRY(w), thread_date_format);
+	connect_signal_to_prefs_changed (w, "changed");
+	gtk_box_pack_start (GTK_BOX(h), w, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX(v), h, FALSE, FALSE, 0);
+	win->thread_date_entry = w;
+
+	/* frame */
+	frame = gtk_frame_new (_("Misc"));
 	gtk_container_add (GTK_CONTAINER(frame), v);
-	gtk_container_set_border_width (GTK_CONTAINER(v), GNOME_PAD_SMALL);
-	gtk_box_pack_start (GTK_BOX(vbox), frame, FALSE, FALSE, GNOME_PAD_SMALL);
+	gtk_box_pack_start (GTK_BOX(vbox), frame, FALSE, FALSE, 0);
 
 	return vbox;
 }
@@ -2062,7 +2146,7 @@ prefs_layout_page (void)
 
 	/* panes clist */
 	clist = gtk_clist_new (1);
-	gtk_clist_set_column_width (GTK_CLIST(clist), 0, 120);
+	gtk_clist_set_column_width (GTK_CLIST(clist), 0, 200);
 
 	pch = layout_str + 1;
 	sprintf (buf, "1. %s", layout_get_pane_name(*pch));
@@ -2326,7 +2410,7 @@ prefs_connections_page (void)
 	w = gtk_label_new (_("Online Status"));
 	gtk_misc_set_padding (GTK_MISC(w), GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 	gtk_table_attach (GTK_TABLE (table), w, 2, 3, 0, 1,  GTK_FILL, GTK_FILL, 0, 0);
-	w = gtk_label_new (_("Reserve one connection\nfor non-decode tasks"));
+	w = gtk_label_new (_("Reserve a connection\nfor reading articles"));
 	gtk_misc_set_padding (GTK_MISC(w), GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 	gtk_table_attach (GTK_TABLE (table), w, 3, 4, 0, 1,  GTK_FILL, GTK_FILL, 0, 0);
 	for (row=1, l=Pan.serverlist; l!=NULL; l=l->next, ++row)
@@ -2371,7 +2455,7 @@ prefs_connections_page (void)
 		w = gtk_check_button_new_with_label (_("Reserve"));
 		sc->reserve_tb = w;
 		gtk_table_attach (GTK_TABLE(table), inset(w), 3, 4, row, row+1, GTK_FILL, GTK_FILL, 0, 0);
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), server->reserve_nonleech);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), server->reserve_connection_for_bodies);
 		connect_signal_to_prefs_changed (w, "toggled");
 
 		/* remember this server */
@@ -2582,7 +2666,6 @@ clear_acache_clicked_cb (GtkButton *button, gpointer data)
 static GtkWidget*
 prefs_cache_page (void)
 {
-	const int cache_max = gnome_config_get_int_with_default("/Pan/Cache/MaxSize=1242880", NULL);
 	const gboolean flush_on_exit = gnome_config_get_bool("/Pan/Cache/FlushOnExit=true");
 	GtkObject * adj;
 	GtkWidget * frame;
@@ -2604,10 +2687,10 @@ prefs_cache_page (void)
 	gtk_table_attach_defaults (GTK_TABLE(table), label, 0, 1, 0, 1);
 
 	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
-	adj = gtk_adjustment_new (cache_max/(1024*1024), 0.1, 1000, 0.1, 0.1, 0.1);
+	adj = gtk_adjustment_new (acache_max_megs, 0.1, 1000, 0.1, 0.1, 0.1);
 	win->cache_megs_sb = w = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 0.1, 1);
 	connect_signal_to_prefs_changed (w, "changed");
-	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, GNOME_PAD_SMALL);
+	gtk_box_pack_start (GTK_BOX(hbox), w, TRUE, TRUE, 0);
 	label = gtk_label_new (_("Megs"));
 	gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, GNOME_PAD_SMALL);
 	gtk_table_attach_defaults (GTK_TABLE(table), hbox, 1, 2, 0, 1);
