@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,13 +45,15 @@ extern ListClickAction         grouplist_mb1_action;
 extern ListClickAction         grouplist_mb2_action;
 extern ListClickAction         articlelist_mb1_action;
 extern ListClickAction         articlelist_mb2_action;
-/* extern gboolean                break_thread_when_subject_changes; */
+extern gboolean                break_thread_when_subject_changes;
 extern gboolean                fetch_new_on_group_load;
 extern gboolean                display_article_dates_in_local_time;
 extern gboolean                hide_mpart_child_nodes;
 extern gboolean                auto_part_selection;
 extern gboolean                pan_mute;
 extern gboolean                use_gdk_fontset_load;
+extern gchar                 * thread_date_format;
+extern gchar                 * body_date_format;
 extern gchar                 * temp_dir;
 extern gchar                 * data_dir;
 extern gchar                 * download_dir;
@@ -67,6 +69,7 @@ extern gchar                 * mail_server_address;
 extern gchar                 * layout_str;
 extern gint                    mail_server_port;
 extern gint                    wrap_column;
+extern gfloat                  acache_max_megs;
 
 
 /** 
