@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -523,10 +523,14 @@ real_queue_set_max_tries (int i)
 QueueTaskStatus
 queue_get_task_status (const Task* task)
 {
-	QueueTaskStatus status = GPOINTER_TO_INT (
-		g_hash_table_lookup (task_to_status, task));
-	if (!status)
-		status = QUEUE_TASK_STATUS_NOT_QUEUED;
+	QueueTaskStatus status = QUEUE_TASK_STATUS_NOT_QUEUED;
+
+	if (task_to_status != NULL) {
+		gpointer p = g_hash_table_lookup (task_to_status, task);
+		if (p != NULL)
+			status = GPOINTER_TO_INT (p);
+	}
+
 	return status;
 }
 
@@ -679,8 +683,8 @@ queue_get_socket_for_task (Task *task)
 {
 	GSList* sl = NULL;
 	int retval = -1;
-	const gboolean is_leech = TASK(task)->is_leech;
-	gboolean skip_first_idle = task->server->reserve_nonleech && is_leech;
+	const gboolean gets_bodies = TASK(task)->gets_bodies;
+	gboolean skip_first_idle = task->server->reserve_connection_for_bodies && !gets_bodies;
 	gboolean reader_socket_exists = FALSE;
 	Server * server = task->server;
 	int socket_qty;
@@ -757,8 +761,8 @@ queue_get_socket_for_task (Task *task)
 
 		/* leave an article reader socket free */
 		if (!reader_socket_exists
-			&& server->reserve_nonleech
-			&& is_leech
+			&& server->reserve_connection_for_bodies
+			&& !gets_bodies
 			&& max_socket_qty>1
 			&& server->max_connections>1)
 				--slots_open;
@@ -787,7 +791,7 @@ queue_get_socket_for_task (Task *task)
 
 			gui_set_connecting_flag (TRUE);
 			sock = pan_socket_new (server->address, server->port);
-			pan_socket_set_nntp_auth (sock, server->username, server->password); 
+			pan_socket_set_nntp_auth (sock, server->need_auth, server->username, server->password); 
 			pan_object_ref (PAN_OBJECT(sock));
 			pan_object_sink (PAN_OBJECT(sock));
 
@@ -806,7 +810,7 @@ queue_get_socket_for_task (Task *task)
 			}
 			if (errmsg!=NULL)
 			{
-				log_add(errmsg);
+				log_add (LOG_ERROR, errmsg);
 				g_warning(errmsg);
 				if (sock)
 					pan_object_unref (PAN_OBJECT(sock));
@@ -1027,31 +1031,33 @@ sockets_upkeep (void)
 	GSList* l = NULL;
 	GSList* cull = NULL;
 	int socket_qty;
+	debug_enter ("socket_upkeep");
 
 	pthread_mutex_lock (&sock_lock);
 
 	socket_qty = g_slist_length (sockets);
 
-	debug3 (DEBUG_QUEUE,
-		"%s:%d socket_upkeep -- %d sockets",
-		__FILE__, __LINE__, socket_qty);
-
 	for (l=sockets; l!=NULL; l=l->next)
 	{
-		const time_t current_time = time(0);
+		gint age;
+		gboolean destroy, idle, old, too_many, offline;
 		SocketInfo * info;
 		PanSocket * sock;
-		int age;
-		gboolean destroy;
+		const time_t current_time = time(0);
 
 		info = (SocketInfo*) l->data;
 		g_assert (info != NULL);
 		sock = info->socket;
 		g_assert (sock != NULL);
+
+		/* calculate some information about the socket
+		   to decide whether or not we want to keep it */
 		age = current_time - sock->last_action_time;
-		destroy = ((info->task==NULL)
-		       	&& ((age > info->server->idle_secs_before_timeout)
-				|| socket_qty > max_socket_qty));
+		idle = info->task == NULL;
+		old = age > info->server->idle_secs_before_timeout;
+		too_many = socket_qty > max_socket_qty;
+		offline = !info->server->online;
+		destroy = idle && (old||too_many||offline);
 
 		/* if it's idle and we're keeping it, send a keepalive msg.
 		   Note that we don't let this affect the socket's
@@ -1088,14 +1094,11 @@ sockets_upkeep (void)
 		StatusItem * status = STATUS_ITEM(status_item_new (disconnect_describe));
 		const int age = current_time - sock->last_action_time;
 
-		debug4 (DEBUG_QUEUE,
-			"%s:%d socket %p idle for %d seconds.. closing",
-			__FILE__, __LINE__, sock, age);
-
 		/* politely disconnect */
 		status_item_set_active (status, TRUE);
 		nntp_disconnect (status, sock);
 		status_item_set_active (status, FALSE);
+		log_add_va (LOG_INFO, _("Closing connection %p after %d seconds idle"), sock, age);
 
 		/* destroy the socket */
 		pan_object_unref (PAN_OBJECT(sock));
@@ -1113,9 +1116,7 @@ sockets_upkeep (void)
 
 	g_slist_free (cull);
 
-	debug2 (DEBUG_QUEUE,
-		"%s:%d socket_upkeep done",
-		__FILE__, __LINE__);
+	debug_exit ("socket_upkeep");
 }
 
 /**
