@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -96,12 +96,12 @@ save_article (Article * a)
 
 	/* if we don't have the article already, download it */
 	if (!article_has_body (a)) {
-		pan_error_dialog ( "Must load the article first." );
+		pan_error_dialog (_("You must load the article first."));
 		return;
 	}
 
 	msg = article_get_message (a);
-	gfs = gtk_file_selection_new ("Save As...");
+	gfs = gtk_file_selection_new (_("Save As..."));
 	gtk_object_set_data_full (GTK_OBJECT(gfs), "msg", msg, g_free);
 
 	gtk_signal_connect (
@@ -150,7 +150,7 @@ real_save_file (GtkFileSelection *gfs)
 	g_free (tmp);
 
 	/* cleanup */
-	log_add_va ("Saved file \"%s\"", fname);
+	log_add_va (LOG_IMPORTANT, "Saved file \"%s\"", fname);
 	fclose(fp);
 	gtk_widget_destroy (GTK_WIDGET(gfs));
 }
@@ -337,8 +337,8 @@ static void
 save_attachment_as_dialog_destroy_cb (GtkObject    * o,
                                       gpointer       user_data)
 {
+	GSList * l;
 	SaveAsDialogStruct *save = (SaveAsDialogStruct*) user_data;
-	GSList* l;
 
 	/* free the save-as-dialog utility structure */
 	for (l=save->gslists_of_unowned_articles; l!=NULL; l=l->next)
@@ -347,6 +347,8 @@ save_attachment_as_dialog_destroy_cb (GtkObject    * o,
 		/*g_slist_foreach (l2, (GFunc)article_free, NULL);*/
 		g_slist_free (l2);
 	}
+
+	/* cleanup */
 	g_slist_free (save->gslists_of_unowned_articles);
 	g_free (save);
 }
