@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
 
 #include <config.h>
 
+#include <errno.h>
+#include <stdio.h>
 #include <string.h>
 
 #include <glib.h>
@@ -28,17 +30,18 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/libgnomeui.h>
 
-#include "articlelist.h"
-#include "debug.h"
-#include "file-grouplist.h"
-#include "file-headers.h"
-#include "globals.h"
-#include "grouplist.h"
-#include "gui.h"
-#include "prefs.h"
-#include "queue.h"
-#include "server.h"
-#include "util.h"
+#include <pan/articlelist.h>
+#include <pan/debug.h>
+#include <pan/file-grouplist.h>
+#include <pan/file-headers.h>
+#include <pan/globals.h>
+#include <pan/grouplist.h>
+#include <pan/gui.h>
+#include <pan/log.h>
+#include <pan/prefs.h>
+#include <pan/queue.h>
+#include <pan/server.h>
+#include <pan/util.h>
 
 static void server_list_save (void);
 
@@ -62,8 +65,10 @@ server_name_compare (gconstpointer a, gconstpointer b)
 static void
 server_folders_load (void)
 {
-	Server * server = server_get_by_name (INTERNAL_SERVER_NAME);
+	Server * server;
+	debug_enter ("server_folders_load");
 
+       	server = server_get_by_name (INTERNAL_SERVER_NAME);
 	if (server == NULL) /* new user; create the folder server */
 	{
 		/* add the Folder Server */
@@ -87,7 +92,7 @@ server_folders_load (void)
 			_("Articles/Mail queued for posting via the \"Send Later\" button"));
 		group_set_is_folder (group, TRUE);
 		server_add_groups (server, &group, 1, NULL, NULL);
-		g_message (_("Created folder pan.sendlater"));
+		log_add (LOG_INFO, _("Created folder pan.sendlater"));
 	}
 
 	if (folder_get_by_name(PAN_SENT) == NULL)
@@ -98,8 +103,10 @@ server_folders_load (void)
 			_("Articles/Mail posted before"));
 		group_set_is_folder (group, TRUE);
 		server_add_groups (server, &group, 1, NULL, NULL);
-		g_message (_("Created folder pan.sent"));
+		log_add (LOG_INFO, _("Created folder pan.sent"));
 	}
+
+	debug_exit ("server_folders_load");
 }
 
 static Server*
@@ -152,8 +159,8 @@ server_fetch (const char *name)
 				gnome_config_get_int ("Idle_Timeout_Secs=180");
 			server->online =
 				gnome_config_get_bool ("Online=TRUE");
-			server->reserve_nonleech =
-				gnome_config_get_bool ("Reserve_Nonleech_Connection=TRUE");
+			server->reserve_connection_for_bodies =
+				gnome_config_get_bool ("Reserve_Reader_Connection=TRUE");
 			/* no good way to get a ulong from gnome_config? */
 			server->last_newgroup_list_time = 1;
 			pch = gnome_config_get_string ("Last_Grouplist_Update");
@@ -248,6 +255,7 @@ server_get_by_name (const char* servername)
 {
 	GSList * l;
 	Server * retval = NULL;
+	debug_enter ("server_get_by_name");
 
 	g_return_val_if_fail (is_nonempty_string(servername), NULL);
 
@@ -259,6 +267,7 @@ server_get_by_name (const char* servername)
 			retval = s;
 	}
 
+	debug_exit ("server_get_by_name");
 	return retval;
 }
 
@@ -284,7 +293,7 @@ server_constructor (Server              * server,
 	server->need_auth = FALSE;
 	server->online = FALSE;
 	server->online_prompted = FALSE;
-	server->reserve_nonleech = TRUE;
+	server->reserve_connection_for_bodies = TRUE;
 	server->address = NULL;
 	server->name = NULL;
 	server->username = NULL;
@@ -374,6 +383,7 @@ static gchar * first_server_name = NULL;
 static GnomeUIInfo * server_menu = NULL;
 static GnomeUIInfo * server_menu_radiobox = NULL;
 static GnomeUIInfo end = GNOMEUIINFO_END;
+static GHashTable * server_to_menubutton = NULL;
 
 void
 server_menu_update (void)
@@ -387,6 +397,9 @@ server_menu_update (void)
 	int i;
 
 	/* build the new menu */
+	if (server_to_menubutton != NULL)
+		g_hash_table_destroy (server_to_menubutton);
+	server_to_menubutton = g_hash_table_new (g_direct_hash, g_direct_equal);
 	l = Pan.serverlist;
 	buf = g_new0 (GnomeUIInfo, g_slist_length(l)+2);
 	i = 0;
@@ -409,6 +422,7 @@ server_menu_update (void)
 		if (!strncmp (s->name, INTERNAL_SERVER_NAME, strlen(INTERNAL_SERVER_NAME)))
 			continue;
 
+		g_hash_table_insert (server_to_menubutton, s, info);
 		info->type = GNOME_APP_UI_ITEM;
 		info->label = s->name;
 		info->hint = s->name;
@@ -461,46 +475,51 @@ server_menu_update (void)
 *****
 ****/
 
+static Server*
+get_default_server (void)
+{
+	GSList * l;
+	gchar * name;
+	Server * retval = NULL;
+
+	g_return_val_if_fail (Pan.serverlist!=NULL, NULL);
+
+	/* look in the gnome config for a default */
+	name = gnome_config_get_string ("/Pan/State/Server");
+	if (is_nonempty_string (name))
+		retval = server_get_by_name (name);
+	g_free (name);
+
+	/* otherwise just use the first server */
+	for (l=Pan.serverlist; retval==NULL && l!=NULL; l=l->next) {
+		Server * s = SERVER(l->data);
+		if (strcmp (s->name, INTERNAL_SERVER_NAME))
+			retval = s;
+	}
+
+	/* return what we've got */
+	return retval;
+}
+
 void
 server_init (void)
 {
-	gchar *last = NULL;
-	GSList *l = NULL;
-	int i;
+	GnomeUIInfo * menu_info = NULL;
+	Server * s = NULL;
 
 	/* This gets called before gtk_main and
 	 * at the end of guru_finish */
 	server_init_servers ();
 	server_menu_update ();
 
-	/* try hard to select a server.. if there's one in the ini file, use
-	 * it; otherwise, go with the first one in the list (if there is one)
-	 */
-	last = gnome_config_get_string ("/Pan/State/Server");
-	if (!last && Pan.serverlist)
-		last = g_strdup ((SERVER(Pan.serverlist->data)->name));
-	if (last!=NULL)
-	{
-		for (i=0, l=Pan.serverlist; l; l=l->next)
-		{
-			Server *server = SERVER(l->data);
-
-			/* skip 'invisible' servers used for folders, etc. */
-			if (!strncmp (server->name, INTERNAL_SERVER_NAME, strlen(INTERNAL_SERVER_NAME)))
-				continue;
-
-			if (!strcmp(server->name, last)) {
-				gtk_check_menu_item_set_active (
-		                	GTK_CHECK_MENU_ITEM(
-						server_menu[i].widget), TRUE);
-				grouplist_set_server (server);
-				break;
-			}
-
-			++i;
-		}
-		g_free (last);
+	/* if we've got a default menu, update the UI. */
+	s = get_default_server ();
+	if (s != NULL) {
+		menu_info = g_hash_table_lookup (server_to_menubutton, s);
+		grouplist_set_server (s);
 	}
+	if (menu_info != NULL)
+		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(menu_info->widget), TRUE);
 }
 
 void
@@ -575,8 +594,8 @@ server_list_save (void)
 		gnome_config_set_string (buf->str, pch);
 		g_free (pch);
 
-		g_string_sprintf (buf, "/Pan/%s/Reserve_Nonleech_Connection", s->name);
-		gnome_config_set_bool (buf->str, s->reserve_nonleech);
+		g_string_sprintf (buf, "/Pan/%s/Reserve_Reader_Connection", s->name);
+		gnome_config_set_bool (buf->str, s->reserve_connection_for_bodies);
 
 		if (servers->len)
 			g_string_append_c (servers, ',');
@@ -648,6 +667,7 @@ import_newsrc (Server * server, const gchar * filename)
 	const gchar * foo;
 	GPtrArray * addme;
 	GArray * body = read_file (filename);
+	debug_enter ("import_newsrc");
 
 	if (!body)
 		return;
@@ -696,6 +716,7 @@ import_newsrc (Server * server, const gchar * filename)
 	g_array_free (body, TRUE);
 	g_string_free (line, TRUE);
 	g_ptr_array_free (addme, TRUE);
+	debug_exit ("import_newsrc");
 }
 
 static void
@@ -758,10 +779,15 @@ server_import_newsrc_dialog (Server * server)
 static void
 export_newsrc (Server * server, const gchar * filename)
 {
-	FILE * fp = fopen (filename, "w");
-	if (fp != NULL)
+	FILE * fp = fopen (filename, "w+");
+	if (fp == NULL)
 	{
-		int i;
+		log_add_va (LOG_ERROR, _("Couldn't open file `%s': %s"),
+		                       filename, g_strerror(errno));
+	}
+	else
+	{
+		gint i;
 		GString * s = g_string_sized_new (4096);
 		for (i=0; i<server->groups->len; ++i)
 		{
@@ -773,7 +799,7 @@ export_newsrc (Server * server, const gchar * filename)
 		}
 
 		fclose (fp);
-		g_message (_("%d groups written to `%s'"), i, filename);
+		log_add_va (LOG_INFO, _("%d groups written to `%s'"), i, filename);
 	}
 }
 
@@ -862,94 +888,63 @@ server_add_groups (Server       * server,
                    GPtrArray    * fillme_used,
                    GPtrArray    * fillme_not_used)
 {
-	guint i, j;
-	GPtrArray * old;
-	GPtrArray * tmp;
-	Group** merged = NULL;
-	int merged_qty = 0;
+	gint i;
+	GHashTable * old;
 	gboolean dirty = FALSE;
+	debug_enter ("server_add_groups");
 
 	/* entry assertions */
 	g_return_if_fail (server != NULL);
 	g_return_if_fail (groups != NULL);
 	g_return_if_fail (group_qty >= 1);
 
-	/* make a temporary sorted array of the groups being added */
-	tmp = g_ptr_array_new ();
-	pan_g_ptr_array_assign (tmp, (gpointer*)groups, group_qty);
-	pan_g_ptr_array_sort (tmp, compare_ppgroup_ppgroup_name);
+	/* convert the existing grouplist to a hashtable */
+	old = g_hash_table_new (g_str_hash, g_str_equal);
+	for (i=0; i<server->groups->len; ++i) {
+		Group * group = GROUP(g_ptr_array_index(server->groups,i));
+		g_hash_table_insert (old, (gpointer)group->name, (gpointer)group);
+	}
 
-	/* so now we have two sorted lists; merge them. */
-	old = server->groups;
-	merged = g_new (Group*, old->len + tmp->len);
-	for (i=j=0; i!=old->len || j!=tmp->len; )
+	/* try to add the new groups */
+	for (i=0; i<group_qty; ++i)
 	{
-		Group * addme = NULL;
+		Group * new_group = groups[i];
+		gboolean has_group = g_hash_table_lookup (old, new_group->name) != NULL;
 
-		if (i==old->len)
+		if (!has_group)
 		{
-			addme = g_ptr_array_index (tmp, j++);
 			dirty = TRUE;
-			if (fillme_used)
-				g_ptr_array_add (fillme_used, addme);
-		}
-		else if (j==tmp->len)
-		{
-			addme = g_ptr_array_index (old, i++);
+
+			/* update the group's server field */
+			if (new_group->server != server) {
+				server_remove_group (new_group->server, new_group);
+				new_group->server = server;
+			}
+
+			/* add the group */
+			g_hash_table_insert (old, (gpointer)new_group->name, new_group);
+			if (fillme_used != NULL)
+				g_ptr_array_add (fillme_used, new_group);
 		}
 		else
 		{
-			Group * g1 = GROUP(g_ptr_array_index(tmp,j));
-			Group * g2 = GROUP(g_ptr_array_index(old,i));
-			const int comp = pan_strcmp (g1->name, g2->name);
-
-			if (comp<0)
-			{
-				addme = g1;
-				dirty = TRUE;
-				if (fillme_used)
-					g_ptr_array_add (fillme_used, addme);
-				++j;
-			}
-			else
-			{
-				if (g2->permission != g1->permission) {
-					g2->permission = g1->permission;
-					dirty = TRUE;
-				}
-				addme = g2;
-				++i;
-				if (!comp && g1!=g2 && fillme_not_used)
-				{
-					g_ptr_array_add (fillme_not_used, g1);
-					++j;
-				}
-			}
-		}
-
-		if (addme != NULL)
-		{
-			if (addme->server != server)
-			{
-				server_remove_group (addme->server, addme);
-				addme->server = server;
-			}
-
-			merged[merged_qty++] = addme;
+			if (fillme_not_used != NULL)
+				g_ptr_array_add (fillme_not_used, new_group);
 		}
 	}
 
-	/* if any change was made, update everything */
+	/* if any change was made, update the groups array */
 	if (dirty)
 	{
-		pan_g_ptr_array_assign (old, (gpointer*)merged, merged_qty);
+		pan_hash_to_ptr_array  (old, server->groups);
+		qsort (server->groups->pdata, server->groups->len, sizeof(gpointer), compare_ppgroup_ppgroup_name);
 		server->groups_dirty = TRUE;
 		grouplist_update_ui (server);
 	}
 
 	/* cleanup */
-	g_free (merged);
-	g_ptr_array_free (tmp, TRUE);
+	g_hash_table_destroy (old);
+	debug_exit ("server_add_groups");
 }
 
 
@@ -958,6 +953,7 @@ server_remove_group (Server * server, Group * group)
 {
 	guint index;
 	gboolean exact_match = FALSE;
+	debug_enter ("server_remove_group");
 
 	g_return_if_fail (server != NULL);
 	g_return_if_fail (group != NULL);
@@ -985,6 +981,8 @@ server_remove_group (Server * server, Group * group)
 		pan_object_unref (PAN_OBJECT(g));
 		server->groups_dirty = TRUE;
 	}
+
+	debug_exit ("server_remove_group");
 }
 
 Group*
@@ -992,6 +990,7 @@ group_get_by_name (Server        * server,
 	           const char    * group_name)
 {
 	Group * retval = NULL;
+	debug_enter ("group_get_by_name");
 
 	g_return_val_if_fail (server!=NULL, NULL);
 	g_return_val_if_fail (is_nonempty_string(group_name), NULL);
@@ -1013,6 +1012,7 @@ group_get_by_name (Server        * server,
 			retval = GROUP(g_ptr_array_index(server->groups,index));
 	}
 
+	debug_exit ("group_get_by_name");
 	return retval;
 }
 
