@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,10 +43,10 @@ typedef struct
 	guint need_auth            : 1;  /* need authorization ? */
 	guint online               : 1;  /* is this server online? */
 	guint online_prompted      : 1;  /* true if user's been asked to go online for some tasks. */
-	guint reserve_nonleech     : 1;
+	guint reserve_connection_for_bodies     : 1;
 	gint8 max_connections;           /* max connections to server */
         gint16 idle_secs_before_timeout;
-	gint16 port;	                 /* #, NNTP usually 119, SMTP usually 25 */
+	gint port;	                 /* port number.  This is usually 119 for NNTP. */
 	time_t last_newgroup_list_time;
 	gchar * address;                 /* server net address */
 	gchar * name;	                 /* pretty name, i.e. Erols */
