@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -389,8 +389,15 @@ smtp_handshake (StatusItem   * item,
 
 	if (smtp_send_line_va (item, sock, "HELO %s\r\n", my_name))
 		return -1;
-	if (!pan_mute && smtp_get_response (item, sock, 250) != 250)
-		return -1;
+
+	if (!pan_mute) {
+		for (;;) {
+			gint val = smtp_get_response (item, sock, 250);
+			if (val == 220) /* apparently the welcome message is wordy */
+				continue;
+			return val==250 ? 0 : -1;
+		}
+	}
 
 	return 0;
 }
