@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,6 +44,7 @@
 #include <libgnome/gnome-i18n.h>
 
 #include "debug.h"
+#include "log.h"
 #include "sockets.h"
 #include "util.h"
 
@@ -51,7 +52,7 @@
 **********************  Defines / Enumerated types
 *********************/
 
-#define BUFSIZE 2048
+#define BUFSIZE 1024
 
 /*********************
 **********************  Macros
@@ -148,6 +149,7 @@ pan_socket_constructor (PanSocket             * sock,
 	sock->inptr = sock->inbuf;
 	sock->line_buffer = g_array_new (TRUE, FALSE, 1);
 	sock->error = FALSE;
+	sock->need_auth = FALSE;
 	sock->nntp_group_name = NULL;
 	sock->nntp_username = NULL;
 	sock->nntp_password = NULL;
@@ -171,13 +173,9 @@ pan_socket_constructor (PanSocket             * sock,
 	{
 		while (NULL==(hp = gethostbyname(address)))
 		{
-			const char* error_str = NULL;
-			if (errno)
-				error_str = strerror(errno);
-			g_warning (
-				"Can't resolve %s -- PanSocket not created\n%s",
-				address,
-				(error_str ? error_str : ""));
+			const char* error_str = errno ? strerror(errno) : "";
+			log_add_va (LOG_ERROR, _("Can't resolve %s: %s"),
+			            address, error_str);
 			return;
 		}
 		memcpy(&serv_addr.sin_addr, hp->h_addr, hp->h_length);
@@ -185,14 +183,16 @@ pan_socket_constructor (PanSocket             * sock,
 
 	sock->sockfd = socket(AF_INET, SOCK_STREAM, 0);
 	if (sock->sockfd < 0) {
-		g_warning ("Can't create socket: %s", strerror(errno));
+		const char* error_str = errno ? strerror(errno) : "";
+		log_add_va (LOG_ERROR, _("Can't connect to %s: %s"),
+		            address, error_str);
 		return;
 	}
 
 	if (connect (sock->sockfd, (struct sockaddr*)&serv_addr, sizeof(serv_addr))<0)
 	{
 		close (sock->sockfd);
-		g_warning ("socket connect to %s, port %d failed: %s", address, port, strerror(errno));
+		log_add_va (LOG_ERROR, _("Connection to %s, port %d failed: %s"), address, port, strerror(errno));
 		return;
 	}
 
@@ -201,8 +201,7 @@ pan_socket_constructor (PanSocket             * sock,
 	                (char*)&on, sizeof(on)) < 0)
 	{
 		close(sock->sockfd);
-		g_warning ("socket keepalive option failed: %s",
-			   strerror(errno));
+		log_add_va (LOG_ERROR, _("Socket keepalive option failed: %s"), strerror(errno));
 		return;
 	}
 
@@ -213,9 +212,7 @@ pan_socket_constructor (PanSocket             * sock,
 	act.sa_flags = 0;
 	sigaction (SIGPIPE, &act, &oact);
 
-	g_message ("Pan: socket created: %p for %s, port %d, sockfd %d", sock, address, port, sock->sockfd);
-	debug3 (DEBUG_SOCKET,
-	        "socket created: %p for %s, port %d", sock, address, port);
+	log_add_va (LOG_INFO, _("New connection %p for %s, port %d, sockfd %d"), sock, address, port, sock->sockfd);
         debug1 (DEBUG_PAN_OBJECT, "pan_socket_constructor: %p", sock);
 }
 
@@ -246,11 +243,13 @@ pan_socket_destructor (PanObject *obj)
  
 void
 pan_socket_set_nntp_auth (PanSocket            * socket,
+                          gboolean               need_auth,
                           const gchar          * username,
                           const gchar          * password)
 {
 	g_return_if_fail (socket!=NULL);
 
+	socket->need_auth = need_auth;
 	replace_gstr (&socket->nntp_username, g_strdup(username));
 	replace_gstr (&socket->nntp_password, g_strdup(password));
 }
@@ -283,11 +282,13 @@ read_from_socket (PanSocket* ps)
 
 	/* did select fail? */
 	if (retval<=0) {
-		if (errno)
-			g_message (_("Timed out waiting to read from the server: %s"), g_strerror(errno));
-		else
-			g_message (_("Timed out waiting to read from the server."));
+		GString * str = g_string_new (NULL);
+		g_string_sprintf (str, _("Timed out waiting to read from the server."));
+		if (errno != 0)
+			g_string_sprintfa (str, " %s", strerror(errno));
+		log_add (LOG_ERROR, str->str);
 		ps->error = TRUE;
+		g_string_free (str, TRUE);
 		return -1;
 	}
 
@@ -299,16 +300,18 @@ read_from_socket (PanSocket* ps)
 
 	/* did the server hang up? */
 	if (read_count == 0) {
-		g_message (_("Lost connection to server"));
+		log_add (LOG_ERROR, _("Lost connection to server"));
 		return -1;
 	}
 
 	/* did we encounter some other error? */
 	if (read_count < 0) {
-		if (errno)
-			g_message (_("Error: reading from socket returned %d: %s"), read_count, g_strerror(errno));
-		else
-			g_message (_("Error: reading from socket returned %d"), read_count);
+		GString * str = g_string_new (NULL);
+		g_string_sprintf (str, _("Error: reading from socket returned %d."), read_count);
+		if (errno != 0)
+			g_string_sprintfa (str, "  %s", g_strerror(errno));
+		log_add (LOG_ERROR, str->str);
+		g_string_free (str, TRUE);
 		return read_count;
 	}
 
@@ -424,7 +427,7 @@ pan_socket_putline (PanSocket     * sock,
 		nwritten = write (sock->sockfd, pch, nleft);
 		sock->bytes_written += nwritten;
 		if (nwritten == -1) {
-			g_message (_("An error occurred writing to the server."));
+			log_add (LOG_ERROR, _("An error occurred writing to the server."));
 			sock->error = TRUE;
 			return -1;
 		}
