@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,6 +41,7 @@ struct _PanSocket
 	gboolean error;
 
 	/* private fields */  
+	gboolean need_auth;
 	gint sockfd;
 	gchar * inbuf;
 	gchar * endbuf;
@@ -76,6 +77,7 @@ PanSocket*   pan_socket_new                  (const gchar          * address,
                                               int                    port);
 
 void         pan_socket_set_nntp_auth        (PanSocket            * socket,
+                                              gboolean               need_auth,
                                               const gchar          * username,
                                               const gchar          * password);
 /**
