@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -64,7 +64,6 @@ static void status_item_view_unregister (StatusItemView *view);
 
 static int progress_cb      (gpointer, gpointer, gpointer);
 static int status_cb        (gpointer, gpointer, gpointer);
-static int error_cb         (gpointer, gpointer, gpointer);
 
 /*********************
 **********************  Variables
@@ -74,6 +73,8 @@ static int error_cb         (gpointer, gpointer, gpointer);
 ************  Extern
 ***********/
 
+extern GtkTooltips * ttips;                                                                                      
+
 /***********
 ************  Public
 ***********/
@@ -84,8 +85,6 @@ static int error_cb         (gpointer, gpointer, gpointer);
 
 static GtkObjectClass *parent_class = NULL;
 
-static GtkTooltips *tips = NULL;
-
 /*********************
 **********************  BEGINNING OF SOURCE
 *********************/
@@ -133,8 +132,6 @@ status_item_view_class_init (StatusItemViewClass *klass)
 
 	object_class->destroy = status_item_view_destroy;
 	object_class->shutdown = status_item_view_shutdown;
-
-	tips = gtk_tooltips_new ();
 }
 
 static void
@@ -203,14 +200,6 @@ status_item_view_new (void)
 	return GTK_WIDGET(view);
 }
 
-GtkWidget*
-status_item_view_new_with_item (StatusItem * item)
-{
-	StatusItemView * l = STATUS_ITEM_VIEW(status_item_view_new());
-	status_item_view_set_item (l, item);
-	return GTK_WIDGET(l);
-}
-
 /*****
 ******
 *****/
@@ -227,7 +216,7 @@ status_item_view_unregister (StatusItemView *view)
 	/* stop listening to the item... */
 	pan_callback_remove (item->progress, progress_cb, view);
 	pan_callback_remove (item->status, status_cb, view);
-	pan_callback_remove (item->error, error_cb, view);
+	pan_callback_remove (item->error, status_cb, view);
 
 	/* unref the item... */
 	pan_object_unref(PAN_OBJECT(item));
@@ -248,15 +237,16 @@ status_item_view_set_item (StatusItemView* view, StatusItem* item)
 		replace_gstr (&view->last_status, g_strdup(""));
 		replace_gstr (&prog->format, g_strdup(""));
 		GTK_PROGRESS_CLASS (GTK_OBJECT(prog)->klass)->update (prog);
-		gtk_tooltips_set_tip (tips, GTK_WIDGET(view->eventbox), NULL, NULL);
+		gtk_tooltips_set_tip (ttips, GTK_WIDGET(view->eventbox), NULL, NULL);
 
-		if (item != NULL) {
+		if (item != NULL)
+		{
 			gchar * description = status_item_describe (item);
 			pan_object_ref (PAN_OBJECT(item));
 			pan_callback_add (item->progress, progress_cb, view);
 			pan_callback_add (item->status, status_cb, view);
-			pan_callback_add (item->error, error_cb, view);
-			gtk_tooltips_set_tip (tips, GTK_WIDGET(view->eventbox), description, NULL);
+			pan_callback_add (item->error, status_cb, view);
+			gtk_tooltips_set_tip (ttips, GTK_WIDGET(view->eventbox), description, NULL);
 			g_free (description);
 		}
 
@@ -322,21 +312,6 @@ status_cb (gpointer call_object,
 	return 0;
 }
 
-static int
-error_cb (gpointer call_object,
-          gpointer call_arg,
-          gpointer user_data)
-{
-	const gchar *error = (const gchar*)call_arg;
-	StatusItemView *v = STATUS_ITEM_VIEW(user_data);
-
-	g_warning ("%s",error);
-	debug1 (DEBUG_QUEUE,"Error: %s", error);
-	status_cb (NULL, (gpointer)error, v);
-
-	return 0;
-}
-
 /**
 ***
 **/
