@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,14 +27,11 @@
 #include <string.h>
 
 #include <glib.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
 
 #include "debug.h"
 #include "gui.h"
 #include "log.h"
 #include "status-item.h"
-#include "util.h"
 
 /*********************
 **********************  Defines / Enumerated types
@@ -82,14 +79,20 @@ static PanCallback * status_item_active_callback = NULL;
 ******
 *****/
 
-char*
-status_item_describe (const StatusItem* item)
+gchar*
+status_item_describe (const StatusItem* status)
 {
-	g_return_val_if_fail (item!=NULL, NULL);
-	return (*item->describe)(item);
+	gchar * pch = NULL;
+
+	if (status != NULL)
+		pch = (*status->describe)(status);
+	if (pch == NULL)
+		pch = g_strdup ("");
+
+	return pch;
 }
 
-StatusItem *
+StatusItem*
 status_item_new (StatusItemDescribeFunc describe)
 {
 	StatusItem *item = g_new (StatusItem, 1);
@@ -176,7 +179,7 @@ status_item_set_active (StatusItem       * item,
 
 void
 status_item_emit_init_steps (StatusItem  * item,
-                             int           steps)
+                             gint          steps)
 {
 	g_return_if_fail (item != NULL);
 	g_return_if_fail (steps >= 0);
@@ -195,7 +198,7 @@ status_item_emit_next_step (StatusItem * item)
 
 void
 status_item_emit_set_step (StatusItem    * item,
-                           int             step)
+                           gint            step)
 {
 	g_return_if_fail (item != NULL);
 	g_return_if_fail (step >= 0);
@@ -209,7 +212,7 @@ status_item_emit_set_step (StatusItem    * item,
 
 void
 status_item_emit_progress (StatusItem    * item,
-                           int             out_of_100)
+                           gint            out_of_100)
 {
 	g_return_if_fail (item != NULL);
 
@@ -225,33 +228,10 @@ status_item_emit_progress (StatusItem    * item,
 	}
 }
 
-void
-status_item_emit_next_step_inside_lock (StatusItem * item)
-{
-        int out_of_100;
-
-        /* increment the step */
-        ++item->step;
-
-        /* if the progress changed, unlock and fire an update */
-        out_of_100 = (int)(item->steps?((item->step*100.0)/item->steps):0);
-        if (0<=out_of_100
-                && out_of_100<=100
-                && item->progress_val!=out_of_100)
-        {
-                gpointer p = GINT_TO_POINTER(out_of_100);
-                item->progress_val = out_of_100;
-
-                pan_unlock ();
-                pan_callback_call (item->progress, item, p);
-                pan_lock ();
-        }
-}
-
 
 void
 status_item_emit_done (StatusItem * item,
-                       int          status)
+                       gint         status)
 {
 	g_return_if_fail (item != NULL);
 	pan_callback_call (item->done, item, GINT_TO_POINTER(status));
@@ -259,7 +239,7 @@ status_item_emit_done (StatusItem * item,
 
 void
 status_item_emit_status (StatusItem    * item,
-                         const char    * status)
+                         const gchar   * status)
 {
 	g_return_if_fail (item!=NULL);
 	g_return_if_fail (status!=NULL);
@@ -270,7 +250,7 @@ status_item_emit_status (StatusItem    * item,
 
 void
 status_item_emit_status_va (StatusItem    * item,
-                            const char    * format,
+                            const gchar   * format,
                             ...)
 {
 	char *pch = NULL;
@@ -288,8 +268,8 @@ status_item_emit_status_va (StatusItem    * item,
 }
 
 void
-status_item_emit_error (StatusItem  * item,
-                        const char  * error)
+status_item_emit_error (StatusItem   * item,
+                        const gchar  * error)
 {
 	gchar * copy;
 
@@ -297,15 +277,15 @@ status_item_emit_error (StatusItem  * item,
 	g_return_if_fail (error != NULL);
 
 	copy = g_strdup (error);
-	g_message (copy);
+	log_add (LOG_ERROR, copy);
 	debug2 (DEBUG_QUEUE, "StatusItem %p has this error: %s", item, copy);
 	item->errors = g_slist_append (item->errors, copy);
 	pan_callback_call (item->error, item, copy);
 }
 
 void
-status_item_emit_error_va (StatusItem  * item,
-                           const char  * fmt,
+status_item_emit_error_va (StatusItem   * item,
+                           const gchar  * fmt,
                            ...)
 {
 	char *pch = NULL;
@@ -322,14 +302,7 @@ status_item_emit_error_va (StatusItem  * item,
 	g_free (pch);
 }
 
-void
-status_item_next_step_gfunc (gpointer group,
-                             gpointer p_status_item)
-{
-	status_item_emit_next_step (STATUS_ITEM(p_status_item));
-}
-
-int
+gint
 status_item_get_progress_of_100 (const StatusItem * item)
 {
 	g_return_val_if_fail (item!=NULL, 0);
