@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,9 +47,9 @@ struct _StatusItem
 
 	/*Fields*/
 	GSList      * errors;         /* the emitted error strings */
-	int           progress_val;   /* value is [0...100] */
-	int           steps;          /* number of steps for progress */
-	int           step;           /* number of steps so far. [0..steps] */
+	gint          progress_val;   /* value is [0...100] */
+	gint          steps;          /* number of steps for progress */
+	gint          step;           /* number of steps so far. [0..steps] */
 
 	/* virtual functions */
 	StatusItemDescribeFunc describe;
@@ -79,47 +79,43 @@ PanCallback*  status_item_get_active_callback  (void);
 
 StatusItem*   status_item_new                  (StatusItemDescribeFunc);
 
-char*         status_item_describe             (const StatusItem * item);
+gchar*        status_item_describe             (const StatusItem * item);
 
 void          status_item_emit_status          (StatusItem       * item,
-                                                const char       * status);
+                                                const gchar      * status);
 
 void          status_item_emit_status_va       (StatusItem       * item,
-                                                const char       * fmt,
+                                                const gchar      * fmt,
                                                 ...);
 
 void          status_item_emit_error           (StatusItem       * item,
-                                                const char       * error);
+                                                const gchar      * error);
 
 void          status_item_emit_error_va        (StatusItem       * item,
-                                                const char       * fmt,
+                                                const gchar      * fmt,
                                                 ...);
 
 void          status_item_emit_progress        (StatusItem       * item,
-                                                int                out_of_100);
+                                                gint               out_of_100);
 
 void          status_item_emit_init_steps      (StatusItem       * item,
-                                                int                steps);
+                                                gint               steps);
 
 void          status_item_emit_next_step       (StatusItem       * item);
 
-void          status_item_emit_next_step_inside_lock (StatusItem * item);
-
 void          status_item_emit_set_step        (StatusItem       * item,
-                                                int                step);
+                                                gint               step);
 
 void          status_item_emit_activity        (StatusItem       * item,
-                                                int                n);
+                                                gint               n);
 
 void          status_item_emit_done            (StatusItem       * item,
-                                                int                status);
+                                                gint               status);
 
 void          status_item_set_active           (StatusItem       * item,
                                                 gboolean           active);
 
-int           status_item_get_progress_of_100  (const StatusItem * item);
+gint          status_item_get_progress_of_100  (const StatusItem * item);
 
-void          status_item_next_step_gfunc      (gpointer           group,
-                                                gpointer           item);
 
 #endif /* __PAN_STATUS_ITEM__H__ */
