@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,8 +42,8 @@ TaskBodies;
 ***  PUBLIC
 **/
 
-PanObject*   task_bodies_new   (Group       * group,
-                                GPtrArray   * articles);
+PanObject*   task_bodies_new   (Group             * group,
+                                const GPtrArray   * articles);
 
 
 #endif /* __TASK_BODY__H__ */
