@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -304,12 +304,8 @@ nntp_grouplist_download_all (TaskGroupList * item)
 		if (!read_err)
 			item->got_list = TRUE;
 
-		if (read_err) {
-			/*FIXME: prompt whether or not to keep the list.
-			  For now, leak. */
-			task_emit_sockread_err (TASK(item));
+		if (read_err)
 			return TASK_FAIL;
-		}
 	}
 
 	/* sort so that we can bsearch them... */
@@ -398,12 +394,11 @@ nntp_grouplist_download_all (TaskGroupList * item)
 		g_free (description);
 	}
 	if (read_err)
-	{
-		task_emit_sockread_err (TASK(item));
 		return TASK_FAIL;
-	}
 
-	log_add_va (_("Got %d groups"), groups->len);
+	log_add_va (LOG_INFO, _("Got %d groups for server `%s'"),
+		groups->len,
+		server->name);
 
 	return TASK_SUCCESS;
 }
@@ -458,10 +453,8 @@ nntp_grouplist_download_new (TaskGroupList * item)
 	**/
 
 	if (*abort || pan_socket_getline (sock, &response))
-	{
-		task_emit_sockread_err (TASK(item));
 		return TASK_FAIL;
-	}
+
 	while (strncmp(response, ".\r\n", 3))
 	{
 		gchar * name;
@@ -485,13 +478,12 @@ nntp_grouplist_download_new (TaskGroupList * item)
 
 		if (*abort || pan_socket_getline (sock, &response))
 		{
-			task_emit_sockread_err (TASK(item));
 			pan_g_ptr_array_foreach (groups, (GFunc)pan_object_unref, NULL);
 			return TASK_FAIL;
 		}
 	}
 
-	log_add_va (_("Got %d new groups"), count);
+	log_add_va (LOG_INFO, _("Got %d new groups"), count);
 
 	status_item_emit_status_va (STATUS_ITEM(item),
 		_("Got %d news groups for \"%s\""), count, server->name);
