@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -51,10 +51,6 @@
 #include "xpm/delete_sheet.xpm"
 #include "xpm/requeue.xpm"
 #include "xpm/server.xpm"
-#if 0 /* soon, soon */
-#include "xpm/queue_load.xpm"
-#include "xpm/queue_save.xpm"
-#endif
 
 /*********************
 **********************  Defines / Enumerated types
@@ -547,12 +543,18 @@ periodic_update_timer (gpointer user_data)
 	return 1;
 }
 
+static gint
+window_delete_event_cb (GtkWidget * w, GdkEvent * e, gpointer data)
+{
+	ManagerUI * ui = (ManagerUI*) data;
+	gui_save_window_size (ui->window, "task_manager_2");
+	gui_save_column_widths (ui->list, "task_manager_2");
+	return FALSE;
+} 
 static void
-ui_destroy_cb (GtkObject   * o,
-               gpointer      user_data)
+ui_destroy_cb (GtkObject * o, gpointer user_data)
 {
 	ManagerUI *ui = (ManagerUI*) user_data;
-	gui_save_column_widths (ui->list, "task_manager");
 
 	pan_callback_remove (queue_task_added, task_added_cb, ui);
 	pan_callback_remove (queue_task_removed, task_removed_cb, ui);
@@ -671,8 +673,8 @@ task_manager_build_ui (void)
 	list_titles[4] = _("Description");
 	ui->list = w = gtk_clist_new_with_titles (5, list_titles);
 	gtk_clist_set_column_width (GTK_CLIST(w), 0, 200);
-
-	gui_restore_column_widths (ui->list, "task_manager");
+	gtk_clist_set_column_width (GTK_CLIST(w), 4, 1500);
+	gui_restore_column_widths (ui->list, "task_manager_2");
 
 	gtk_clist_set_selection_mode (GTK_CLIST(w), GTK_SELECTION_EXTENDED);
 	gtk_signal_connect (GTK_OBJECT(w), "row_move", queue_row_move_cb, ui);
@@ -704,7 +706,6 @@ update_titlebar_cb (gpointer call_obj,
 	((ManagerUI*)(user_data))->titlebar_needs_refresh = TRUE;
 	return 0;
 }
-
 static void
 remove_update_titlebar_callbacks_cb (GtkObject * o, gpointer user_data)
 {
@@ -739,11 +740,15 @@ task_manager_spawn (void)
 	pan_callback_add (queue_task_added, update_titlebar_cb, manager_ui);
 	pan_callback_add (queue_task_removed, update_titlebar_cb, manager_ui);
 	pan_callback_add (queue_task_status_changed, update_titlebar_cb, manager_ui);
-	gtk_signal_connect (GTK_OBJECT(manager_ui->window), "destroy", remove_update_titlebar_callbacks_cb, manager_ui);
-
+	gtk_signal_connect (GTK_OBJECT(manager_ui->window), "delete_event",
+	                    GTK_SIGNAL_FUNC(window_delete_event_cb), manager_ui);
+	gtk_signal_connect (GTK_OBJECT(manager_ui->window), "destroy",
+	                    GTK_SIGNAL_FUNC(remove_update_titlebar_callbacks_cb), manager_ui);
 	gtk_window_set_title (GTK_WINDOW(manager_ui->window), _("Pan - Task Manager"));
 	gtk_container_add (GTK_CONTAINER(manager_ui->window), manager_ui->top);
+
 	gtk_window_set_default_size (GTK_WINDOW (manager_ui->window), 600, 400);
+	gui_restore_window_size (manager_ui->window, "task_manager_2");
 	gtk_widget_show_all (manager_ui->window);
 }
 
