@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001 Pan Development Team (pan@rebelbase.com)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,12 +29,19 @@
 #ifdef USE_GTKHTML
 #include <gtkhtml/gtkhtml.h>
 #include <gtkhtml/htmltext.h>
+#include <gtkhtml/htmlengine.h>
+#include <gtkhtml/htmlfontmanager.h>
+#include <gtkhtml/htmlpainter.h>
 #include <gtkhtml/htmltextslave.h>
+#include <gtkhtml/htmlselection.h>
+#include <gtkhtml/gtkhtml-properties.h>
+#include <gal/widgets/e-font.h>
 #endif
 
 #include "../gmime/gmime.h"
 
 #include "articlelist.h"
+#include "debug.h"
 #include "gui.h"
 #include "gui-headers.h"
 #include "globals.h"
@@ -110,17 +117,6 @@ static GnomeUIInfo respond_menu [] =
 	GNOMEUIINFO_END
 };
 
-static GnomeUIInfo search_menu[] =
-{
-	{
-		GNOME_APP_UI_ITEM,
-		N_("Search for user on deja"),
-		N_("Search for other postings by this use on deja.com"),
-		search_deja,
-	},
-	GNOMEUIINFO_END
-};
-
 static GnomeUIInfo filter_menu[] =
 {
 	{
@@ -141,14 +137,14 @@ static GnomeUIInfo filter_menu[] =
 
 static GnomeUIInfo text_menu_popup[] =
 {
-	{
+	{ /* 0 */
 		GNOME_APP_UI_ITEM,
 		N_("Open in New Window"),
 		N_("Open this message for reading."),
 		message_read_window
 	},
-	GNOMEUIINFO_SEPARATOR,
-        {
+	GNOMEUIINFO_SEPARATOR, /* 1 */
+        { /* 2 */
 		GNOME_APP_UI_SUBTREE,
 		N_("Respond"),
 		N_("Respond"),
@@ -157,16 +153,7 @@ static GnomeUIInfo text_menu_popup[] =
 		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BLANK,
 		'\0', 0, NULL
 	},
-	{
-		GNOME_APP_UI_SUBTREE,
-		N_("Search Offsite"),
-		N_("Search Offsite"),
-		&search_menu, NULL,
-		NULL,
-		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BLANK,
-		'\0', 0, NULL
-	},
-	{
+	{ /* 3 */
 		GNOME_APP_UI_SUBTREE,
 		N_("Filter"),
 		N_("Filter"),
@@ -175,20 +162,20 @@ static GnomeUIInfo text_menu_popup[] =
 		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BLANK,
 		'\0', 0, NULL
 	},
-	GNOMEUIINFO_SEPARATOR,
-	{
+	GNOMEUIINFO_SEPARATOR, /* 4 */
+	{ /* 5 */
 		GNOME_APP_UI_TOGGLEITEM,
 		N_("Show All Headers in Body"),
 		N_("Show All Headers in Body"),
 		show_all_headers_cb
 	},
-	{
+	{ /* 6 */
 		GNOME_APP_UI_TOGGLEITEM,
 		N_("Mute Quoted Text"),
 		N_("Mute Quoted Text"),
 		mute_quoted_text_cb
 	},
-	{
+	{ /* 7 */
 		GNOME_APP_UI_TOGGLEITEM,
 		N_("Rot13 the Body"),
 		N_("Rot13 the Body"),
@@ -202,42 +189,42 @@ static gchar welcome[] =
 #ifdef USE_GTKHTML
 "<html><body><font face=\"times\" size=\"12\">"
 "<b>Pan " VERSION "</b><br>"
-"Copyright (C) 1999, 2000<br>"
+"Copyright (C) 1999, 2000, 2001 <br>"
 "Pan Development Team"
 "<p>"
-"This is beta software.  If you find a bug, please <a href=\"http://www.superpimp.org/bugreport.html\">report it</a>."
+"This is beta software.  If you find a bug, please <a href=\"http://pan.rebelbase.com/bugreport.html\">report it</a>."
 "<p>"
-"<a href=\"http://www.superpimp.org/contact.html\">Request a Feature</a><br>"
-"<a href=\"http://www.superpimp.org/bugreport.html\">Report a Bug</a><br>"
-"<a href=\"http://www.superpimp.org/download.html\">Upgrade Pan</a>"
+"<a href=\"http://pan.rebelbase.com/contact.html\">Request a Feature</a><br>"
+"<a href=\"http://pan.rebelbase.com/bugreport.html\">Report a Bug</a><br>"
+"<a href=\"http://pan.rebelbase.com/download.html\">Upgrade Pan</a>"
 "<p>"
-"This program is free software; you can redistribute it"
-"and/or modify it under the terms of the GNU General Public"
-"License as published by the Free Software Foundation;"
-"either version 2 of the License, or (at your option) any"
+"This program is free software; you can redistribute it "
+"and/or modify it under the terms of the GNU General Public "
+"License as published by the Free Software Foundation; "
+"either version 2 of the License, or (at your option) any "
 "later version."
 "<p>"
-"This program is distributed in the hope that it will be"
-"useful, but WITHOUT ANY WARRANTY; without even the implied"
-"warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR"
-"PURPOSE.  See the GNU General Public License for more"
+"This program is distributed in the hope that it will be "
+"useful, but WITHOUT ANY WARRANTY; without even the implied "
+"warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR "
+"PURPOSE.  See the GNU General Public License for more "
 "details."
 "<p>"
-"You should have received a copy of the GNU General Public"
-"License along with this program; if not, write to the"
-"Free Software Foundation, Inc., 59 Temple Place,"
+"You should have received a copy of the GNU General Public "
+"License along with this program; if not, write to the "
+"Free Software Foundation, Inc., 59 Temple Place, "
 "Suite 330, Boston, MA 02111-1307, USA"
 "</font></body></html>"
 #else
 "Pan " VERSION "\n"
-"Copyright (C) 1999, 2000 \n"
-"Pan Development Team (pan@superpimp.org) \n"
+"Copyright (C) 1999, 2000, 2001 \n"
+"Pan Development Team (pan@rebelbase.com) \n"
 "\n"
 "  This is beta software.  If you find a bug, please report it.\n"
 "\n"
-"  http://www.superpimp.org/contact.html   -  Request a Feature\n"
-"  http://www.superpimp.org/bugreport.html -  Report a Bug\n"
-"  http://www.superpimp.org/download.html  -  Upgrade Pan\n"
+"  http://pan.rebelbase.com/contact.html   -  Request a Feature\n"
+"  http://pan.rebelbase.com/bugreport.html -  Report a Bug\n"
+"  http://pan.rebelbase.com/download.html  -  Upgrade Pan\n"
 "\n"
 "This program is free software; you can redistribute it \n"
 "and/or modify it under the terms of the GNU General Public \n"
@@ -284,9 +271,13 @@ show_all_headers_cb (GtkCheckMenuItem * item, gpointer data)
 void
 text_select_all (void)
 {
+	debug_enter ("text_select_all");
+
 	pan_lock();
 	gtk_editable_select_region (GTK_EDITABLE(Pan.text), 0, -1);
 	pan_unlock();
+
+	debug_exit ("text_select_all");
 }
 
 /**
@@ -295,85 +286,106 @@ text_select_all (void)
 
 #ifdef USE_GTKHTML
 
-			 
-static void
-get_default_font_stats (gchar ** setme_font_name, gint * setme_font_size)
-{
-	gchar ** p;
-	int size = 0;
-	int font_size = 10;
-
-	/* split up the message body */
-       	p = g_strsplit (message_body_font, "-", -1);
-	size = 0;
-	while (p[size] != NULL)
-		++size;
-
-	/* find font name */
-	*setme_font_name = g_strdup (size>2 ? p[2] : "Courier");
-
-	/* find font size */
-	if (size>7) {
-		gint tmp = atoi(p[7]);
-		if (tmp > 0)
-			font_size = tmp;
-	}
-	*setme_font_size = font_size;
-
-	/* cleanup */
-	g_strfreev (p);
-}
-
 static void
 text_page_down_or_next_article (GtkWidget * w)
 {
-//	GtkHTML * html = GTK_HTML(w);
-
-	g_message ("text_page_down_or_next_article");
-
-	/* calculate a page down */
-#if 0
-	old_val = text->vadj->value;
-	text->vadj->value += text->vadj->page_increment;
-	upper = text->vadj->upper - text->vadj->page_size;
-	text->vadj->value = CLAMP(text->vadj->value, 0.0, upper-18);
-
-	/* either go down a page or, if we're at the end, read the next article */
-	if (text->vadj->value != old_val)
-		gtk_signal_emit_by_name (GTK_OBJECT (text->vadj), "value_changed");
-	else
-		articlelist_view_next ();
-#endif
 }
 
+
 static gchar*          
 make_safe_for_html (const gchar * str)
 {
-	gchar * pch = g_strdup (str);
+	gint size_needed;
+	gchar * retval;
+	gchar * out;
+	const gchar * in;
+	debug_enter ("make_safe_for_html");
 
-	if (pch != NULL)
-	{
-		replace_gstr (&pch, pan_substitute(pch, "&", "&amp;"));
-		replace_gstr (&pch, pan_substitute(pch, ">", "&gt;"));
-		replace_gstr (&pch, pan_substitute(pch, "<", "&lt;"));
-		replace_gstr (&pch, pan_substitute(pch, "\"", "&quot;"));
+	/* sanity clause */
+	if (!is_nonempty_string (str))
+		return g_strdup ("");
+
+	/* how big must the buffer be? */
+	size_needed = 1;
+	for (in=str; *in; ++in) {
+		gint inc;
+		switch (*in) {
+			case '&': inc = 5; break;
+			case '>': case '<': inc = 4; break;
+			case '"': inc = 6; break;
+			default: inc = 1; break;
+		}
+		size_needed += inc;
 	}
 
-        return pch;
+	/* build the output string */
+	retval = out = g_malloc (size_needed);
+	for (in=str; *in; ++in) {
+		switch (*in) {
+			case '&': memcpy (out, "&amp;", 5); out += 5; break;
+			case '>': memcpy (out, "&gt;", 4); out += 4; break;
+			case '<': memcpy (out, "&lt;", 4); out += 4; break;
+			case '"': memcpy (out, "&quot;", 6); out += 6; break;
+			default: *out++ = *in; break;
+		}
+	}
+	*out = '\0';
+
+	debug_exit ("make_safe_for_html");
+	return retval;
 }
 
 
 static void
 add_header (GString * buf, const gchar * header_name, const gchar * val)
 {
-	gchar * tmp_val = make_safe_for_html (val);
+	gchar * tmp_val;
+	debug_enter ("add_header");
 
+       	tmp_val = make_safe_for_html (val);
 	if (is_nonempty_string (tmp_val))
 		g_string_sprintfa (buf, "<tr><td><b>%s</b> %s</td></tr>", header_name, tmp_val);
 
 	g_free (tmp_val);
+	debug_exit ("add_header");
 }
 
+typedef struct
+{
+	gint i;
+	const Article * a;
+	GString * str;
+}
+AttachmentWorkStruct;
+
+static void
+add_inline_images_mpfunc (GMimePart * part, gpointer user_data)
+{
+	AttachmentWorkStruct * data = (AttachmentWorkStruct*) user_data;
+	GString * str = data->str;
+	const gchar * content_id;
+	const GMimeContentType * type;
+
+	if (pan_strcmp ("inline", g_mime_part_get_content_disposition (part)))
+		return;
+
+	type = g_mime_part_get_content_type (part);
+	if (!g_mime_content_type_is_type (type, "image", "*"))
+		return;
+
+	content_id = g_mime_part_get_content_id (part);
+	if (!is_nonempty_string(content_id)) /* uh oh, no cid, let's make one */
+	{
+		GString * id = g_string_new (data->a->message_id);
+		g_string_erase (id, id->len-1, 1); /* '>' */
+		g_string_sprintfa (id, "___%d>", data->i++);
+		g_mime_part_set_content_id (part, id->str);
+		content_id = g_mime_part_get_content_id (part);
+		g_string_free (id, TRUE);
+	}
+
+	g_string_sprintfa (str, "<img alt=\"%s\" src=\"cid:%s\"><p>\n", content_id, content_id);
+}
 
 static void
 text_get_article_body (const Article * a, GString * buf)
@@ -383,6 +395,8 @@ text_get_article_body (const Article * a, GString * buf)
 	gboolean is_html = FALSE;
 	const GMimeContentType * type;
 
+	debug_enter ("text_get_article_body");
+
 	g_return_if_fail (mm!=NULL);
 	mp = mm->mime_part;
 	type = g_mime_part_get_content_type (mp);
@@ -435,45 +449,22 @@ text_get_article_body (const Article * a, GString * buf)
 			if (text_get_mute_quoted())
 				replace_gstr (&body, mute_quoted_text_in_body(body));
 			if (text_get_wrap())
-				replace_gstr (&body, fill_body(body));
+				replace_gstr (&body, fill_body(body,wrap_column));
 			replace_gstr (&body, make_safe_for_html (body));
+			replace_gstr (&body, linkify_text(body));
 		}
 
-		/* show all the inlined images */
-		if (mp->children != NULL)
+		/* attachments */
+		if (1)
 		{
-			GList * l;
-			GString * str = g_string_new (NULL);
-			int i = 1;
-			for (l=mp->children; l!=NULL; l=l->next, ++i)
-			{
-				const gchar * content_id;
-				const GMimeContentType * type;
-				GMimePart * part = (GMimePart*) l->data;
-
-				if (pan_strcmp ("inline", g_mime_part_get_content_disposition (part)))
-					continue;
-
-				type = g_mime_part_get_content_type (part);
-				if (!g_mime_content_type_is_type (type, "image", "*"))
-					continue;
-
-				content_id = g_mime_part_get_content_id (part);
-				if (!is_nonempty_string(content_id)) /* uh oh, no cid, let's make one */
-				{
-					GString * id = g_string_new (a->message_id);
-					g_string_erase (id, id->len-1, 1); /* '>' */
-					g_string_sprintfa (id, "___%d>", i);
-					g_mime_part_set_content_id (part, id->str);
-					content_id = g_mime_part_get_content_id (part);
-					g_string_free (id, TRUE);
-				}
-				g_string_sprintfa (str, "<img alt=\"%s\" src=\"cid:%s\"><p>\n", content_id, content_id);
-			}
-
-			if (str->len)
-				replace_gstr (&body, g_strdup_printf("%s%s", str->str, (body?body:"")));
-			g_string_free (str, TRUE);
+			AttachmentWorkStruct tmp;
+			tmp.i = 0;
+			tmp.a = a;
+			tmp.str = g_string_new (NULL);
+			g_mime_part_foreach (mp, add_inline_images_mpfunc, &tmp);
+			if (tmp.str->len)
+				replace_gstr (&body, g_strdup_printf("%s%s", tmp.str->str, (body?body:"")));
+			g_string_free (tmp.str, TRUE);
 		}
 
 		/* the headers */
@@ -494,11 +485,17 @@ text_get_article_body (const Article * a, GString * buf)
 
 	/* cleanup */
 	g_free (body);
+	debug_exit ("text_get_article_body");
 }
 
 static void
 text_get_article_headers (const Article * a, GString * buf)
 {
+	debug_enter ("text_get_article_headers");
+
+	g_return_if_fail (a!=NULL);
+	g_return_if_fail (buf!=NULL);
+
 	g_string_append (buf, "<table width=\"100%\" border=\"0\" cellspacing=\"0\" cellpadding=\"5\">");
 	g_string_append (buf, "<tr> <td width=\"20\" bgcolor=\"#008899\">&nbsp; </td><td><table>");
 
@@ -526,7 +523,7 @@ text_get_article_headers (const Article * a, GString * buf)
 	if (header_flags & UI_HEADER_NEWSGROUPS)
 		add_header (buf, _("Newsgroups:"), article_get_header(a, HEADER_NEWSGROUPS));
 	if (header_flags & UI_HEADER_DATE) {
-		gchar * date = get_date_display_string (a->date);     
+		gchar * date = get_date_display_string (a->date, body_date_format);
 		add_header (buf, _("Date:"), date);
 		g_free (date);
 	}
@@ -555,37 +552,39 @@ text_get_article_headers (const Article * a, GString * buf)
 
 	/* end of headers */
 	g_string_append (buf, "</table>");
+	debug_exit ("text_get_article_headers");
 }
 
 static void
 set_text_from_article (const Article * a)
 {
+	gboolean set_face = is_nonempty_string (message_body_font);
 	GString * buf = g_string_new (NULL);
-	gchar * font_name = NULL;
-	gint font_size = 0;
-
-	get_default_font_stats (&font_name, &font_size);
+	debug_enter ("set_text_from_article");
 
 	/* build the html string */
 	g_string_append (buf, "<html><body>");
-	g_string_sprintfa (buf, "<font size=\"%d\" face=\"%s\">", font_size, font_name);
+	if (set_face)
+		g_string_sprintfa (buf, "<font face=\"%s\">", message_body_font);
 	text_get_article_headers (a, buf);
 	text_get_article_body (a, buf);
-	g_string_append (buf, "</font></body></html>");
+	if (set_face)
+		g_string_append (buf, "</font>");
+	g_string_append (buf, "</body></html>");
 
 	/* update the text window */
 	text_set_raw (buf->str);
 
 	/* cleanup */
 	g_string_free (buf, TRUE);
+	debug_exit ("set_text_from_article");
 }
 
 static void
 url_requested (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpointer data)
 {
 	int end = GTK_HTML_STREAM_ERROR;
-
-	g_message ("url_requested [%s]", url!=NULL ? url : "(Null)");
+	debug_enter ("url_requested");
 
 	g_return_if_fail (html != NULL);
 	g_return_if_fail (is_nonempty_string(url));
@@ -621,6 +620,7 @@ url_requested (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpointer d
 	}
 
 	gtk_html_end (html, handle, end);
+	debug_exit ("url_requested");
 }
 
 static void
@@ -631,19 +631,17 @@ load_done (GtkHTML *html)
 static void
 on_url (GtkHTML *html, const gchar *url, gpointer data)
 {
-	g_message ("on_url [%s]", url!=NULL ? url : "(Null)");
 }
 
 static void
 on_set_base (GtkHTML *html, const gchar *url, gpointer data)
 {
-	g_message ("on_set_base [%s]", url!=NULL ? url : "(Null)");
 }  
 
 static void
 on_link_clicked (GtkHTML *html, const gchar *url, gpointer data)
 {
-	g_message ("on_link_clicked [%s]", url!=NULL ? url : "(Null)");
+	debug_enter ("on_link_clicked");
 
 	if (!strncmp (url, "message-id:", 11))
 	{
@@ -656,18 +654,17 @@ on_link_clicked (GtkHTML *html, const gchar *url, gpointer data)
 		gnome_url_show (url);
 		pan_unlock ();
 	}
+	debug_exit ("on_link_clicked");
 }
 
 static void
 on_redirect (GtkHTML *html, const gchar *url, int delay, gpointer data)
 {
-	g_message ("on_redirect [%s]", url!=NULL ? url : "(Null)");
 }
 
 static gboolean
 object_requested_cmd (GtkHTML *html, GtkHTMLEmbedded *eb, void *data)
 {
-	g_message ("object_requested");
 	return TRUE;
 }
 
@@ -681,19 +678,22 @@ text_page_down_or_next_article (GtkWidget * w)
         gfloat upper;
 	gfloat old_val;
 
+	debug_enter ("text_page_down_or_next_article");
 	g_return_if_fail (text != NULL);
 
 	/* calculate a page down */
 	old_val = text->vadj->value;
 	text->vadj->value += text->vadj->page_increment;
 	upper = text->vadj->upper - text->vadj->page_size;
-	text->vadj->value = CLAMP (text->vadj->value, 0.0, upper-18);
+	text->vadj->value = CLAMP (text->vadj->value, 0.0, upper);
 
 	/* either go down a page or, if we're at the end, read the next article */
 	if (text->vadj->value>=0.0 && text->vadj->value!=old_val)
 		gtk_signal_emit_by_name (GTK_OBJECT (text->vadj), "value_changed");
 	else
 		articlelist_view_next ();
+
+	debug_exit ("text_page_down_or_next_article");
 }
 
 /**
@@ -710,7 +710,9 @@ SetTextFromArticleIdleStruct;
 static gint
 set_text_from_article_idle (gpointer data)
 {
+	Group * g;
 	SetTextFromArticleIdleStruct * s = (SetTextFromArticleIdleStruct*) data;
+	debug_enter ("set_text_from_article_idle");
 
 	/* update the gui */
 	pan_lock_unconditional ();
@@ -718,11 +720,15 @@ set_text_from_article_idle (gpointer data)
 	update_body_pane_nolock (Pan.text, s->str->str, text_get_mute_quoted());
 	pan_unlock_unconditional ();
 
-	/* cleanup */
+	/* undo the reference cound added in set_text_from_article */
+	g = s->a->group;
 	pan_object_unref (PAN_OBJECT(s->a));
+	group_unref_articles (g, NULL);
+
+	/* cleanup */
 	g_string_free (s->str, TRUE);
 	g_free (s);
-
+	debug_exit ("set_text_from_article_idle");
 	return 0;
 }
 static void
@@ -730,6 +736,7 @@ set_text_from_article (const Article * a)
 {
 	GString * buf;
 	GMimePart * mp;
+	debug_enter ("set_text_from_article");
 
 	g_assert (mm != NULL);
 	g_return_if_fail (a!=NULL);
@@ -752,7 +759,7 @@ set_text_from_article (const Article * a)
 			if (rot13_enabled)
 				replace_gstr (&pch, rot13(pch));
                 	if (text_get_wrap())
-				replace_gstr (&pch, fill_body(pch));      
+				replace_gstr (&pch, fill_body(pch,wrap_column));      
 			g_string_append (buf, pch);
 		}
 		g_free (pch);
@@ -790,13 +797,17 @@ set_text_from_article (const Article * a)
 		SetTextFromArticleIdleStruct * foo = g_new (SetTextFromArticleIdleStruct, 1);
 		foo->str = buf;
 		foo->a = a;
+
+		/* make sure the article's still around by the time we finish */
 		pan_object_ref (PAN_OBJECT(a));
+		group_ref_articles (a->group, NULL);
 
 		/* tell the UI to refresh when it's got some free time */
 		pan_lock ();
 		gtk_idle_add (set_text_from_article_idle, foo);
 		pan_unlock ();
 	}
+	debug_exit ("set_text_from_article");
 }
 #endif
 
@@ -808,6 +819,7 @@ static gint
 text_set_raw_idle (gpointer data)
 {
 	gchar * text = (gchar*) data;
+	debug_enter ("text_set_raw_idle");
 
 	if (text == NULL)
 		text = g_strdup ("");
@@ -831,18 +843,20 @@ text_set_raw_idle (gpointer data)
 			gtk_editable_insert_text (editable_w, text, strlen(text), &position);
 		gtk_text_thaw (text_w);
 #endif
-		g_message ("text: [%s]", text);
 		g_free (text);
 	}
 	pan_unlock_unconditional ();
+	debug_exit ("text_set_raw_idle");
 	return 0;
 }
 void
 text_set_raw (const gchar * text)
 {
+	debug_enter ("text_set_raw");
 	pan_lock ();
 	gtk_idle_add (text_set_raw_idle, g_strdup(text));
 	pan_unlock ();
+	debug_exit ("text_set_raw");
 }
 
 /***
@@ -852,13 +866,19 @@ text_set_raw (const gchar * text)
 void
 text_refresh (void)
 {
+	debug_enter ("text_refresh");
+
 	if (current_article != NULL)
 		set_text_from_article (current_article);
+
+	debug_exit ("text_refresh");
 }
 
 void
 text_set_rot13 (gboolean on)
 {
+	debug_enter ("text_set_rot13");
+
 	if (on != rot13_enabled)
 	{
 		rot13_enabled = on;
@@ -866,6 +886,8 @@ text_set_rot13 (gboolean on)
 		if (current_article != NULL)
 			text_update (current_article);
 	}
+
+	debug_exit ("text_set_rot13");
 }
 
 
@@ -873,8 +895,11 @@ text_set_rot13 (gboolean on)
 void
 text_read_more (void)
 {
-	g_message ("text_read_more");
+	debug_enter ("text_read_more");
+
 	text_page_down_or_next_article (Pan.text);
+
+	debug_exit ("text_read_more");
 }
 
 
@@ -882,15 +907,21 @@ text_read_more (void)
 static void
 realize_text_cb (GtkWidget *text, gpointer data)
 {
+	debug_enter ("realize_text_cb");
+
 	gtk_idle_add (text_set_raw_idle, g_strdup(welcome));
 	gtk_signal_disconnect_by_func (
 		GTK_OBJECT(text), GTK_SIGNAL_FUNC(realize_text_cb), NULL);
+
+	debug_exit ("realize_text_cb");
 }
 
 /* Process a mouse button events inside the text widget */
 static void
 text_button_press_cb (GtkWidget *widget, GdkEventButton *bevent)
 {
+	debug_enter ("text_button_press_cb");
+
 	gtk_widget_grab_focus (GTK_WIDGET (Pan.text));
 
 	if (bevent->button != 1)
@@ -903,6 +934,8 @@ text_button_press_cb (GtkWidget *widget, GdkEventButton *bevent)
 		gnome_popup_menu_do_popup (text_rightclick_popup, NULL,
 					   NULL, bevent, NULL);
 	}
+
+	debug_exit ("text_button_press_cb");
 }
 
 /* Process a keyboard input event in the text pane */
@@ -939,49 +972,94 @@ set_current_article_idle (gpointer data)
 	Article * old_article;
 	GMimeMessage * old_mm;
 
-	if (a!=NULL)
-		group_ref_articles (a->group, NULL);
+	debug_enter ("set_current_article_idle");
+
+	/**
+	***  Update the current article
+	**/
 
-	/* update old, new */
 	old_mm = mm;
 	old_article = current_article;
 	current_article = a;
-	if (a != NULL) {
+	mm = NULL;
+	if (a != NULL)
+	{
 		gchar * raw = article_get_message (a);
 		mm = pan_g_mime_parser_construct_message (raw);
-		g_message ("mm is now %p", mm);
 		g_free (raw);
 	}
-
 	pan_callback_call (current_article_changed, old_article, current_article);
 
-	if (old_article != NULL)
-		group_unref_articles (old_article->group, NULL);
+	/**
+	***  Cleanup
+	**/
+
 	if (old_mm != NULL)
+	{
 		g_mime_message_destroy (old_mm);
-
+	}
+	if (old_article != NULL)
+	{
+		Group * g = old_article->group;
+		pan_object_unref (PAN_OBJECT(old_article));
+		group_unref_articles (g, NULL);
+	}
 	if (current_article != NULL)
+	{
 		set_text_from_article (current_article);
+	}
+	else
+	{
+#ifndef USE_GTKHTML
+		gui_headers_set_default (header_table, NULL, TRUE);
+#endif
+		text_set_raw (NULL);
+	}
 
-	if (a != NULL)
-		pan_object_unref (PAN_OBJECT(a));
-
+	debug_exit ("set_current_article_idle");
 	return 0;
 }
 void
 set_current_article (Article * a)
 {
-	pan_lock ();
-	if (a != NULL)
+	debug_enter ("set_current_article");
+
+	if (a != NULL) {
 		pan_object_ref (PAN_OBJECT(a));
-	gtk_idle_add (set_current_article_idle, a);
-	pan_unlock ();
+		group_ref_articles (a->group, NULL);
+	}
+	set_current_article_idle (a);
+
+	debug_enter ("set_current_article");
 }
 
 /**
 ***
 **/
 
+static int
+articlelist_group_changed_cb (gpointer call_obj,
+                              gpointer call_arg,
+                              gpointer client_data)
+{
+	Group * group = GROUP(call_arg);
+
+	if (current_article!=NULL && current_article->group!=group)
+		set_current_article (NULL);
+
+	return 0;
+}
+
+static int
+compare_pgchar_pparticle_msgid (const void * a, const void *b)
+{
+	const gchar * msgid_a = (const gchar*) a;
+	const gchar * msgid_b = (*(const Article**)b)->message_id;
+	pan_warn_if_fail (is_nonempty_string(msgid_a));
+	pan_warn_if_fail (is_nonempty_string(msgid_b));
+	return pan_strcmp (msgid_a, msgid_b);
+}     
+
 static int
 group_articles_removed_cb (gpointer call_obj,
                            gpointer call_arg,
@@ -989,6 +1067,7 @@ group_articles_removed_cb (gpointer call_obj,
 {
 	Group * group = GROUP(call_obj);
 	GPtrArray * removed = (GPtrArray*) call_arg;
+	debug_enter ("group_articles_removed_cb");
 
 	/* sanity checks */
 	g_return_val_if_fail (group!=NULL, 0);
@@ -996,17 +1075,22 @@ group_articles_removed_cb (gpointer call_obj,
 	g_return_val_if_fail (removed->len>0, 0);
 
 	/* unset the current article if we need to */
-	if (current_article!=NULL) {
-		guint i;
-		gboolean current_removed = FALSE;
-		for (i=0; !current_removed && i!=removed->len; ++i)
-		if (ARTICLE(g_ptr_array_index(removed,i)) == current_article)
-			current_removed = TRUE;
-		if (current_removed)
+	if (current_article!=NULL)
+	{
+		gboolean exact_match = FALSE;
+
+		lower_bound (current_article->message_id,
+		             removed->pdata,
+		             removed->len,
+			     sizeof(gpointer),
+			     compare_pgchar_pparticle_msgid,
+			     &exact_match);
+
+		if (exact_match)
 			set_current_article (NULL);
-		text_set_raw (NULL);
 	}
 
+	debug_exit ("group_articles_removed_cb");
 	return 0;
 }
 /**
@@ -1028,6 +1112,7 @@ text_create (void)
 
 #ifdef USE_GTKHTML
 	Pan.text = gtk_html_new ();
+	text_set_font (message_body_font);
 #else
         Pan.text = gtk_text_new (NULL, NULL);
         gtk_text_set_editable (GTK_TEXT(Pan.text), FALSE);
@@ -1089,6 +1174,11 @@ text_create (void)
 	                  group_articles_removed_cb,
 	                  NULL);
 
+	/* watch for the articlelist group changing */
+	pan_callback_add (articlelist_group_changed,
+	                  articlelist_group_changed_cb,
+			  NULL);
+
 	return retval;
 }
 
@@ -1115,14 +1205,16 @@ ensure_header_col_inited (void)
 void
 text_update (Article * article)
 {
+	debug_enter ("text_update");
+
         /* sanity checks */
         g_return_if_fail (article!=NULL);
 
 	/* update the current article */
-	pan_object_ref (PAN_OBJECT(article));
 	set_current_article (article);
 	gui_page_set (MESSAGE_PAGE, Pan.text);
-	pan_object_unref (PAN_OBJECT(article));
+
+	debug_exit ("text_update");
 }
 
 void
@@ -1130,10 +1222,14 @@ text_set_from_article (Article *article,
 		       MessageWindowType type,
 		       gboolean needs_window)
 {
+	debug_enter ("text_set_from_article");
+
 	/* sanity checks */
 	g_return_if_fail (article!=NULL);
 
 	queue_add (TASK(task_body_new(article, needs_window, type)));
+
+	debug_exit ("text_set_from_article");
 }
 
 gchar*
@@ -1141,6 +1237,7 @@ text_get_message_to_reply_to (void)
 {
 	gchar * body = NULL;
 	gboolean has_selection;
+	debug_enter ("text_get_message_to_reply_to");
 
 #ifdef USE_GTKHTML
 	{
@@ -1155,8 +1252,10 @@ text_get_message_to_reply_to (void)
 		has_selection = editable->has_selection
 			&& editable->selection_end_pos!=editable->selection_start_pos;
 		if (has_selection) {
-			gulong start_pos = editable->selection_start_pos;
-			gulong end_pos = editable->selection_end_pos;
+			gulong a = editable->selection_start_pos;
+			gulong b = editable->selection_end_pos;
+			gulong start_pos = MIN (a, b);
+			gulong end_pos = MAX (a, b);
 			body = gtk_editable_get_chars (editable, start_pos, end_pos);
 		}
 	}
@@ -1166,13 +1265,15 @@ text_get_message_to_reply_to (void)
 	if (mm!=NULL && !has_selection)
 	{
 		gboolean is_html = FALSE;
-		gchar * pch;
+		gchar * pch = NULL;
 		replace_gstr (&body, g_mime_message_get_body (mm, FALSE, &is_html));
-		pch = strstr (body, "\n-- \n");
+		if (body != NULL)
+			pch = strstr (body, "\n-- \n");
 		if (pch != NULL)
 			pch[1] = '\0';
 	}
 
+	debug_exit ("text_get_message_to_reply_to");
 	return body;
 }
 
@@ -1196,6 +1297,8 @@ text_get_fill_body_changed_callback (void)
 void
 text_set_wrap (gboolean wrap)
 {
+	debug_enter ("text_set_wrap");
+
 	if (wrap != _do_wrap)
 	{
 		_do_wrap = wrap;
@@ -1203,6 +1306,8 @@ text_set_wrap (gboolean wrap)
 		pan_callback_call (text_get_fill_body_changed_callback(), NULL, GINT_TO_POINTER(wrap));
 		text_refresh ();                            
 	}
+
+	debug_exit ("text_set_wrap");
 }
 
 gboolean
@@ -1231,13 +1336,17 @@ text_get_show_all_headers_changed_callback (void)
 void
 text_set_show_all_headers (gboolean show)
 {
+	debug_enter ("text_set_show_all_headers");
+
 	if (_show_all_headers != show)
 	{
 		_show_all_headers = show;
 		pan_callback_call (text_get_show_all_headers_changed_callback(), NULL, GINT_TO_POINTER(show));
 		text_refresh ();
-		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(text_menu_popup[6].widget), show);
+		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(text_menu_popup[5].widget), show);
 	}
+
+	debug_exit ("text_set_show_all_headers");
 }
 
 gboolean
@@ -1266,13 +1375,17 @@ text_get_mute_quoted_changed_callback (void)
 void
 text_set_mute_quoted (gboolean quoted)
 {
+	debug_enter ("text_set_mute_quoted");
+
 	if (_mute_quoted != quoted)
 	{
 		_mute_quoted = quoted;
 		pan_callback_call (text_get_mute_quoted_changed_callback(), NULL, GINT_TO_POINTER(quoted));
 		text_refresh ();
-		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(text_menu_popup[7].widget), quoted);
+		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(text_menu_popup[6].widget), quoted);
 	}
+
+	debug_exit ("text_set_mute_quoted");
 }
 
 gboolean
@@ -1280,3 +1393,58 @@ text_get_mute_quoted (void)
 {
 	return _mute_quoted;
 }
+
+/**
+***  Font
+**/
+
+#ifdef USE_GTKHTML
+static HTMLFont *
+alloc_this_font_and_squeal_like_a_pig (HTMLPainter      * painter,
+                                       gchar            * face,
+                                       gdouble            size,
+                                       GtkHTMLFontStyle   style)
+{
+	HTMLFont * retval = NULL;
+	EFont * font;
+
+	font  = e_font_from_gdk_name (message_body_font);
+	if (font != NULL)
+	{
+		EFontStyle efs = E_FONT_PLAIN;
+		if (style & GTK_HTML_FONT_STYLE_BOLD)   efs |= E_FONT_BOLD;
+		if (style & GTK_HTML_FONT_STYLE_ITALIC) efs |= E_FONT_ITALIC;
+		retval = html_font_new (font, e_font_utf8_text_width (font, efs, " ", 1));
+	}
+
+	return retval;
+}
+#endif
+
+void
+text_set_font (const gchar * font_name)
+{
+#ifndef USE_GTKHTML
+	widget_set_font (GTK_WIDGET(Pan.text), message_body_font);
+#else
+	GtkHTML * html = GTK_HTML (Pan.text);
+
+	if (html->engine)
+	{
+		static struct _HTMLPainterClass * painter_class = NULL;
+		static HTMLFont *(*old_alloc_font)(HTMLPainter*,gchar*,gdouble,GtkHTMLFontStyle) = NULL;
+		static gboolean painter_found = FALSE;
+
+		if (!painter_found) {
+			painter_class = HTML_PAINTER_CLASS(GTK_OBJECT(html->engine->painter)->klass);
+			old_alloc_font = painter_class->alloc_font;
+			painter_found = TRUE;
+		}
+
+		painter_class->alloc_font = is_nonempty_string(message_body_font)
+			? alloc_this_font_and_squeal_like_a_pig
+			: old_alloc_font;
+	}
+#endif
+	text_refresh ();
+}
