@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +18,15 @@
  * 
  */
 
-#ifndef __UTIL_TEXT__H__
-#define __UTIL_TEXT__H__
+#ifndef __UTIL_WRAP__H__
+#define __UTIL_WRAP__H__
 
 #include <glib.h>
 
-gchar * rot13                     (const gchar * body);
-gchar * fill_body                 (const gchar * body);
-gchar * mute_quoted_text_in_body  (const gchar * body);
+gchar * rot13                       (const gchar * body);
+gchar * mute_quoted_text_in_body    (const gchar * body);
+gchar * fill_body                   (const gchar * body, gint wrap_column);
 
-#endif /* __UTIL_TEXT__H__ */
+void    wrap_regression_tests       (void);
+
+#endif /* __UTIL_WRAP__H__ */
