@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -218,22 +218,58 @@ pan_clean_temp_dir (void)
 ****
 ***/
 
+
 gchar*
-pan_normalize_filename (const gchar * filename)
+pan_normalize_filename_inplace (gchar * filename)
 {
-	gchar * pch;
-	gchar * retval;
-	gchar doublesep[3] = { G_DIR_SEPARATOR, G_DIR_SEPARATOR, '\0' };
+	gchar * in;
+	gchar * out;
 
-	g_return_val_if_fail (filename!=NULL, NULL);
+	for (in=out=filename; *in; )
+		if (in[0]=='/' && in[1]=='/')
+			++in;
+		else
+			*out++ = *in++;
+	*out = '\0';
 
-       	retval = g_strdup (filename);
-
-	while ((pch = strstr (retval, doublesep)))
-		g_memmove (pch, pch+1, strlen(pch+1)+1);
-
-	return retval;
+	return filename;
 }
+static void
+test_pan_normalize_filename_inplace (void)
+{
+	gint num = 0;
+	gchar * out = g_new0 (gchar, 1024);
+	const gchar * in;
+	const gchar * expected;
+	const gchar * test_type;
+
+	test_type = "normalize string";
+	strcpy (out, in = "//tmp/foo/bar.txt");
+	expected = "/tmp/foo/bar.txt";
+	pan_normalize_filename_inplace (out);
+	regression_test_result_va (!strcmp(out,expected), test_type, ++num, "\"%s\" -> \"%s\"", in, out);
+
+	strcpy (out, in = "//tmp////foo/bar/");
+	expected = "/tmp/foo/bar/";
+	pan_normalize_filename_inplace (out);
+	regression_test_result_va (!strcmp(out,expected), test_type, ++num, "\"%s\" -> \"%s\"", in, out);
+
+	strcpy (out, in = "//tmp////foo/bar///");
+	expected = "/tmp/foo/bar/";
+	pan_normalize_filename_inplace (out);
+	regression_test_result_va (!strcmp(out,expected), test_type, ++num, "\"%s\" -> \"%s\"", in, out);
+
+	strcpy (out, in = "tmp////foo/bar///");
+	expected = "tmp/foo/bar/";
+	pan_normalize_filename_inplace (out);
+	regression_test_result_va (!strcmp(out,expected), test_type, ++num, "\"%s\" -> \"%s\"", in, out);
+
+	g_free (out);
+}
+
+/**
+***
+**/
 
 gboolean
 file_exists (const char* filename)
@@ -413,26 +449,6 @@ get_fqdn (const char * host)
 ****
 ***/
 
-/* null-safe strncmp.  NULLs < strings. */
-int
-pan_strncmp (const char * a,
-             const char * b,
-             int          len)
-{
-	int retval;
-
-	if (!a && !b)
-		retval = 0;
-	else if (!a)
-		retval = -1;
-	else if (!b)
-		retval = 1;
-	else
-		retval = strncmp (a, b, len);
-
-	return retval;
-}
-
 /* null-safe strcmp.  NULLs < strings. */
 int 
 pan_strcmp (const char * a,
@@ -491,28 +507,6 @@ pan_stristr (const char * string, const char * pattern)
 	return NULL;
 }
 
-#if 0
-void
-array_shrink (gpointer   array,
-              size_t     index,
-              size_t     element_size,
-              size_t     element_qty)
-{
-	g_return_if_fail (array!=NULL);
-	g_return_if_fail (element_size!=0);
-	g_return_if_fail (index>=0);
-	g_return_if_fail (index<=element_qty);
-
-	if (index != element_qty-1)
-	{
-		g_memmove (
-			((char*)array) + element_size * index,
-			((char*)array) + element_size * (index + 1),
-			element_size * (element_qty - index -1));
-	}
-}
-#endif
-
 gboolean
 string_ends_with (const gchar * str,
                   const gchar * end)
@@ -612,52 +606,6 @@ pan_substitute (const char * original,
 ****
 ***/
 
-void
-pan_g_ptr_array_sort (GPtrArray   * a,
-                      int  (*compare)(const void *, const void *))
-{
-	msort (a->pdata, a->len, sizeof(gpointer), compare);
-}
-
-void
-pan_g_ptr_array_merge_sorted (GPtrArray * target,
-                              GPtrArray * s1,
-                              GPtrArray * s2,
-                              int  (*compare)(const void *, const void *))
-{
-	guint i1, i2;
-
-	g_return_if_fail (target!=NULL);
-	g_return_if_fail (s1!=NULL);
-	g_return_if_fail (s2!=NULL);
-
-	pan_g_ptr_array_reserve (target, s1->len + s2->len);
-
-	for (i1=0, i2=0; i1!=s1->len && i2!=s2->len; )
-	{
-		gpointer * addme = NULL;
-
-		if (i1==s1->len)
-			addme = g_ptr_array_index (s2, i2++);
-		else if (i2==s2->len)
-			addme = g_ptr_array_index (s1, i1++);
-		else {
-			gpointer p1 = g_ptr_array_index (s1, i1);
-			gpointer p2 = g_ptr_array_index (s2, i2);
-			const int result = compare (&p1, &p2);
-			if (result <= 0) {
-				addme = p1;
-				++i1;
-			} else {
-				addme = p2;
-				++i2;
-			}
-		}
-
-		g_ptr_array_add (target, addme);
-	}
-}
-
 void
 pan_g_ptr_array_append (GPtrArray   * target,
                         gpointer    * source_ptr,
@@ -686,6 +634,23 @@ pan_g_ptr_array_dup     (GPtrArray  * source)
 }
 
 
+static void
+pan_hash_to_ptr_array_ghfunc (gpointer key, gpointer val, gpointer data)
+{
+	g_ptr_array_add ((GPtrArray*)data, val);
+}
+void
+pan_hash_to_ptr_array  (GHashTable   * hash,
+                        GPtrArray    * fillme)
+{
+	g_return_if_fail (hash!=NULL);
+	g_return_if_fail (fillme!=NULL);
+
+	g_ptr_array_set_size (fillme, 0);
+	pan_g_ptr_array_reserve (fillme, g_hash_table_size(hash));
+	g_hash_table_foreach (hash, pan_hash_to_ptr_array_ghfunc, fillme);
+}
+
 
 void
 pan_g_ptr_array_reserve (GPtrArray  * a,
@@ -776,6 +741,22 @@ pan_g_string_strstrip (GString * string)
 	}
 }
 
+void
+pan_g_string_append_len  (GString       * str,
+                          const gchar   * appendme,
+                          guint           len)
+{
+	gchar * pch;
+
+	g_return_if_fail (str!=NULL);
+	g_return_if_fail (appendme!=NULL);
+	g_return_if_fail (len!=0);
+
+	pch = g_strndup (appendme, len);
+	g_string_append (str, pch);
+	g_free (pch);
+}
+
 /***
 ****
 ****  TOKENS
@@ -904,8 +885,8 @@ get_next_token_g_str (const char   * pch,
 ***/
 
 
-#define DB_FORMAT_LAST_CHANGED_IN_VERSION "0.8.1 beta 3"
-#define DB_FORMAT_VERSION 2
+#define DB_FORMAT_LAST_CHANGED_IN_VERSION "0.9.4"
+#define DB_FORMAT_VERSION 3
 
 void
 check_and_warn_if_version_change (void)
@@ -939,7 +920,7 @@ _("Thanks for upgrading to Pan %s!"
 "\nversion of Pan."
 "\n"
 "\nFor more information, see the installation section of the"
-"\nPan FAQ at <http://www.superpimp.org/faq.html>."),
+"\nPan FAQ at <http://pan.rebelbase.com/faq.html>."),
 				VERSION,
 				DB_FORMAT_LAST_CHANGED_IN_VERSION);
 			GtkWidget * dialog = gnome_message_box_new (msg,
@@ -969,153 +950,143 @@ _("Thanks for upgrading to Pan %s!"
 ****
 ***/
 
-#if 0
-static gboolean
-badchar (gchar c)
-{
-	return c==' '
-		|| c==','
-		|| c=='('
-		|| c==')'
-		|| c=='\0'
-		|| c=='\n'
-		|| c=='<'
-		|| c=='>';
-}
-
+/* This is pretty much taken straight from
+   slrn's art.c, written by John E. Davis */
 static const gchar*
-end_of_url (const gchar * pch)
+find_url (const gchar  * in,
+          guint        * setme_url_len)
 {
-	while (!badchar(*pch))
-		++pch;
-	if (pch[-1] == '.')
-		--pch;
-	return pch;
-}
+	const gchar * ptr;
 
+	while ((ptr = strstr (in, "://")) != NULL)
+	{
+		const gchar * begin;
+		const gchar * end;
+		gboolean again;
+
+		/* walk back to the beginning of the word */
+		begin = ptr;
+		while ((begin>in) && isalpha((int)begin[-1]))
+			--begin;
+
+		/* all registered and reserved scheme names are >= 3 chars long */
+		if (begin+3 > ptr) {
+			in = ptr + 3;
+			continue;
+		}
+
+		end = ptr;
+		again = TRUE;
+		while (again)
+		{
+			gchar ch = *end;
+
+			switch (ch)
+			{
+				case ',':
+				case ';':
+				case '.':
+					ch = *++end;
+					if (isspace((int)ch)) {
+						--end;
+						again = FALSE;
+					}
+					break;
+
+				case ' ':
+				case '\t':
+				case '\n':
+				case '\"':
+				case '}':
+				case '{':
+				case ')':
+				case '>':
+				case '<':
+				case 0:
+					again = FALSE;
+					break;
+
+				default:
+					++end;
+					break;
+			}
+		}
+
+		*setme_url_len = (guint)(end - begin);
+		if (*setme_url_len < 7)
+			continue;
+
+		if (!strncmp (end-3, "://", 3))
+			continue;
+
+		return begin;
+	}
+
+	return NULL;
+}
 
 gchar*
-linkify_text (const char * text)
+linkify_text (const gchar * text)
 {
-	GString * out = g_string_new (NULL);
-        const gchar * c;
+	guint len = 0;
+	gchar * retval;
+	GString * str;
+	const gchar * prev = text;
+	const gchar * pch;
+	debug_enter ("linkify_text");
 
-	/* walk through the text */
-	c = text;
-	while (*c)
+       	str = g_string_new (NULL);
+	while ((pch = find_url(prev,&len)) != NULL)
 	{
-		/* pass over already-existing anchors */
-		if (!g_strncasecmp (c, "<A", 2))
-		{
-			for (;;)
-			{
-                                if (!g_strncasecmp(c, "/A>", 3))
-                                        break;
+		gchar * url = g_strndup (pch, len);
+		if (pch != prev)
+			pan_g_string_append_len (str, prev, (guint)(pch-prev));
 
-				g_string_append_c (out, *c);
-				++c;
-                                if (!*c)
-                                        break;
-                        }
-                }
+		g_string_append (str, "<a href=\"");
+		g_string_append (str, url);
+		g_string_append (str, "\">");
+		g_string_append (str, url);
+		g_string_append (str, "</a>");
 
-		/* add an anchor to a url */
-		else if ((!g_strncasecmp(c, "http://", 7) ||
-			 (!g_strncasecmp(c, "https://", 8))))
-		{
-			const gchar * end = end_of_url (c);
-			gchar * url = g_strndup (c, end-c);
-			g_string_sprintfa (out,
-				"<A HREF=\"%s\">%s</A>", url, url);
-			g_free (url);
-			c = end;
-		}
+		prev = pch + len;
+		g_free (url);
+	}
+	g_string_append (str, prev);
 
-		/* add anchors to likely web sites */
-		else if (!g_strncasecmp(c, "www.", 4) &&
-		          g_strncasecmp(c, "www..", 5))
-		{
-			const gchar * end = end_of_url (c);
-			if (end-c != 4)
-			{
-				gchar * url = g_strndup (c, end-c);
-				g_string_sprintfa (out,
-					"<A HREF=\"http://%s\">%s</A>",
-					url, url);
-				g_free (url);
-				c = end;
-			}
-                }
+	retval = str->str;
+	g_string_free (str, FALSE);
 
-		/* add anchors to ftp links */
-		else if (!g_strncasecmp(c, "ftp://", 6))
-		{
-			const gchar * end = end_of_url (c);
-			gchar * url = g_strndup (c, end-c);
-			g_string_sprintfa (out,
-				"<A HREF=\"ftp://%s\">%s</A>", url, url);
-			g_free (url);
-			c = end;
-		}
+	debug_exit ("linkify_text");
+	return retval;
+}
 
-		/* more ftp anchors */
-		else if (!strncasecmp(c, "ftp.", 4))
-		{
-			const gchar * end = end_of_url (c);
-			if (end-c != 4)
-			{
-				gchar * url = g_strndup (c, end-c);
-				g_string_sprintfa (out,
-					"<A HREF=\"ftp://%s\">%s</A>",
-					url, url);
-				g_free (url);
-				c = end;
-                        }
-                }
+static void
+test_linkify_text (void)
+{
+	const gchar * test_type = "linkify_text";
+	gint test_num = 0;
+	const gchar * in;
+	const gchar * expect;
+	gchar * out;
 
-		/* anchor a mailto */
-		else if (!g_strncasecmp(c, "mailto:", 7))
-		{
-			const gchar * end = end_of_url(c);
-			gchar * url = g_strndup (c, end-c);
-			g_string_sprintfa (out,
-				"<a href=\"%s\">%s</a>",
-				url, url);
-			g_free (url);
-			c = end;
-                }
+	in = "Go visit http://pan.rebelbase.com/ right now.";
+	expect = "Go visit <a href=\"http://pan.rebelbase.com/\">http://pan.rebelbase.com/</a> right now.";
+	out = linkify_text (in);
+	regression_test_result_va (!strcmp(out,expect), test_type, ++test_num, "%s", in);
+	g_free (out);
 
-		/* possible mailto 
-		else if (!strncasecmp(c, "@", 1))
-		{
-			const gchar * t = c;
-			const gchar * end = end_of_url (c);
-			gchar * url;
-			while (!badchar(*t) && t!=text) --t;
-			if (*t=='<') ++t;
-			url = g_strndup (t, end-t);
-			if (GNKSA_OK == gnksa_check_address (url))
-			{
-				g_string_sprintfa (out,
-					"<a href=\"mailto:%s\">%s</a>",
-					url, url);
-			}
-			g_free (url);
-			c = end;
-                } */
+	in = "Go visit http://pan.rebelbase.com/. right now.";
+	expect = "Go visit <a href=\"http://pan.rebelbase.com/\">http://pan.rebelbase.com/</a>. right now.";
+	out = linkify_text (in);
+	regression_test_result_va (!strcmp(out,expect), test_type, ++test_num, "%s", in);
+	g_free (out);
 
-		/* keep appending */
-		else
-		{
-			g_string_append_c (out, *c++);
-		}
-        }
-
-	c = out->str;
-	g_string_free (out, FALSE);
-	return (gchar*)c;
+	in = "The url is http://pan.rebelbase.com/; foo";
+	expect = "The url is <a href=\"http://pan.rebelbase.com/\">http://pan.rebelbase.com/</a>; foo";
+	out = linkify_text (in);
+	regression_test_result_va (!strcmp(out,expect), test_type, ++test_num, "%s", in);
+	g_free (out);
 }
-#endif
 
 /***
 ****
@@ -1229,6 +1200,30 @@ rfc822_date_generate (time_t secs)
 	return g_strdup (buf);
 }
 
+static void
+test_date (void)
+{
+	gint test_num = 0;
+	time_t time_1;
+	time_t time_2;
+	gchar * pch_1;
+	gchar * pch_2;
+	const gchar * test_type = "date test";
+
+	time_1 = time(0);
+	pch_1 = rfc822_date_generate (time_1);
+	time_2 = g_mime_utils_header_decode_date (pch_1, NULL);
+	pch_2 = rfc822_date_generate (time_2);
+	regression_test_result_va (time_1==time_2, test_type, ++test_num,
+	                           "time is %lu, generated as `%s', parsed back to %lu",
+	                           time_1, pch_1, time_2);
+	regression_test_result_va (!strcmp(pch_1, pch_2), test_type, ++test_num,
+	                           "dogfood compare: `%s' and `%s'", pch_1, pch_2);
+
+	g_free (pch_1);
+	g_free (pch_2);
+}
+
 
 /***
 ****
@@ -1240,25 +1235,9 @@ rfc822_date_generate (time_t secs)
 void
 util_regression_tests (void)
 {
-	gchar * test_type;
-
-	/* date parsing/generation test... */
-	if (1)
-	{
-		time_t time_1, time_2;
-		gchar *pch_1, *pch_2;
-
-		test_type = "date test";
-		time_1 = time(0);
-		pch_1 = rfc822_date_generate (time_1);
-		time_2 = g_mime_utils_header_decode_date (pch_1, NULL);
-		pch_2 = rfc822_date_generate (time_2);
-		regression_test_result_va (time_1==time_2, test_type, 1, "time is %lu, generated as `%s', parsed back to %lu", time_1, pch_1, time_2);
-		regression_test_result_va (!strcmp(pch_1, pch_2), test_type, 2, "dogfood compare: `%s' and `%s'", pch_1, pch_2);
-
-		g_free (pch_1);
-		g_free (pch_2);
-	}
+	test_date ();
+	test_linkify_text ();
+	test_pan_normalize_filename_inplace ();
 }
 
 static gint regression_test_pass = 0;
@@ -1397,9 +1376,15 @@ mbox_format_message (const gchar * message,
 	GString * tmp;
 	const gchar * march;
 
-	/* sanity checking */
+	/* default author address */
+	if (!is_nonempty_string(author_address)) {
+		pan_warn_if_reached ();
+		author_address = _("unknown@spammesenseless.com");
+	}
+
+	/* sanity clause */
 	g_return_val_if_fail (message!=NULL, NULL);
-	g_return_val_if_fail (is_nonempty_string(author_address), NULL);
+	g_return_val_if_fail (author_address!=NULL, NULL);
 
 	/* add the From_ line.  note ctime() adds the '\n' */
        	tmp = g_string_new (NULL);
@@ -1575,23 +1560,19 @@ get_default_author_from (void)
 }
 
 gchar*
-get_date_display_string (time_t date)
+get_date_display_string (time_t date, const gchar * fmt)
 {
-	gchar * retval;
 	struct tm tm_date;
+	gchar date_buf[256];
 
         if (display_article_dates_in_local_time)
                 localtime_r (&date, &tm_date);
         else
                 gmtime_r (&date, &tm_date);
 
-	retval = g_strdup_printf ("%d.%02d %02d:%02d",
-                tm_date.tm_mon+1,
-                tm_date.tm_mday,
-                tm_date.tm_hour,
-                tm_date.tm_min);
-
-	return retval;
+	strftime (date_buf, sizeof(date_buf), fmt, &tm_date);
+	date_buf[sizeof(date_buf)-1] = '\0';
+	return g_strdup (date_buf);
 }
 
 void
