@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -167,6 +167,10 @@ char* pan_substitute (const char  * original,
                       const char  * search,
                       const char  * replace);
 
+void pan_g_string_append_len (GString       * str,
+                              const gchar   * appendme,
+                              guint           len);
+
 void pan_g_string_replace (GString    * str,
                            const char * search,
                            const char * replace);
@@ -187,18 +191,8 @@ void array_shrink (gpointer    a,
 int pan_strcmp (const char  * a,
                 const char  * b);
 
-int pan_strncmp (const char  * a,
-                 const char  * b,
-                 int len);
-
-
-void pan_g_ptr_array_sort   (GPtrArray   * target,
-                             int  (*compare)(const void *, const void *));
-
-void pan_g_ptr_array_merge_sorted (GPtrArray * target,
-                                   GPtrArray * s1,
-                                   GPtrArray * s2,
-                                   int  (*compare)(const void *, const void *));
+void pan_hash_to_ptr_array  (GHashTable * hash,
+                             GPtrArray  * fillme);
 
 void pan_g_ptr_array_append (GPtrArray   * target,
                              gpointer    * source_ptr,
@@ -266,9 +260,9 @@ gboolean string_ends_with (const gchar* str, const gchar* end);
 
 gchar* pan_stristr (const char * string, const char * pattern);
 
-gchar* pan_normalize_filename (const gchar * filename);
+gchar* pan_normalize_filename_inplace (gchar * filename);
 
-gchar* get_date_display_string (time_t date);
+gchar* get_date_display_string (time_t date, const gchar * strftime_fmt);
 
 #define is_nonempty_string(A) ((A) && *(A)!='\0')
 
